@@ -24,14 +24,17 @@ import swift.common.db
 from swift.account.backend import AccountBroker, DATADIR
 from swift.account.utils import account_listing_response, get_response_headers
 from swift.common.db import DatabaseConnectionError, DatabaseAlreadyExists
-from swift.common.request_helpers import get_param, get_listing_content_type, \
+from swift.common.request_helpers import get_param, \
     split_and_validate_path
 from swift.common.utils import get_logger, hash_path, public, \
     Timestamp, storage_directory, config_true_value, \
-    json, timing_stats, replication, get_log_line
-from swift.common.constraints import check_mount, valid_timestamp, check_utf8
+    json, timing_stats, replication, get_log_line, \
+    config_fallocate_value, fs_has_free_space
+from swift.common.constraints import valid_timestamp, check_utf8, check_drive
 from swift.common import constraints
 from swift.common.db_replicator import ReplicatorRpc
+from swift.common.base_storage_server import BaseStorageServer
+from swift.common.middleware import listing_formats
 from swift.common.swob import HTTPAccepted, HTTPBadRequest, \
     HTTPCreated, HTTPForbidden, HTTPInternalServerError, \
     HTTPMethodNotAllowed, HTTPNoContent, HTTPNotFound, \
@@ -40,18 +43,17 @@ from swift.common.swob import HTTPAccepted, HTTPBadRequest, \
 from swift.common.request_helpers import is_sys_or_user_meta
 
 
-class AccountController(object):
+class AccountController(BaseStorageServer):
     """WSGI controller for the account server."""
 
+    server_type = 'account-server'
+
     def __init__(self, conf, logger=None):
+        super(AccountController, self).__init__(conf)
         self.logger = logger or get_logger(conf, log_route='account-server')
         self.log_requests = config_true_value(conf.get('log_requests', 'true'))
         self.root = conf.get('devices', '/srv/node')
         self.mount_check = config_true_value(conf.get('mount_check', 'true'))
-        replication_server = conf.get('replication_server', None)
-        if replication_server is not None:
-            replication_server = config_true_value(replication_server)
-        self.replication_server = replication_server
         self.replicator_rpc = ReplicatorRpc(self.root, DATADIR, AccountBroker,
                                             self.mount_check,
                                             logger=self.logger)
@@ -59,6 +61,8 @@ class AccountController(object):
             conf.get('auto_create_account_prefix') or '.'
         swift.common.db.DB_PREALLOCATION = \
             config_true_value(conf.get('db_preallocation', 'f'))
+        self.fallocate_reserve, self.fallocate_is_percent = \
+            config_fallocate_value(conf.get('fallocate_reserve', '1%'))
 
     def _get_account_broker(self, drive, part, account, **kwargs):
         hsh = hash_path(account)
@@ -82,12 +86,19 @@ class AccountController(object):
             pass
         return resp(request=req, headers=headers, charset='utf-8', body=body)
 
+    def check_free_space(self, drive):
+        drive_root = os.path.join(self.root, drive)
+        return fs_has_free_space(
+            drive_root, self.fallocate_reserve, self.fallocate_is_percent)
+
     @public
     @timing_stats()
     def DELETE(self, req):
         """Handle HTTP DELETE request."""
         drive, part, account = split_and_validate_path(req, 3)
-        if self.mount_check and not check_mount(self.root, drive):
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
             return HTTPInsufficientStorage(drive=drive, request=req)
         req_timestamp = valid_timestamp(req)
         broker = self._get_account_broker(drive, part, account)
@@ -101,11 +112,15 @@ class AccountController(object):
     def PUT(self, req):
         """Handle HTTP PUT request."""
         drive, part, account, container = split_and_validate_path(req, 3, 4)
-        if self.mount_check and not check_mount(self.root, drive):
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
+            return HTTPInsufficientStorage(drive=drive, request=req)
+        if not self.check_free_space(drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
         if container:   # put account container
             if 'x-timestamp' not in req.headers:
-                timestamp = Timestamp(time.time())
+                timestamp = Timestamp.now()
             else:
                 timestamp = valid_timestamp(req)
             pending_timeout = None
@@ -153,7 +168,7 @@ class AccountController(object):
                     return HTTPConflict(request=req)
             metadata = {}
             metadata.update((key, (value, timestamp.internal))
-                            for key, value in req.headers.iteritems()
+                            for key, value in req.headers.items()
                             if is_sys_or_user_meta('account', key))
             if metadata:
                 broker.update_metadata(metadata, validate_metadata=True)
@@ -167,8 +182,10 @@ class AccountController(object):
     def HEAD(self, req):
         """Handle HTTP HEAD request."""
         drive, part, account = split_and_validate_path(req, 3)
-        out_content_type = get_listing_content_type(req)
-        if self.mount_check and not check_mount(self.root, drive):
+        out_content_type = listing_formats.get_listing_content_type(req)
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
             return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_account_broker(drive, part, account,
                                           pending_timeout=0.1,
@@ -191,6 +208,7 @@ class AccountController(object):
             return HTTPPreconditionFailed(body='Bad delimiter')
         limit = constraints.ACCOUNT_LISTING_LIMIT
         given_limit = get_param(req, 'limit')
+        reverse = config_true_value(get_param(req, 'reverse'))
         if given_limit and given_limit.isdigit():
             limit = int(given_limit)
             if limit > constraints.ACCOUNT_LISTING_LIMIT:
@@ -200,9 +218,11 @@ class AccountController(object):
                     constraints.ACCOUNT_LISTING_LIMIT)
         marker = get_param(req, 'marker', '')
         end_marker = get_param(req, 'end_marker')
-        out_content_type = get_listing_content_type(req)
+        out_content_type = listing_formats.get_listing_content_type(req)
 
-        if self.mount_check and not check_mount(self.root, drive):
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
             return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_account_broker(drive, part, account,
                                           pending_timeout=0.1,
@@ -211,7 +231,7 @@ class AccountController(object):
             return self._deleted_response(broker, req, HTTPNotFound)
         return account_listing_response(account, req, out_content_type, broker,
                                         limit, marker, end_marker, prefix,
-                                        delimiter)
+                                        delimiter, reverse)
 
     @public
     @replication
@@ -223,7 +243,11 @@ class AccountController(object):
         """
         post_args = split_and_validate_path(req, 3)
         drive, partition, hash = post_args
-        if self.mount_check and not check_mount(self.root, drive):
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
+            return HTTPInsufficientStorage(drive=drive, request=req)
+        if not self.check_free_space(drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
         try:
             args = json.load(req.environ['wsgi.input'])
@@ -239,14 +263,18 @@ class AccountController(object):
         """Handle HTTP POST request."""
         drive, part, account = split_and_validate_path(req, 3)
         req_timestamp = valid_timestamp(req)
-        if self.mount_check and not check_mount(self.root, drive):
+        try:
+            check_drive(self.root, drive, self.mount_check)
+        except ValueError:
+            return HTTPInsufficientStorage(drive=drive, request=req)
+        if not self.check_free_space(drive):
             return HTTPInsufficientStorage(drive=drive, request=req)
         broker = self._get_account_broker(drive, part, account)
         if broker.is_deleted():
             return self._deleted_response(broker, req, HTTPNotFound)
         metadata = {}
         metadata.update((key, (value, req_timestamp.internal))
-                        for key, value in req.headers.iteritems()
+                        for key, value in req.headers.items()
                         if is_sys_or_user_meta('account', key))
         if metadata:
             broker.update_metadata(metadata, validate_metadata=True)
@@ -261,17 +289,10 @@ class AccountController(object):
         else:
             try:
                 # disallow methods which are not publicly accessible
-                try:
-                    method = getattr(self, req.method)
-                    getattr(method, 'publicly_accessible')
-                    replication_method = getattr(method, 'replication', False)
-                    if (self.replication_server is not None and
-                            self.replication_server != replication_method):
-                        raise AttributeError('Not allowed method.')
-                except AttributeError:
+                if req.method not in self.allowed_methods:
                     res = HTTPMethodNotAllowed()
                 else:
-                    res = method(req)
+                    res = getattr(self, req.method)(req)
             except HTTPException as error_response:
                 res = error_response
             except (Exception, Timeout):
