@@ -19,10 +19,11 @@ from hashlib import md5
 from six.moves import urllib
 
 from swift.common.utils import hash_path, storage_directory, \
-    Timestamp
+    Timestamp, is_valid_ipv6
 from swift.common.ring import Ring
 from swift.common.request_helpers import is_sys_meta, is_user_meta, \
-    strip_sys_meta_prefix, strip_user_meta_prefix
+    strip_sys_meta_prefix, strip_user_meta_prefix, \
+    is_object_transient_sysmeta, strip_object_transient_sysmeta_prefix
 from swift.account.backend import AccountBroker, DATADIR as ABDATADIR
 from swift.container.backend import ContainerBroker, DATADIR as CBDATADIR
 from swift.obj.diskfile import get_data_dir, read_metadata, DATADIR_BASE, \
@@ -37,6 +38,64 @@ class InfoSystemExit(Exception):
     pass
 
 
+def parse_get_node_args(options, args):
+    """
+    Parse the get_nodes commandline args
+
+    :returns: a tuple, (ring_path, args)
+    """
+    ring_path = None
+
+    if options.policy_name:
+        if POLICIES.get_by_name(options.policy_name) is None:
+            raise InfoSystemExit('No policy named %r' % options.policy_name)
+    elif args and args[0].endswith('.ring.gz'):
+        if os.path.exists(args[0]):
+            ring_path = args.pop(0)
+        else:
+            raise InfoSystemExit('Ring file does not exist')
+
+    if len(args) == 1:
+        args = args[0].strip('/').split('/', 2)
+
+    if not ring_path and not options.policy_name:
+        raise InfoSystemExit('Need to specify policy_name or <ring.gz>')
+
+    if not (args or options.partition):
+        raise InfoSystemExit('No target specified')
+
+    if len(args) > 3:
+        raise InfoSystemExit('Invalid arguments')
+
+    return ring_path, args
+
+
+def curl_head_command(ip, port, device, part, target, policy_index):
+    """
+    Provide a string that is a well formatted curl command to HEAD an object
+    on a storage node.
+
+    :param ip: the ip of the node
+    :param port: the port of the node
+    :param device: the device of the node
+    :param target: the path of the target resource
+    :param policy_index: the policy_index of the target resource (can be None)
+
+    :returns: a string, a well formatted curl command
+    """
+    if is_valid_ipv6(ip):
+        formatted_ip = '[%s]' % ip
+    else:
+        formatted_ip = ip
+
+    cmd = 'curl -g -I -XHEAD "http://%s:%s/%s/%s/%s"' % (
+        formatted_ip, port, device, part, urllib.parse.quote(target))
+    if policy_index is not None:
+        cmd += ' -H "%s: %s"' % ('X-Backend-Storage-Policy-Index',
+                                 policy_index)
+    return cmd
+
+
 def print_ring_locations(ring, datadir, account, container=None, obj=None,
                          tpart=None, all_nodes=False, policy_index=None):
     """
@@ -99,20 +158,12 @@ def print_ring_locations(ring, datadir, account, container=None, obj=None,
     print("\n")
 
     for node in primary_nodes:
-        cmd = 'curl -I -XHEAD "http://%s:%s/%s/%s/%s"' \
-            % (node['ip'], node['port'], node['device'], part,
-               urllib.parse.quote(target))
-        if policy_index is not None:
-            cmd += ' -H "%s: %s"' % ('X-Backend-Storage-Policy-Index',
-                                     policy_index)
+        cmd = curl_head_command(node['ip'], node['port'], node['device'],
+                                part, target, policy_index)
         print(cmd)
     for node in handoff_nodes:
-        cmd = 'curl -I -XHEAD "http://%s:%s/%s/%s/%s"' \
-            % (node['ip'], node['port'], node['device'], part,
-               urllib.parse.quote(target))
-        if policy_index is not None:
-            cmd += ' -H "%s: %s"' % ('X-Backend-Storage-Policy-Index',
-                                     policy_index)
+        cmd = curl_head_command(node['ip'], node['port'], node['device'],
+                                part, target, policy_index)
         cmd += ' # [Handoff]'
         print(cmd)
 
@@ -140,13 +191,17 @@ def print_ring_locations(ring, datadir, account, container=None, obj=None,
           'real value is set in the config file on each storage node.')
 
 
-def print_db_info_metadata(db_type, info, metadata):
+def print_db_info_metadata(db_type, info, metadata, drop_prefixes=False):
     """
     print out data base info/metadata based on its type
 
     :param db_type: database type, account or container
     :param info: dict of data base info
     :param metadata: dict of data base metadata
+    :param drop_prefixes: if True, strip "X-Account-Meta-",
+                          "X-Container-Meta-", "X-Account-Sysmeta-", and
+                          "X-Container-Sysmeta-" when displaying
+                          User Metadata and System Metadata dicts
     """
     if info is None:
         raise ValueError('DB info is None')
@@ -223,9 +278,13 @@ def print_db_info_metadata(db_type, info, metadata):
     sys_metadata = {}
     for key, (value, timestamp) in metadata.items():
         if is_user_meta(db_type, key):
-            user_metadata[strip_user_meta_prefix(db_type, key)] = value
+            if drop_prefixes:
+                key = strip_user_meta_prefix(db_type, key)
+            user_metadata[key] = value
         elif is_sys_meta(db_type, key):
-            sys_metadata[strip_sys_meta_prefix(db_type, key)] = value
+            if drop_prefixes:
+                key = strip_sys_meta_prefix(db_type, key)
+            sys_metadata[key] = value
         else:
             title = key.replace('_', '-').title()
             print('  %s: %s' % (title, value))
@@ -239,8 +298,29 @@ def print_db_info_metadata(db_type, info, metadata):
     else:
         print('No user metadata found in db file')
 
+    if db_type == 'container':
+        print('Sharding Metadata:')
+        shard_type = 'root' if info['is_root'] else 'shard'
+        print('  Type: %s' % shard_type)
+        print('  State: %s' % info['db_state'])
+    if info.get('shard_ranges'):
+        print('Shard Ranges (%d):' % len(info['shard_ranges']))
+        for srange in info['shard_ranges']:
+            srange = dict(srange, state_text=srange.state_text)
+            print('  Name: %(name)s' % srange)
+            print('    lower: %(lower)r, upper: %(upper)r' % srange)
+            print('    Object Count: %(object_count)d, Bytes Used: '
+                  '%(bytes_used)d, State: %(state_text)s (%(state)d)'
+                  % srange)
+            print('    Created at: %s (%s)'
+                  % (Timestamp(srange['timestamp']).isoformat,
+                     srange['timestamp']))
+            print('    Meta Timestamp: %s (%s)'
+                  % (Timestamp(srange['meta_timestamp']).isoformat,
+                     srange['meta_timestamp']))
 
-def print_obj_metadata(metadata):
+
+def print_obj_metadata(metadata, drop_prefixes=False):
     """
     Print out basic info and metadata from object, as returned from
     :func:`swift.obj.diskfile.read_metadata`.
@@ -251,11 +331,16 @@ def print_obj_metadata(metadata):
     Additional metadata is displayed unmodified.
 
     :param metadata: dict of object metadata
+    :param drop_prefixes: if True, strip "X-Object-Meta-", "X-Object-Sysmeta-",
+                          and "X-Object-Transient-Sysmeta-" when displaying
+                          User Metadata, System Metadata, and Transient
+                          System Metadata entries
 
-    :raises: ValueError
+    :raises ValueError:
     """
     user_metadata = {}
     sys_metadata = {}
+    transient_sys_metadata = {}
     other_metadata = {}
 
     if not metadata:
@@ -289,26 +374,36 @@ def print_obj_metadata(metadata):
 
     for key, value in metadata.items():
         if is_user_meta('Object', key):
+            if drop_prefixes:
+                key = strip_user_meta_prefix('Object', key)
             user_metadata[key] = value
         elif is_sys_meta('Object', key):
+            if drop_prefixes:
+                key = strip_sys_meta_prefix('Object', key)
             sys_metadata[key] = value
+        elif is_object_transient_sysmeta(key):
+            if drop_prefixes:
+                key = strip_object_transient_sysmeta_prefix(key)
+            transient_sys_metadata[key] = value
         else:
             other_metadata[key] = value
 
     def print_metadata(title, items):
         print(title)
         if items:
-            for meta_key in sorted(items):
-                print('  %s: %s' % (meta_key, items[meta_key]))
+            for key, value in sorted(items.items()):
+                print('  %s: %s' % (key, value))
         else:
             print('  No metadata found')
 
     print_metadata('System Metadata:', sys_metadata)
+    print_metadata('Transient System Metadata:', transient_sys_metadata)
     print_metadata('User Metadata:', user_metadata)
     print_metadata('Other Metadata:', other_metadata)
 
 
-def print_info(db_type, db_file, swift_dir='/etc/swift', stale_reads_ok=False):
+def print_info(db_type, db_file, swift_dir='/etc/swift', stale_reads_ok=False,
+               drop_prefixes=False):
     if db_type not in ('account', 'container'):
         print("Unrecognized DB type: internal error")
         raise InfoSystemExit()
@@ -332,8 +427,14 @@ def print_info(db_type, db_file, swift_dir='/etc/swift', stale_reads_ok=False):
             raise InfoSystemExit()
         raise
     account = info['account']
-    container = info['container'] if db_type == 'container' else None
-    print_db_info_metadata(db_type, info, broker.metadata)
+    container = None
+    if db_type == 'container':
+        container = info['container']
+        info['is_root'] = broker.is_root_container()
+        sranges = broker.get_shard_ranges()
+        if sranges:
+            info['shard_ranges'] = sranges
+    print_db_info_metadata(db_type, info, broker.metadata, drop_prefixes)
     try:
         ring = Ring(swift_dir, ring_name=db_type)
     except Exception:
@@ -343,7 +444,7 @@ def print_info(db_type, db_file, swift_dir='/etc/swift', stale_reads_ok=False):
 
 
 def print_obj(datafile, check_etag=True, swift_dir='/etc/swift',
-              policy_name=''):
+              policy_name='', drop_prefixes=False):
     """
     Display information about an object read from the datafile.
     Optionally verify the datafile content matches the ETag metadata.
@@ -354,6 +455,10 @@ def print_obj(datafile, check_etag=True, swift_dir='/etc/swift',
                        metadata.
     :param swift_dir: the path on disk to rings
     :param policy_name: optionally the name to use when finding the ring
+    :param drop_prefixes: if True, strip "X-Object-Meta-", "X-Object-Sysmeta-",
+                          and "X-Object-Transient-Sysmeta-" when displaying
+                          User Metadata, System Metadata, and Transient
+                          System Metadata entries
     """
     if not os.path.exists(datafile):
         print("Data file doesn't exist")
@@ -403,7 +508,7 @@ def print_obj(datafile, check_etag=True, swift_dir='/etc/swift',
         etag = metadata.pop('ETag', '')
         length = metadata.pop('Content-Length', '')
         path = metadata.get('name', '')
-        print_obj_metadata(metadata)
+        print_obj_metadata(metadata, drop_prefixes)
 
         # Optional integrity check; it's useful, but slow.
         file_len = None
