@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
@@ -18,18 +17,27 @@
 
 from __future__ import print_function
 
-from eventlet.green import urllib2
-from swift.common.utils import SWIFT_CONF_FILE
+from eventlet.green import socket
+from six import string_types
+from six.moves.urllib.parse import urlparse
+
+from swift.common.utils import (
+    SWIFT_CONF_FILE, md5_hash_for_file, set_swift_dir)
 from swift.common.ring import Ring
-from urlparse import urlparse
-from hashlib import md5
+from swift.common.storage_policy import POLICIES, reload_storage_policies
 import eventlet
 import json
 import optparse
 import time
 import sys
+import six
 import os
 
+if six.PY3:
+    from eventlet.green.urllib import request as urllib2
+else:
+    from eventlet.green import urllib2
+
 
 def seconds2timeunit(seconds):
     elapsed = seconds
@@ -69,16 +77,18 @@ class Scout(object):
 
     def scout_host(self, base_url, recon_type):
         """
-        Perform the actual HTTP request to obtain swift recon telemtry.
+        Perform the actual HTTP request to obtain swift recon telemetry.
 
         :param base_url: the base url of the host you wish to check. str of the
-                        format 'http://127.0.0.1:6000/recon/'
+                        format 'http://127.0.0.1:6200/recon/'
         :param recon_type: the swift recon check to request.
         :returns: tuple of (recon url used, response body, and status)
         """
         url = base_url + recon_type
         try:
             body = urllib2.urlopen(url, timeout=self.timeout).read()
+            if six.PY3 and isinstance(body, six.binary_type):
+                body = body.decode('utf8')
             content = json.loads(body)
             if self.verbose:
                 print("-> %s: %s" % (url, content))
@@ -88,7 +98,7 @@ class Scout(object):
                 print("-> %s: %s" % (url, err))
             content = err
             status = err.code
-        except urllib2.URLError as err:
+        except (urllib2.URLError, socket.timeout) as err:
             if not self.suppress_errors or self.verbose:
                 print("-> %s: %s" % (url, err))
             content = err
@@ -121,7 +131,7 @@ class Scout(object):
             req = urllib2.Request(url)
             req.get_method = lambda: 'OPTIONS'
             conn = urllib2.urlopen(req)
-            header = conn.info().getheader('Server')
+            header = conn.info().get('Server')
             server_header = header.split('/')
             content = server_header[0]
             status = 200
@@ -130,7 +140,7 @@ class Scout(object):
                 print("-> %s: %s" % (url, err))
             content = err
             status = err.code
-        except urllib2.URLError as err:
+        except (urllib2.URLError, socket.timeout) as err:
             if not self.suppress_errors or self.verbose:
                 print("-> %s: %s" % (url, err))
             content = err
@@ -181,40 +191,26 @@ class SwiftRecon(object):
     def _ptime(self, timev=None):
         """
         :param timev: a unix timestamp or None
-        :returns: a pretty string of the current time or provided time
+        :returns: a pretty string of the current time or provided time in UTC
         """
         if timev:
-            return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timev))
+            return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(timev))
         else:
-            return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
+            return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())
 
-    def _md5_file(self, path):
+    def get_hosts(self, region_filter, zone_filter, swift_dir, ring_names):
         """
-        Get the MD5 checksum of a file.
-
-        :param path: path to file
-        :returns: MD5 checksum, hex encoded
-        """
-        md5sum = md5()
-        with open(path, 'rb') as f:
-            block = f.read(4096)
-            while block:
-                md5sum.update(block)
-                block = f.read(4096)
-        return md5sum.hexdigest()
-
-    def get_devices(self, region_filter, zone_filter, swift_dir, ring_name):
-        """
-        Get a list of hosts in the ring
+        Get a list of hosts in the rings.
 
         :param region_filter: Only list regions matching given filter
         :param zone_filter: Only list zones matching given filter
         :param swift_dir: Directory of swift config, usually /etc/swift
-        :param ring_name: Name of the ring, such as 'object'
+        :param ring_names: Collection of ring names, such as
+         ['object', 'object-2']
         :returns: a set of tuples containing the ip and port of hosts
         """
-        ring_data = Ring(swift_dir, ring_name=ring_name)
-        devs = [d for d in ring_data.devs if d]
+        rings = [Ring(swift_dir, ring_name=n) for n in ring_names]
+        devs = [d for r in rings for d in r.devs if d]
         if region_filter is not None:
             devs = [d for d in devs if d['region'] == region_filter]
         if zone_filter is not None:
@@ -232,24 +228,18 @@ class SwiftRecon(object):
         matches = 0
         errors = 0
         ring_names = set()
-        for server_type in ('account', 'container'):
-            ring_name = '%s.ring.gz' % server_type
+        if self.server_type == 'object':
+            for ring_name in os.listdir(swift_dir):
+                if ring_name.startswith('object') and \
+                        ring_name.endswith('.ring.gz'):
+                    ring_names.add(ring_name)
+        else:
+            ring_name = '%s.ring.gz' % self.server_type
             ring_names.add(ring_name)
-        # include any other object ring files
-        for ring_name in os.listdir(swift_dir):
-            if ring_name.startswith('object') and \
-                    ring_name.endswith('ring.gz'):
-                ring_names.add(ring_name)
         rings = {}
         for ring_name in ring_names:
-            md5sum = md5()
-            with open(os.path.join(swift_dir, ring_name), 'rb') as f:
-                block = f.read(4096)
-                while block:
-                    md5sum.update(block)
-                    block = f.read(4096)
-            ring_sum = md5sum.hexdigest()
-            rings[ring_name] = ring_sum
+            rings[ring_name] = md5_hash_for_file(
+                os.path.join(swift_dir, ring_name))
         recon = Scout("ringmd5", self.verbose, self.suppress_errors,
                       self.timeout)
         print("[%s] Checking ring md5sums" % self._ptime())
@@ -264,6 +254,8 @@ class SwiftRecon(object):
             success = True
             for remote_ring_file, remote_ring_sum in response.items():
                 remote_ring_name = os.path.basename(remote_ring_file)
+                if not remote_ring_name.startswith(self.server_type):
+                    continue
                 ring_sum = rings.get(remote_ring_name, None)
                 if remote_ring_sum != ring_sum:
                     success = False
@@ -289,7 +281,7 @@ class SwiftRecon(object):
         """
         matches = 0
         errors = 0
-        conf_sum = self._md5_file(SWIFT_CONF_FILE)
+        conf_sum = md5_hash_for_file(SWIFT_CONF_FILE)
         recon = Scout("swiftconfmd5", self.verbose, self.suppress_errors,
                       self.timeout)
         printfn("[%s] Checking swift.conf md5sum" % self._ptime())
@@ -495,16 +487,14 @@ class SwiftRecon(object):
                 elapsed = time.time() - least_recent_time
                 elapsed, elapsed_unit = seconds2timeunit(elapsed)
                 print('Oldest completion was %s (%d %s ago) by %s.' % (
-                    time.strftime('%Y-%m-%d %H:%M:%S',
-                                  time.gmtime(least_recent_time)),
+                    self._ptime(least_recent_time),
                     elapsed, elapsed_unit, host))
         if most_recent_url is not None:
             host = urlparse(most_recent_url).netloc
             elapsed = time.time() - most_recent_time
             elapsed, elapsed_unit = seconds2timeunit(elapsed)
             print('Most recent completion was %s (%d %s ago) by %s.' % (
-                time.strftime('%Y-%m-%d %H:%M:%S',
-                              time.gmtime(most_recent_time)),
+                self._ptime(most_recent_time),
                 elapsed, elapsed_unit, host))
         print("=" * 79)
 
@@ -879,14 +869,16 @@ class SwiftRecon(object):
                     host = urlparse(url).netloc.split(':')[0]
                     print('%.02f%%  %s' % (used, '%-15s %s' % (host, device)))
 
-    def time_check(self, hosts):
+    def time_check(self, hosts, jitter=0.0):
         """
         Check a time synchronization of hosts with current time
 
         :param hosts: set of hosts to check. in the format of:
             set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+        :param jitter: Maximal allowed time jitter
         """
 
+        jitter = abs(jitter)
         matches = 0
         errors = 0
         recon = Scout("time", self.verbose, self.suppress_errors,
@@ -897,17 +889,13 @@ class SwiftRecon(object):
             if status != 200:
                 errors = errors + 1
                 continue
-            if (ts_remote < ts_start or ts_remote > ts_end):
+            if (ts_remote + jitter < ts_start or ts_remote - jitter > ts_end):
                 diff = abs(ts_end - ts_remote)
-                ts_end_f = time.strftime(
-                    "%Y-%m-%d %H:%M:%S",
-                    time.localtime(ts_end))
-                ts_remote_f = time.strftime(
-                    "%Y-%m-%d %H:%M:%S",
-                    time.localtime(ts_remote))
+                ts_end_f = self._ptime(ts_end)
+                ts_remote_f = self._ptime(ts_remote)
 
                 print("!! %s current time is %s, but remote is %s, "
-                      "differs by %.2f sec" % (
+                      "differs by %.4f sec" % (
                           url,
                           ts_end_f,
                           ts_remote_f,
@@ -920,13 +908,71 @@ class SwiftRecon(object):
             matches, len(hosts), errors))
         print("=" * 79)
 
+    def version_check(self, hosts):
+        """
+        Check OS Swift version of hosts. Inform if differs.
+
+        :param hosts: set of hosts to check. in the format of:
+            set([('127.0.0.1', 6020), ('127.0.0.2', 6030)])
+        """
+        versions = set()
+        errors = 0
+        print("[%s] Checking versions" % self._ptime())
+        recon = Scout("version", self.verbose, self.suppress_errors,
+                      self.timeout)
+        for url, response, status, ts_start, ts_end in self.pool.imap(
+                recon.scout, hosts):
+            if status != 200:
+                errors = errors + 1
+                continue
+            versions.add(response['version'])
+            if self.verbose:
+                print("-> %s installed version %s" % (
+                    url, response['version']))
+
+        if not len(versions):
+            print("No hosts returned valid data.")
+        elif len(versions) == 1:
+            print("Versions matched (%s), "
+                  "%s error[s] while checking hosts." % (
+                      versions.pop(), errors))
+        else:
+            print("Versions not matched (%s), "
+                  "%s error[s] while checking hosts." % (
+                      ", ".join(sorted(versions)), errors))
+
+        print("=" * 79)
+
+    def _get_ring_names(self, policy=None):
+        """
+        Retrieve name of ring files.
+
+        If no policy is passed and the server type is object,
+        the ring names of all storage-policies are retrieved.
+
+        :param policy: name or index of storage policy, only applicable
+         with server_type==object.
+         :returns: list of ring names.
+        """
+        if self.server_type == 'object':
+            ring_names = [p.ring_name for p in POLICIES if (
+                p.name == policy or not policy or (
+                    policy.isdigit() and int(policy) == int(p) or
+                    (isinstance(policy, string_types)
+                     and policy in p.aliases)))]
+        else:
+            ring_names = [self.server_type]
+
+        return ring_names
+
     def main(self):
         """
         Retrieve and report cluster info from hosts running recon middleware.
         """
         print("=" * 79)
         usage = '''
-        usage: %prog <server_type> [-v] [--suppress] [-a] [-r] [-u] [-d]
+        usage: %prog <server_type> [<server_type> [<server_type>]]
+        [-v] [--suppress] [-a] [-r] [-u] [-d]
         [-l] [-T] [--md5] [--auditor] [--updater] [--expirer] [--sockstat]
         [--human-readable]
 
@@ -971,6 +1017,10 @@ class SwiftRecon(object):
                         help="Get drive audit error stats")
         args.add_option('--time', '-T', action="store_true",
                         help="Check time synchronization")
+        args.add_option('--jitter', type="float", default=0.0,
+                        help="Maximal allowed time jitter")
+        args.add_option('--swift-versions', action="store_true",
+                        help="Check swift versions")
         args.add_option('--top', type='int', metavar='COUNT', default=0,
                         help='Also show the top COUNT entries in rank order.')
         args.add_option('--lowest', type='int', metavar='COUNT', default=0,
@@ -978,7 +1028,8 @@ class SwiftRecon(object):
                         order.')
         args.add_option('--all', action="store_true",
                         help="Perform all checks. Equal to \t\t\t-arudlqT "
-                        "--md5 --sockstat --auditor --updater --expirer")
+                        "--md5 --sockstat --auditor --updater --expirer "
+                        "--driveaudit --validate-servers --swift-versions")
         args.add_option('--region', type="int",
                         help="Only query servers in specified region")
         args.add_option('--zone', '-z', type="int",
@@ -988,100 +1039,121 @@ class SwiftRecon(object):
                         default=5)
         args.add_option('--swiftdir', default="/etc/swift",
                         help="Default = /etc/swift")
+        args.add_option('--policy', '-p',
+                        help='Only query object servers in specified '
+                        'storage policy (specified as name or index).')
         options, arguments = args.parse_args()
 
-        if len(sys.argv) <= 1 or len(arguments) > 1:
+        if len(sys.argv) <= 1 or len(arguments) > len(self.check_types):
             args.print_help()
             sys.exit(0)
 
         if arguments:
-            if arguments[0] in self.check_types:
-                self.server_type = arguments[0]
+            arguments = set(arguments)
+            if arguments.issubset(self.check_types):
+                server_types = arguments
             else:
                 print("Invalid Server Type")
                 args.print_help()
                 sys.exit(1)
-        else:
-            self.server_type = 'object'
+        else:  # default
+            server_types = ['object']
 
         swift_dir = options.swiftdir
+        if set_swift_dir(swift_dir):
+            reload_storage_policies()
+
         self.verbose = options.verbose
         self.suppress_errors = options.suppress
         self.timeout = options.timeout
 
-        hosts = self.get_devices(options.region, options.zone,
-                                 swift_dir, self.server_type)
-
-        print("--> Starting reconnaissance on %s hosts" % len(hosts))
-        print("=" * 79)
-
-        if options.all:
-            if self.server_type == 'object':
-                self.async_check(hosts)
-                self.replication_check(hosts)
-                self.object_auditor_check(hosts)
-                self.updater_check(hosts)
-                self.expirer_check(hosts)
-            elif self.server_type == 'container':
-                self.replication_check(hosts)
-                self.auditor_check(hosts)
-                self.updater_check(hosts)
-            elif self.server_type == 'account':
-                self.replication_check(hosts)
-                self.auditor_check(hosts)
-            self.umount_check(hosts)
-            self.load_check(hosts)
-            self.disk_usage(hosts, options.top, options.lowest,
-                            options.human_readable)
-            self.get_ringmd5(hosts, swift_dir)
-            self.quarantine_check(hosts)
-            self.socket_usage(hosts)
-            self.server_type_check(hosts)
-            self.driveaudit_check(hosts)
-            self.time_check(hosts)
-        else:
-            if options.async:
+        for server_type in server_types:
+            self.server_type = server_type
+            ring_names = self._get_ring_names(options.policy)
+            if not ring_names:
+                print('Invalid Storage Policy: %s' % options.policy)
+                args.print_help()
+                sys.exit(0)
+            hosts = self.get_hosts(options.region, options.zone,
+                                   swift_dir, ring_names)
+            print("--> Starting reconnaissance on %s hosts (%s)" %
+                  (len(hosts), self.server_type))
+            print("=" * 79)
+            if options.all:
                 if self.server_type == 'object':
                     self.async_check(hosts)
-                else:
-                    print("Error: Can't check asyncs on non object servers.")
-            if options.unmounted:
-                self.umount_check(hosts)
-            if options.replication:
-                self.replication_check(hosts)
-            if options.auditor:
-                if self.server_type == 'object':
                     self.object_auditor_check(hosts)
-                else:
-                    self.auditor_check(hosts)
-            if options.updater:
-                if self.server_type == 'account':
-                    print("Error: Can't check updaters on account servers.")
-                else:
                     self.updater_check(hosts)
-            if options.expirer:
-                if self.server_type == 'object':
                     self.expirer_check(hosts)
-                else:
-                    print("Error: Can't check expired on non object servers.")
-            if options.validate_servers:
-                self.server_type_check(hosts)
-            if options.loadstats:
+                elif self.server_type == 'container':
+                    self.auditor_check(hosts)
+                    self.updater_check(hosts)
+                elif self.server_type == 'account':
+                    self.auditor_check(hosts)
+                self.replication_check(hosts)
+                self.umount_check(hosts)
                 self.load_check(hosts)
-            if options.diskusage:
                 self.disk_usage(hosts, options.top, options.lowest,
                                 options.human_readable)
-            if options.md5:
                 self.get_ringmd5(hosts, swift_dir)
                 self.get_swiftconfmd5(hosts)
-            if options.quarantined:
                 self.quarantine_check(hosts)
-            if options.sockstat:
                 self.socket_usage(hosts)
-            if options.driveaudit:
+                self.server_type_check(hosts)
                 self.driveaudit_check(hosts)
-            if options.time:
-                self.time_check(hosts)
+                self.time_check(hosts, options.jitter)
+                self.version_check(hosts)
+            else:
+                if options.async:
+                    if self.server_type == 'object':
+                        self.async_check(hosts)
+                    else:
+                        print("Error: Can't check asyncs on non object "
+                              "servers.")
+                        print("=" * 79)
+                if options.unmounted:
+                    self.umount_check(hosts)
+                if options.replication:
+                    self.replication_check(hosts)
+                if options.auditor:
+                    if self.server_type == 'object':
+                        self.object_auditor_check(hosts)
+                    else:
+                        self.auditor_check(hosts)
+                if options.updater:
+                    if self.server_type == 'account':
+                        print("Error: Can't check updaters on account "
+                              "servers.")
+                        print("=" * 79)
+                    else:
+                        self.updater_check(hosts)
+                if options.expirer:
+                    if self.server_type == 'object':
+                        self.expirer_check(hosts)
+                    else:
+                        print("Error: Can't check expired on non object "
+                              "servers.")
+                        print("=" * 79)
+                if options.validate_servers:
+                    self.server_type_check(hosts)
+                if options.loadstats:
+                    self.load_check(hosts)
+                if options.diskusage:
+                    self.disk_usage(hosts, options.top, options.lowest,
+                                    options.human_readable)
+                if options.md5:
+                    self.get_ringmd5(hosts, swift_dir)
+                    self.get_swiftconfmd5(hosts)
+                if options.quarantined:
+                    self.quarantine_check(hosts)
+                if options.sockstat:
+                    self.socket_usage(hosts)
+                if options.driveaudit:
+                    self.driveaudit_check(hosts)
+                if options.time:
+                    self.time_check(hosts, options.jitter)
+                if options.swift_versions:
+                    self.version_check(hosts)
 
 
 def main():
@@ -1090,7 +1162,3 @@ def main():
         reconnoiter.main()
     except KeyboardInterrupt:
         print('\n')
-
-
-if __name__ == '__main__':
-    main()
