@@ -13,16 +13,18 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import functools
 import os
-import urllib
-import time
-from urllib import unquote
-from ConfigParser import ConfigParser, NoSectionError, NoOptionError
+from os.path import isdir  # tighter scoped import for mocking
+
+import six
+from six.moves.configparser import ConfigParser, NoSectionError, NoOptionError
+from six.moves import urllib
 
 from swift.common import utils, exceptions
 from swift.common.swob import HTTPBadRequest, HTTPLengthRequired, \
     HTTPRequestEntityTooLarge, HTTPPreconditionFailed, HTTPNotImplemented, \
-    HTTPException
+    HTTPException, wsgi_to_str, wsgi_to_bytes
 
 MAX_FILE_SIZE = 5368709122
 MAX_META_NAME_LENGTH = 128
@@ -35,6 +37,8 @@ CONTAINER_LISTING_LIMIT = 10000
 ACCOUNT_LISTING_LIMIT = 10000
 MAX_ACCOUNT_NAME_LENGTH = 256
 MAX_CONTAINER_NAME_LENGTH = 256
+VALID_API_VERSIONS = ["v1", "v1.0"]
+EXTRA_HEADER_COUNT = 0
 
 # If adding an entry to DEFAULT_CONSTRAINTS, note that
 # these constraints are automatically published by the
@@ -52,6 +56,8 @@ DEFAULT_CONSTRAINTS = {
     'account_listing_limit': ACCOUNT_LISTING_LIMIT,
     'max_account_name_length': MAX_ACCOUNT_NAME_LENGTH,
     'max_container_name_length': MAX_CONTAINER_NAME_LENGTH,
+    'valid_api_versions': VALID_API_VERSIONS,
+    'extra_header_count': EXTRA_HEADER_COUNT,
 }
 
 SWIFT_CONSTRAINTS_LOADED = False
@@ -61,7 +67,7 @@ EFFECTIVE_CONSTRAINTS = {}  # populated by reload_constraints
 
 def reload_constraints():
     """
-    Parse SWIFT_CONF_FILE and reset module level global contraint attrs,
+    Parse SWIFT_CONF_FILE and reset module level global constraint attrs,
     populating OVERRIDE_CONSTRAINTS AND EFFECTIVE_CONSTRAINTS along the way.
     """
     global SWIFT_CONSTRAINTS_LOADED, OVERRIDE_CONSTRAINTS
@@ -72,13 +78,17 @@ def reload_constraints():
         SWIFT_CONSTRAINTS_LOADED = True
         for name in DEFAULT_CONSTRAINTS:
             try:
-                value = int(constraints_conf.get('swift-constraints', name))
+                value = constraints_conf.get('swift-constraints', name)
             except NoOptionError:
                 pass
             except NoSectionError:
                 # We are never going to find the section for another option
                 break
             else:
+                try:
+                    value = int(value)
+                except ValueError:
+                    value = utils.list_from_csv(value)
                 OVERRIDE_CONSTRAINTS[name] = value
     for name, default in DEFAULT_CONSTRAINTS.items():
         value = OVERRIDE_CONSTRAINTS.get(name, default)
@@ -94,9 +104,12 @@ reload_constraints()
 MAX_BUFFERED_SLO_SEGMENTS = 10000
 
 
-#: Query string format= values to their corresponding content-type values
-FORMAT2CONTENT_TYPE = {'plain': 'text/plain', 'json': 'application/json',
-                       'xml': 'application/xml'}
+# By default the maximum number of allowed headers depends on the number of max
+# allowed metadata settings plus a default value of 36 for swift internally
+# generated headers and regular http headers.  If for some reason this is not
+# enough (custom middleware for example) it can be increased with the
+# extra_header_count constraint.
+MAX_HEADER_COUNT = MAX_META_COUNT + 36 + max(EXTRA_HEADER_COUNT, 0)
 
 
 def check_metadata(req, target_type):
@@ -111,38 +124,47 @@ def check_metadata(req, target_type):
                         which type the target storage for the metadata is
     :returns: HTTPBadRequest with bad metadata otherwise None
     """
-    prefix = 'x-%s-meta-' % target_type.lower()
+    target_type = target_type.lower()
+    prefix = 'x-%s-meta-' % target_type
     meta_count = 0
     meta_size = 0
-    for key, value in req.headers.iteritems():
-        if isinstance(value, basestring) and len(value) > MAX_HEADER_SIZE:
-            return HTTPBadRequest(body='Header value too long: %s' %
-                                  key[:MAX_META_NAME_LENGTH],
+    for key, value in req.headers.items():
+        if (isinstance(value, six.string_types)
+           and len(value) > MAX_HEADER_SIZE):
+
+            return HTTPBadRequest(body=b'Header value too long: %s' %
+                                  wsgi_to_bytes(key[:MAX_META_NAME_LENGTH]),
                                   request=req, content_type='text/plain')
         if not key.lower().startswith(prefix):
             continue
         key = key[len(prefix):]
         if not key:
-            return HTTPBadRequest(body='Metadata name cannot be empty',
+            return HTTPBadRequest(body=b'Metadata name cannot be empty',
+                                  request=req, content_type='text/plain')
+        bad_key = not check_utf8(wsgi_to_str(key))
+        bad_value = value and not check_utf8(wsgi_to_str(value))
+        if target_type in ('account', 'container') and (bad_key or bad_value):
+            return HTTPBadRequest(body=b'Metadata must be valid UTF-8',
                                   request=req, content_type='text/plain')
         meta_count += 1
         meta_size += len(key) + len(value)
         if len(key) > MAX_META_NAME_LENGTH:
             return HTTPBadRequest(
-                body='Metadata name too long: %s%s' % (prefix, key),
+                body=wsgi_to_bytes('Metadata name too long: %s%s' % (
+                    prefix, key)),
                 request=req, content_type='text/plain')
-        elif len(value) > MAX_META_VALUE_LENGTH:
+        if len(value) > MAX_META_VALUE_LENGTH:
             return HTTPBadRequest(
-                body='Metadata value longer than %d: %s%s' % (
-                    MAX_META_VALUE_LENGTH, prefix, key),
+                body=wsgi_to_bytes('Metadata value longer than %d: %s%s' % (
+                    MAX_META_VALUE_LENGTH, prefix, key)),
                 request=req, content_type='text/plain')
-        elif meta_count > MAX_META_COUNT:
+        if meta_count > MAX_META_COUNT:
             return HTTPBadRequest(
-                body='Too many metadata items; max %d' % MAX_META_COUNT,
+                body=b'Too many metadata items; max %d' % MAX_META_COUNT,
                 request=req, content_type='text/plain')
-        elif meta_size > MAX_META_OVERALL_SIZE:
+        if meta_size > MAX_META_OVERALL_SIZE:
             return HTTPBadRequest(
-                body='Total metadata too large; max %d'
+                body=b'Total metadata too large; max %d'
                 % MAX_META_OVERALL_SIZE,
                 request=req, content_type='text/plain')
     return None
@@ -154,43 +176,39 @@ def check_object_creation(req, object_name):
 
     :param req: HTTP request object
     :param object_name: name of object to be created
-    :returns HTTPRequestEntityTooLarge: the object is too large
-    :returns HTTPLengthRequired: missing content-length header and not
-                                 a chunked request
-    :returns HTTPBadRequest: missing or bad content-type header, or
-                             bad metadata
-    :returns HTTPNotImplemented: unsupported transfer-encoding header value
+    :returns: HTTPRequestEntityTooLarge -- the object is too large
+    :returns: HTTPLengthRequired -- missing content-length header and not
+                                    a chunked request
+    :returns: HTTPBadRequest -- missing or bad content-type header, or
+                                bad metadata
+    :returns: HTTPNotImplemented -- unsupported transfer-encoding header value
     """
     try:
         ml = req.message_length()
     except ValueError as e:
         return HTTPBadRequest(request=req, content_type='text/plain',
-                              body=str(e))
+                              body=str(e).encode('ascii'))
     except AttributeError as e:
         return HTTPNotImplemented(request=req, content_type='text/plain',
-                                  body=str(e))
+                                  body=str(e).encode('ascii'))
     if ml is not None and ml > MAX_FILE_SIZE:
-        return HTTPRequestEntityTooLarge(body='Your request is too large.',
+        return HTTPRequestEntityTooLarge(body=b'Your request is too large.',
                                          request=req,
                                          content_type='text/plain')
     if req.content_length is None and \
             req.headers.get('transfer-encoding') != 'chunked':
-        return HTTPLengthRequired(body='Missing Content-Length header.',
+        return HTTPLengthRequired(body=b'Missing Content-Length header.',
                                   request=req,
                                   content_type='text/plain')
 
-    if 'X-Copy-From' in req.headers and req.content_length:
-        return HTTPBadRequest(body='Copy requests require a zero byte body',
-                              request=req, content_type='text/plain')
-
     if len(object_name) > MAX_OBJECT_NAME_LENGTH:
-        return HTTPBadRequest(body='Object name length of %d longer than %d' %
+        return HTTPBadRequest(body=b'Object name length of %d longer than %d' %
                               (len(object_name), MAX_OBJECT_NAME_LENGTH),
                               request=req, content_type='text/plain')
 
     if 'Content-Type' not in req.headers:
         return HTTPBadRequest(request=req, content_type='text/plain',
-                              body='No content type')
+                              body=b'No content type')
 
     try:
         req = check_delete_headers(req)
@@ -198,12 +216,26 @@ def check_object_creation(req, object_name):
         return HTTPBadRequest(request=req, body=e.body,
                               content_type='text/plain')
 
-    if not check_utf8(req.headers['Content-Type']):
-        return HTTPBadRequest(request=req, body='Invalid Content-Type',
+    if not check_utf8(wsgi_to_str(req.headers['Content-Type'])):
+        return HTTPBadRequest(request=req, body=b'Invalid Content-Type',
                               content_type='text/plain')
     return check_metadata(req, 'object')
 
 
+def check_dir(root, drive):
+    """
+    Verify that the path to the device is a directory and is a lesser
+    constraint that is enforced when a full mount_check isn't possible
+    with, for instance, a VM using loopback or partitions.
+
+    :param root:  base path where the dir is
+    :param drive: drive name to be checked
+    :returns: full path to the device
+    :raises ValueError: if drive fails to validate
+    """
+    return check_drive(root, drive, False)
+
+
 def check_mount(root, drive):
     """
     Verify that the path to the device is a mount point and mounted.  This
@@ -213,12 +245,33 @@ def check_mount(root, drive):
 
     :param root:  base path where the devices are mounted
     :param drive: drive name to be checked
-    :returns: True if it is a valid mounted device, False otherwise
+    :returns: full path to the device
+    :raises ValueError: if drive fails to validate
     """
-    if not (urllib.quote_plus(drive) == drive):
-        return False
+    return check_drive(root, drive, True)
+
+
+def check_drive(root, drive, mount_check):
+    """
+    Validate the path given by root and drive is a valid existing directory.
+
+    :param root:  base path where the devices are mounted
+    :param drive: drive name to be checked
+    :param mount_check: additionally require path is mounted
+
+    :returns: full path to the device
+    :raises ValueError: if drive fails to validate
+    """
+    if not (urllib.parse.quote_plus(drive) == drive):
+        raise ValueError('%s is not a valid drive name' % drive)
     path = os.path.join(root, drive)
-    return utils.ismount(path)
+    if mount_check:
+        if not utils.ismount(path):
+            raise ValueError('%s is not mounted' % path)
+    else:
+        if not isdir(path):
+            raise ValueError('%s is not a directory' % path)
+    return path
 
 
 def check_float(string):
@@ -242,40 +295,45 @@ def valid_timestamp(request):
     :param request: the swob request object
 
     :returns: a valid Timestamp instance
-    :raises: HTTPBadRequest on missing or invalid X-Timestamp
+    :raises HTTPBadRequest: on missing or invalid X-Timestamp
     """
     try:
         return request.timestamp
     except exceptions.InvalidTimestamp as e:
-        raise HTTPBadRequest(body=str(e), request=request,
+        raise HTTPBadRequest(body=str(e).encode('ascii'), request=request,
                              content_type='text/plain')
 
 
 def check_delete_headers(request):
     """
-    Validate if 'x-delete' headers are have correct values
-    values should be positive integers and correspond to
-    a time in the future.
+    Check that 'x-delete-after' and 'x-delete-at' headers have valid values.
+    Values should be positive integers and correspond to a time greater than
+    the request timestamp.
+
+    If the 'x-delete-after' header is found then its value is used to compute
+    an 'x-delete-at' value which takes precedence over any existing
+    'x-delete-at' header.
 
     :param request: the swob request object
-
-    :returns: HTTPBadRequest in case of invalid values
-              or None if values are ok
+    :raises: HTTPBadRequest in case of invalid values
+    :returns: the swob request object
     """
+    now = float(valid_timestamp(request))
     if 'x-delete-after' in request.headers:
         try:
             x_delete_after = int(request.headers['x-delete-after'])
         except ValueError:
             raise HTTPBadRequest(request=request,
                                  content_type='text/plain',
-                                 body='Non-integer X-Delete-After')
-        actual_del_time = time.time() + x_delete_after
-        if actual_del_time < time.time():
+                                 body=b'Non-integer X-Delete-After')
+        actual_del_time = utils.normalize_delete_at_timestamp(
+            now + x_delete_after)
+        if int(actual_del_time) <= now:
             raise HTTPBadRequest(request=request,
                                  content_type='text/plain',
-                                 body='X-Delete-After in past')
-        request.headers['x-delete-at'] = utils.normalize_delete_at_timestamp(
-            actual_del_time)
+                                 body=b'X-Delete-After in past')
+        request.headers['x-delete-at'] = actual_del_time
+        del request.headers['x-delete-after']
 
     if 'x-delete-at' in request.headers:
         try:
@@ -283,11 +341,12 @@ def check_delete_headers(request):
                 int(request.headers['x-delete-at'])))
         except ValueError:
             raise HTTPBadRequest(request=request, content_type='text/plain',
-                                 body='Non-integer X-Delete-At')
+                                 body=b'Non-integer X-Delete-At')
 
-        if x_delete_at < time.time():
+        if x_delete_at <= now and not utils.config_true_value(
+                request.headers.get('x-backend-replication', 'f')):
             raise HTTPBadRequest(request=request, content_type='text/plain',
-                                 body='X-Delete-At in past')
+                                 body=b'X-Delete-At in past')
     return request
 
 
@@ -303,89 +362,73 @@ def check_utf8(string):
     if not string:
         return False
     try:
-        if isinstance(string, unicode):
-            string.encode('utf-8')
+        if isinstance(string, six.text_type):
+            encoded = string.encode('utf-8')
+            decoded = string
         else:
-            string.decode('UTF-8')
-        return '\x00' not in string
+            encoded = string
+            decoded = string.decode('UTF-8')
+            if decoded.encode('UTF-8') != encoded:
+                return False
+        # A UTF-8 string with surrogates in it is invalid.
+        #
+        # Note: this check is only useful on Python 2. On Python 3, a
+        # bytestring with a UTF-8-encoded surrogate codepoint is (correctly)
+        # treated as invalid, so the decode() call above will fail.
+        #
+        # Note 2: this check requires us to use a wide build of Python 2. On
+        # narrow builds of Python 2, potato = u"\U0001F954" will have length
+        # 2, potato[0] == u"\ud83e" (surrogate), and potato[1] == u"\udda0"
+        # (also a surrogate), so even if it is correctly UTF-8 encoded as
+        # b'\xf0\x9f\xa6\xa0', it will not pass this check. Fortunately,
+        # most Linux distributions build Python 2 wide, and Python 3.3+
+        # removed the wide/narrow distinction entirely.
+        if any(0xD800 <= ord(codepoint) <= 0xDFFF
+               for codepoint in decoded):
+            return False
+        return b'\x00' not in encoded
     # If string is unicode, decode() will raise UnicodeEncodeError
     # So, we should catch both UnicodeDecodeError & UnicodeEncodeError
     except UnicodeError:
         return False
 
 
-def check_path_header(req, name, length, error_msg):
+def check_name_format(req, name, target_type):
     """
-    Validate that the value of path-like header is
-    well formatted. We assume the caller ensures that
-    specific header is present in req.headers.
+    Validate that the header contains valid account or container name.
 
     :param req: HTTP request object
-    :param name: header name
-    :param length: length of path segment check
-    :param error_msg: error message for client
-    :returns: A tuple with path parts according to length
-    :raise: HTTPPreconditionFailed if header value
+    :param name: header value to validate
+    :param target_type: which header is being validated (Account or Container)
+    :returns: A properly encoded account name or container name
+    :raise HTTPPreconditionFailed: if account header
             is not well formatted.
     """
-    src_header = unquote(req.headers.get(name))
-    if not src_header.startswith('/'):
-        src_header = '/' + src_header
-    try:
-        return utils.split_path(src_header, length, length, True)
-    except ValueError:
+    if not name:
         raise HTTPPreconditionFailed(
             request=req,
-            body=error_msg)
-
-
-def check_copy_from_header(req):
-    """
-    Validate that the value from x-copy-from header is
-    well formatted. We assume the caller ensures that
-    x-copy-from header is present in req.headers.
-
-    :param req: HTTP request object
-    :returns: A tuple with container name and object name
-    :raise: HTTPPreconditionFailed if x-copy-from value
-            is not well formatted.
-    """
-    return check_path_header(req, 'X-Copy-From', 2,
-                             'X-Copy-From header must be of the form '
-                             '<container name>/<object name>')
-
-
-def check_destination_header(req):
-    """
-    Validate that the value from destination header is
-    well formatted. We assume the caller ensures that
-    destination header is present in req.headers.
-
-    :param req: HTTP request object
-    :returns: A tuple with container name and object name
-    :raise: HTTPPreconditionFailed if destination value
-            is not well formatted.
-    """
-    return check_path_header(req, 'Destination', 2,
-                             'Destination header must be of the form '
-                             '<container name>/<object name>')
-
-
-def check_account_format(req, account):
-    """
-    Validate that the header contains valid account name.
-    We assume the caller ensures that
-    destination header is present in req.headers.
-
-    :param req: HTTP request object
-    :returns: A properly encoded account name
-    :raise: HTTPPreconditionFailed if account header
-            is not well formatted.
-    """
-    if isinstance(account, unicode):
-        account = account.encode('utf-8')
-    if '/' in account:
+            body=b'%s name cannot be empty' % target_type)
+    if isinstance(name, six.text_type):
+        name = name.encode('utf-8')
+    if b'/' in name:
         raise HTTPPreconditionFailed(
             request=req,
-            body='Account name cannot contain slashes')
-    return account
+            body=b'%s name cannot contain slashes' % target_type)
+    return name
+
+check_account_format = functools.partial(check_name_format,
+                                         target_type=b'Account')
+check_container_format = functools.partial(check_name_format,
+                                           target_type=b'Container')
+
+
+def valid_api_version(version):
+    """
+    Checks if the requested version is valid.
+
+    Currently Swift only supports "v1" and "v1.0".
+    """
+    global VALID_API_VERSIONS
+    if not isinstance(VALID_API_VERSIONS, list):
+        VALID_API_VERSIONS = [str(VALID_API_VERSIONS)]
+    return version in VALID_API_VERSIONS
