@@ -18,11 +18,13 @@ Internal client library for making calls directly to the servers rather than
 through the proxy.
 """
 
+import json
 import os
 import socket
-from time import time
 
 from eventlet import sleep, Timeout
+import six
+import six.moves.cPickle as pickle
 from six.moves.http_client import HTTPException
 
 from swift.common.bufferedhttp import http_connect
@@ -30,55 +32,136 @@ from swift.common.exceptions import ClientException
 from swift.common.utils import Timestamp, FileLikeIter
 from swift.common.http import HTTP_NO_CONTENT, HTTP_INSUFFICIENT_STORAGE, \
     is_success, is_server_error
-from swift.common.swob import HeaderKeyDict
+from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.utils import quote
 
-try:
-    import simplejson as json
-except ImportError:
-    import json
-
 
 class DirectClientException(ClientException):
 
-    def __init__(self, stype, method, node, part, path, resp):
+    def __init__(self, stype, method, node, part, path, resp, host=None):
+        # host can be used to override the node ip and port reported in
+        # the exception
+        host = host if host is not None else node
+        if not isinstance(path, six.text_type):
+            path = path.decode("utf-8")
         full_path = quote('/%s/%s%s' % (node['device'], part, path))
         msg = '%s server %s:%s direct %s %r gave status %s' % (
-            stype, node['ip'], node['port'], method, full_path, resp.status)
+            stype, host['ip'], host['port'], method, full_path, resp.status)
         headers = HeaderKeyDict(resp.getheaders())
         super(DirectClientException, self).__init__(
-            msg, http_host=node['ip'], http_port=node['port'],
+            msg, http_host=host['ip'], http_port=host['port'],
             http_device=node['device'], http_status=resp.status,
             http_reason=resp.reason, http_headers=headers)
 
 
+def _make_req(node, part, method, path, headers, stype,
+              conn_timeout=5, response_timeout=15, send_timeout=15,
+              contents=None, content_length=None, chunk_size=65535):
+    """
+    Make request to backend storage node.
+    (i.e. 'Account', 'Container', 'Object')
+    :param node: a node dict from a ring
+    :param part: an integer, the partition number
+    :param method: a string, the HTTP method (e.g. 'PUT', 'DELETE', etc)
+    :param path: a string, the request path
+    :param headers: a dict, header name => value
+    :param stype: a string, describing the type of service
+    :param conn_timeout: timeout while waiting for connection; default is 5
+        seconds
+    :param response_timeout: timeout while waiting for response; default is 15
+        seconds
+    :param send_timeout: timeout for sending request body; default is 15
+        seconds
+    :param contents: an iterable or string to read object data from
+    :param content_length: value to send as content-length header
+    :param chunk_size: if defined, chunk size of data to send
+    :returns: an HTTPResponse object
+    :raises DirectClientException: if the response status is not 2xx
+    :raises eventlet.Timeout: if either conn_timeout or response_timeout is
+        exceeded
+    """
+    if contents is not None:
+        if content_length is not None:
+            headers['Content-Length'] = str(content_length)
+        else:
+            for n, v in headers.items():
+                if n.lower() == 'content-length':
+                    content_length = int(v)
+        if not contents:
+            headers['Content-Length'] = '0'
+        if isinstance(contents, six.string_types):
+            contents = [contents]
+        if content_length is None:
+            headers['Transfer-Encoding'] = 'chunked'
+
+    with Timeout(conn_timeout):
+        conn = http_connect(node['ip'], node['port'], node['device'], part,
+                            method, path, headers=headers)
+
+    if contents is not None:
+        contents_f = FileLikeIter(contents)
+
+        with Timeout(send_timeout):
+            if content_length is None:
+                chunk = contents_f.read(chunk_size)
+                while chunk:
+                    conn.send('%x\r\n%s\r\n' % (len(chunk), chunk))
+                    chunk = contents_f.read(chunk_size)
+                conn.send('0\r\n\r\n')
+            else:
+                left = content_length
+                while left > 0:
+                    size = chunk_size
+                    if size > left:
+                        size = left
+                    chunk = contents_f.read(size)
+                    if not chunk:
+                        break
+                    conn.send(chunk)
+                    left -= len(chunk)
+
+    with Timeout(response_timeout):
+        resp = conn.getresponse()
+        resp.read()
+    if not is_success(resp.status):
+        raise DirectClientException(stype, method, node, part, path, resp)
+    return resp
+
+
 def _get_direct_account_container(path, stype, node, part,
-                                  account, marker=None, limit=None,
-                                  prefix=None, delimiter=None, conn_timeout=5,
-                                  response_timeout=15):
+                                  marker=None, limit=None,
+                                  prefix=None, delimiter=None,
+                                  conn_timeout=5, response_timeout=15,
+                                  end_marker=None, reverse=None, headers=None):
     """Base class for get direct account and container.
 
     Do not use directly use the get_direct_account or
     get_direct_container instead.
     """
-    qs = 'format=json'
+    params = ['format=json']
     if marker:
-        qs += '&marker=%s' % quote(marker)
+        params.append('marker=%s' % quote(marker))
     if limit:
-        qs += '&limit=%d' % limit
+        params.append('limit=%d' % limit)
     if prefix:
-        qs += '&prefix=%s' % quote(prefix)
+        params.append('prefix=%s' % quote(prefix))
     if delimiter:
-        qs += '&delimiter=%s' % quote(delimiter)
+        params.append('delimiter=%s' % quote(delimiter))
+    if end_marker:
+        params.append('end_marker=%s' % quote(end_marker))
+    if reverse:
+        params.append('reverse=%s' % quote(reverse))
+    qs = '&'.join(params)
     with Timeout(conn_timeout):
         conn = http_connect(node['ip'], node['port'], node['device'], part,
                             'GET', path, query_string=qs,
-                            headers=gen_headers())
+                            headers=gen_headers(hdrs_in=headers))
     with Timeout(response_timeout):
         resp = conn.getresponse()
     if not is_success(resp.status):
         resp.read()
         raise DirectClientException(stype, 'GET', node, part, path, resp)
+
     resp_headers = HeaderKeyDict()
     for header, value in resp.getheaders():
         resp_headers[header] = value
@@ -88,17 +171,18 @@ def _get_direct_account_container(path, stype, node, part,
     return resp_headers, json.loads(resp.read())
 
 
-def gen_headers(hdrs_in=None, add_ts=False):
+def gen_headers(hdrs_in=None, add_ts=False, add_user_agent=True):
     hdrs_out = HeaderKeyDict(hdrs_in) if hdrs_in else HeaderKeyDict()
     if add_ts:
-        hdrs_out['X-Timestamp'] = Timestamp(time()).internal
-    hdrs_out['User-Agent'] = 'direct-client %s' % os.getpid()
+        hdrs_out['X-Timestamp'] = Timestamp.now().internal
+    if add_user_agent:
+        hdrs_out['User-Agent'] = 'direct-client %s' % os.getpid()
     return hdrs_out
 
 
 def direct_get_account(node, part, account, marker=None, limit=None,
                        prefix=None, delimiter=None, conn_timeout=5,
-                       response_timeout=15):
+                       response_timeout=15, end_marker=None, reverse=None):
     """
     Get listings directly from the account server.
 
@@ -111,14 +195,18 @@ def direct_get_account(node, part, account, marker=None, limit=None,
     :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
+    :param end_marker: end_marker query
+    :param reverse: reverse the returned listing
     :returns: a tuple of (response headers, a list of containers) The response
               headers will HeaderKeyDict.
     """
     path = '/' + account
     return _get_direct_account_container(path, "Account", node, part,
-                                         account, marker=marker,
+                                         marker=marker,
                                          limit=limit, prefix=prefix,
                                          delimiter=delimiter,
+                                         end_marker=end_marker,
+                                         reverse=reverse,
                                          conn_timeout=conn_timeout,
                                          response_timeout=response_timeout)
 
@@ -129,16 +217,8 @@ def direct_delete_account(node, part, account, conn_timeout=5,
         headers = {}
 
     path = '/%s' % account
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'DELETE', path,
-                            headers=gen_headers(headers, True))
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Account', 'DELETE',
-                                    node, part, path, resp)
+    _make_req(node, part, 'DELETE', path, gen_headers(headers, True),
+              'Account', conn_timeout, response_timeout)
 
 
 def direct_head_container(node, part, account, container, conn_timeout=5,
@@ -156,15 +236,9 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
     :raises ClientException: HTTP HEAD request failed
     """
     path = '/%s/%s' % (account, container)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'HEAD', path, headers=gen_headers())
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Container', 'HEAD',
-                                    node, part, path, resp)
+    resp = _make_req(node, part, 'HEAD', path, gen_headers(),
+                     'Container', conn_timeout, response_timeout)
+
     resp_headers = HeaderKeyDict()
     for header, value in resp.getheaders():
         resp_headers[header] = value
@@ -173,7 +247,8 @@ def direct_head_container(node, part, account, container, conn_timeout=5,
 
 def direct_get_container(node, part, account, container, marker=None,
                          limit=None, prefix=None, delimiter=None,
-                         conn_timeout=5, response_timeout=15):
+                         conn_timeout=5, response_timeout=15, end_marker=None,
+                         reverse=None, headers=None):
     """
     Get container listings directly from the container server.
 
@@ -187,16 +262,22 @@ def direct_get_container(node, part, account, container, marker=None,
     :param delimiter: delimiter for the query
     :param conn_timeout: timeout in seconds for establishing the connection
     :param response_timeout: timeout in seconds for getting the response
+    :param end_marker: end_marker query
+    :param reverse: reverse the returned listing
+    :param headers: headers to be included in the request
     :returns: a tuple of (response headers, a list of objects) The response
               headers will be a HeaderKeyDict.
     """
     path = '/%s/%s' % (account, container)
     return _get_direct_account_container(path, "Container", node,
-                                         part, account, marker=marker,
+                                         part, marker=marker,
                                          limit=limit, prefix=prefix,
                                          delimiter=delimiter,
+                                         end_marker=end_marker,
+                                         reverse=reverse,
                                          conn_timeout=conn_timeout,
-                                         response_timeout=response_timeout)
+                                         response_timeout=response_timeout,
+                                         headers=headers)
 
 
 def direct_delete_container(node, part, account, container, conn_timeout=5,
@@ -218,16 +299,39 @@ def direct_delete_container(node, part, account, container, conn_timeout=5,
 
     path = '/%s/%s' % (account, container)
     add_timestamp = 'x-timestamp' not in (k.lower() for k in headers)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'DELETE', path,
-                            headers=gen_headers(headers, add_timestamp))
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Container', 'DELETE',
-                                    node, part, path, resp)
+    _make_req(node, part, 'DELETE', path, gen_headers(headers, add_timestamp),
+              'Container', conn_timeout, response_timeout)
+
+
+def direct_put_container(node, part, account, container, conn_timeout=5,
+                         response_timeout=15, headers=None, contents=None,
+                         content_length=None, chunk_size=65535):
+    """
+    Make a PUT request to a container server.
+
+    :param node: node dictionary from the ring
+    :param part: partition the container is on
+    :param account: account name
+    :param container: container name
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: additional headers to include in the request
+    :param contents: an iterable or string to send in request body (optional)
+    :param content_length: value to send as content-length header (optional)
+    :param chunk_size: chunk size of data to send (optional)
+    :raises ClientException: HTTP PUT request failed
+    """
+    if headers is None:
+        headers = {}
+
+    lower_headers = set(k.lower() for k in headers)
+    headers_out = gen_headers(headers,
+                              add_ts='x-timestamp' not in lower_headers,
+                              add_user_agent='user-agent' not in lower_headers)
+    path = '/%s/%s' % (account, container)
+    _make_req(node, part, 'PUT', path, headers_out, 'Container', conn_timeout,
+              response_timeout, contents=contents,
+              content_length=content_length, chunk_size=chunk_size)
 
 
 def direct_put_container_object(node, part, account, container, obj,
@@ -239,17 +343,9 @@ def direct_put_container_object(node, part, account, container, obj,
     have_x_timestamp = 'x-timestamp' in (k.lower() for k in headers)
 
     path = '/%s/%s/%s' % (account, container, obj)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'PUT', path,
-                            headers=gen_headers(headers,
-                                                add_ts=(not have_x_timestamp)))
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Container', 'PUT',
-                                    node, part, path, resp)
+    _make_req(node, part, 'PUT', path,
+              gen_headers(headers, add_ts=(not have_x_timestamp)),
+              'Container', conn_timeout, response_timeout)
 
 
 def direct_delete_container_object(node, part, account, container, obj,
@@ -262,16 +358,8 @@ def direct_delete_container_object(node, part, account, container, obj,
         k.lower() for k in headers))
 
     path = '/%s/%s/%s' % (account, container, obj)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'DELETE', path, headers=headers)
-
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Container', 'DELETE',
-                                    node, part, path, resp)
+    _make_req(node, part, 'DELETE', path, headers,
+              'Container', conn_timeout, response_timeout)
 
 
 def direct_head_object(node, part, account, container, obj, conn_timeout=5,
@@ -296,15 +384,9 @@ def direct_head_object(node, part, account, container, obj, conn_timeout=5,
     headers = gen_headers(headers)
 
     path = '/%s/%s/%s' % (account, container, obj)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'HEAD', path, headers=headers)
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Object', 'HEAD',
-                                    node, part, path, resp)
+    resp = _make_req(node, part, 'HEAD', path, headers,
+                     'Object', conn_timeout, response_timeout)
+
     resp_headers = HeaderKeyDict()
     for header, value in resp.getheaders():
         resp_headers[header] = value
@@ -340,8 +422,8 @@ def direct_get_object(node, part, account, container, obj, conn_timeout=5,
         resp = conn.getresponse()
     if not is_success(resp.status):
         resp.read()
-        raise DirectClientException('Object', 'GET',
-                                    node, part, path, resp)
+        raise DirectClientException('Object', 'GET', node, part, path, resp)
+
     if resp_chunk_size:
 
         def _object_body():
@@ -387,56 +469,18 @@ def direct_put_object(node, part, account, container, name, contents,
         headers = {}
     if etag:
         headers['ETag'] = etag.strip('"')
-    if content_length is not None:
-        headers['Content-Length'] = str(content_length)
-    else:
-        for n, v in headers.items():
-            if n.lower() == 'content-length':
-                content_length = int(v)
     if content_type is not None:
         headers['Content-Type'] = content_type
     else:
         headers['Content-Type'] = 'application/octet-stream'
-    if not contents:
-        headers['Content-Length'] = '0'
-    if isinstance(contents, basestring):
-        contents = [contents]
     # Incase the caller want to insert an object with specific age
     add_ts = 'X-Timestamp' not in headers
 
-    if content_length is None:
-        headers['Transfer-Encoding'] = 'chunked'
+    resp = _make_req(
+        node, part, 'PUT', path, gen_headers(headers, add_ts=add_ts),
+        'Object', conn_timeout, response_timeout, contents=contents,
+        content_length=content_length, chunk_size=chunk_size)
 
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'PUT', path, headers=gen_headers(headers, add_ts))
-
-    contents_f = FileLikeIter(contents)
-
-    if content_length is None:
-        chunk = contents_f.read(chunk_size)
-        while chunk:
-            conn.send('%x\r\n%s\r\n' % (len(chunk), chunk))
-            chunk = contents_f.read(chunk_size)
-        conn.send('0\r\n\r\n')
-    else:
-        left = content_length
-        while left > 0:
-            size = chunk_size
-            if size > left:
-                size = left
-            chunk = contents_f.read(size)
-            if not chunk:
-                break
-            conn.send(chunk)
-            left -= len(chunk)
-
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Object', 'PUT',
-                                    node, part, path, resp)
     return resp.getheader('etag').strip('"')
 
 
@@ -456,15 +500,8 @@ def direct_post_object(node, part, account, container, name, headers,
     :raises ClientException: HTTP POST request failed
     """
     path = '/%s/%s/%s' % (account, container, name)
-    with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'POST', path, headers=gen_headers(headers, True))
-    with Timeout(response_timeout):
-        resp = conn.getresponse()
-        resp.read()
-    if not is_success(resp.status):
-        raise DirectClientException('Object', 'POST',
-                                    node, part, path, resp)
+    _make_req(node, part, 'POST', path, gen_headers(headers, True),
+              'Object', conn_timeout, response_timeout)
 
 
 def direct_delete_object(node, part, account, container, obj,
@@ -488,15 +525,40 @@ def direct_delete_object(node, part, account, container, obj,
         k.lower() for k in headers))
 
     path = '/%s/%s/%s' % (account, container, obj)
+    _make_req(node, part, 'DELETE', path, headers,
+              'Object', conn_timeout, response_timeout)
+
+
+def direct_get_suffix_hashes(node, part, suffixes, conn_timeout=5,
+                             response_timeout=15, headers=None):
+    """
+    Get suffix hashes directly from the object server.
+
+    :param node: node dictionary from the ring
+    :param part: partition the container is on
+    :param conn_timeout: timeout in seconds for establishing the connection
+    :param response_timeout: timeout in seconds for getting the response
+    :param headers: dict to be passed into HTTPConnection headers
+    :returns: dict of suffix hashes
+    :raises ClientException: HTTP REPLICATE request failed
+    """
+    if headers is None:
+        headers = {}
+
+    path = '/%s' % '-'.join(suffixes)
     with Timeout(conn_timeout):
-        conn = http_connect(node['ip'], node['port'], node['device'], part,
-                            'DELETE', path, headers=headers)
+        conn = http_connect(node['replication_ip'], node['replication_port'],
+                            node['device'], part, 'REPLICATE', path,
+                            headers=gen_headers(headers))
     with Timeout(response_timeout):
         resp = conn.getresponse()
-        resp.read()
     if not is_success(resp.status):
-        raise DirectClientException('Object', 'DELETE',
-                                    node, part, path, resp)
+        raise DirectClientException('Object', 'REPLICATE',
+                                    node, part, path, resp,
+                                    host={'ip': node['replication_ip'],
+                                          'port': node['replication_port']}
+                                    )
+    return pickle.loads(resp.read())
 
 
 def retry(func, *args, **kwargs):
@@ -513,14 +575,8 @@ def retry(func, *args, **kwargs):
     :returns: result of func
     :raises ClientException: all retries failed
     """
-    retries = 5
-    if 'retries' in kwargs:
-        retries = kwargs['retries']
-        del kwargs['retries']
-    error_log = None
-    if 'error_log' in kwargs:
-        error_log = kwargs['error_log']
-        del kwargs['error_log']
+    retries = kwargs.pop('retries', 5)
+    error_log = kwargs.pop('error_log', None)
     attempts = 0
     backoff = 1
     while attempts <= retries:
