@@ -15,15 +15,14 @@
 
 import base64
 import json
-import xml.etree.cElementTree as ElementTree
 
 from swift import gettext_ as _
 from swift.common.http import is_success
 from swift.common.middleware.crypto.crypto_utils import CryptoWSGIContext, \
     load_crypto_meta, extract_crypto_meta, Crypto
-from swift.common.exceptions import EncryptionException
+from swift.common.exceptions import EncryptionException, UnknownSecretIdError
 from swift.common.request_helpers import get_object_transient_sysmeta, \
-    get_listing_content_type, get_sys_meta_prefix, get_user_meta_prefix
+    get_sys_meta_prefix, get_user_meta_prefix
 from swift.common.swob import Request, HTTPException, HTTPInternalServerError
 from swift.common.utils import get_logger, config_true_value, \
     parse_content_range, closing_if_possible, parse_content_type, \
@@ -40,11 +39,12 @@ def purge_crypto_sysmeta_headers(headers):
 
 
 class BaseDecrypterContext(CryptoWSGIContext):
-    def get_crypto_meta(self, header_name):
+    def get_crypto_meta(self, header_name, check=True):
         """
         Extract a crypto_meta dict from a header.
 
         :param header_name: name of header that may have crypto_meta
+        :param check: if True validate the crypto meta
         :return: A dict containing crypto_meta items
         :raises EncryptionException: if an error occurs while parsing the
                                      crypto meta
@@ -54,7 +54,8 @@ class BaseDecrypterContext(CryptoWSGIContext):
         if crypto_meta_json is None:
             return None
         crypto_meta = load_crypto_meta(crypto_meta_json)
-        self.crypto.check_crypto_meta(crypto_meta)
+        if check:
+            self.crypto.check_crypto_meta(crypto_meta)
         return crypto_meta
 
     def get_unwrapped_key(self, crypto_meta, wrapping_key):
@@ -65,8 +66,8 @@ class BaseDecrypterContext(CryptoWSGIContext):
         :param crypto_meta: a dict of crypto-meta
         :param wrapping_key: key to be used to decrypt the wrapped key
         :return: an unwrapped key
-        :raises EncryptionException: if the crypto-meta has no wrapped key or
-                                     the unwrapped key is invalid
+        :raises HTTPInternalServerError: if the crypto-meta has no wrapped key
+                                         or the unwrapped key is invalid
         """
         try:
             return self.crypto.unwrap_key(wrapping_key,
@@ -104,13 +105,14 @@ class BaseDecrypterContext(CryptoWSGIContext):
                                      to be decrypted but crypto meta was not
                                      found.
         """
-        value, crypto_meta = extract_crypto_meta(value)
+        extracted_value, crypto_meta = extract_crypto_meta(value)
         if crypto_meta:
             self.crypto.check_crypto_meta(crypto_meta)
-            value = self.decrypt_value(value, key, crypto_meta)
+            value = self.decrypt_value(extracted_value, key, crypto_meta)
         elif required:
             raise EncryptionException(
                 "Missing crypto meta in value %s" % value)
+
         return value
 
     def decrypt_value(self, value, key, crypto_meta):
@@ -129,18 +131,20 @@ class BaseDecrypterContext(CryptoWSGIContext):
             key, crypto_meta['iv'], 0)
         return crypto_ctxt.update(base64.b64decode(value))
 
-    def get_decryption_keys(self, req):
+    def get_decryption_keys(self, req, crypto_meta=None):
         """
         Determine if a response should be decrypted, and if so then fetch keys.
 
         :param req: a Request object
+        :param crypto_meta: a dict of crypto metadata
         :returns: a dict of decryption keys
         """
         if config_true_value(req.environ.get('swift.crypto.override')):
             self.logger.debug('No decryption is necessary because of override')
             return None
 
-        return self.get_keys(req.environ)
+        key_id = crypto_meta.get('key_id') if crypto_meta else None
+        return self.get_keys(req.environ, key_id=key_id)
 
 
 class DecrypterObjContext(BaseDecrypterContext):
@@ -186,11 +190,12 @@ class DecrypterObjContext(BaseDecrypterContext):
                 result.append((new_prefix + short_name, decrypted_value))
         return result
 
-    def decrypt_resp_headers(self, keys):
+    def decrypt_resp_headers(self, put_keys, post_keys):
         """
         Find encrypted headers and replace with the decrypted versions.
 
-        :param keys: a dict of decryption keys.
+        :param put_keys: a dict of decryption keys used for object PUT.
+        :param post_keys: a dict of decryption keys used for object POST.
         :return: A list of headers with any encrypted headers replaced by their
                  decrypted values.
         :raises HTTPInternalServerError: if any error occurs while decrypting
@@ -198,20 +203,23 @@ class DecrypterObjContext(BaseDecrypterContext):
         """
         mod_hdr_pairs = []
 
-        # Decrypt plaintext etag and place in Etag header for client response
-        etag_header = 'X-Object-Sysmeta-Crypto-Etag'
-        encrypted_etag = self._response_header_value(etag_header)
-        if encrypted_etag:
-            decrypted_etag = self._decrypt_header(
-                etag_header, encrypted_etag, keys['object'], required=True)
-            mod_hdr_pairs.append(('Etag', decrypted_etag))
+        if put_keys:
+            # Decrypt plaintext etag and place in Etag header for client
+            # response
+            etag_header = 'X-Object-Sysmeta-Crypto-Etag'
+            encrypted_etag = self._response_header_value(etag_header)
+            if encrypted_etag:
+                decrypted_etag = self._decrypt_header(
+                    etag_header, encrypted_etag, put_keys['object'],
+                    required=True)
+                mod_hdr_pairs.append(('Etag', decrypted_etag))
 
-        etag_header = 'X-Object-Sysmeta-Container-Update-Override-Etag'
-        encrypted_etag = self._response_header_value(etag_header)
-        if encrypted_etag:
-            decrypted_etag = self._decrypt_header(
-                etag_header, encrypted_etag, keys['container'])
-            mod_hdr_pairs.append((etag_header, decrypted_etag))
+            etag_header = 'X-Object-Sysmeta-Container-Update-Override-Etag'
+            encrypted_etag = self._response_header_value(etag_header)
+            if encrypted_etag:
+                decrypted_etag = self._decrypt_header(
+                    etag_header, encrypted_etag, put_keys['container'])
+                mod_hdr_pairs.append((etag_header, decrypted_etag))
 
         # Decrypt all user metadata. Encrypted user metadata values are stored
         # in the x-object-transient-sysmeta-crypto-meta- namespace. Those are
@@ -220,7 +228,8 @@ class DecrypterObjContext(BaseDecrypterContext):
         # if it does then they will be overwritten by any decrypted headers
         # that map to the same x-object-meta- header names i.e. decrypted
         # headers win over unexpected, unencrypted headers.
-        mod_hdr_pairs.extend(self.decrypt_user_metadata(keys))
+        if post_keys:
+            mod_hdr_pairs.extend(self.decrypt_user_metadata(post_keys))
 
         mod_hdr_names = {h.lower() for h, v in mod_hdr_pairs}
         mod_hdr_pairs.extend([(h, v) for h, v in self._response_headers
@@ -233,7 +242,7 @@ class DecrypterObjContext(BaseDecrypterContext):
 
         :param resp: application response
         :param boundary: multipart boundary string
-        :param keys: a dict of decryption keys.
+        :param body_key: decryption key for the response body
         :param crypto_meta: crypto_meta for the response body
         :return: generator for decrypted response body
         """
@@ -262,7 +271,7 @@ class DecrypterObjContext(BaseDecrypterContext):
         Decrypts a response body.
 
         :param resp: application response
-        :param keys: a dict of decryption keys.
+        :param body_key: decryption key for the response body
         :param crypto_meta: crypto_meta for the response body
         :param offset: offset into object content at which response body starts
         :return: generator for decrypted response body
@@ -273,31 +282,39 @@ class DecrypterObjContext(BaseDecrypterContext):
             for chunk in resp:
                 yield decrypt_ctxt.update(chunk)
 
+    def _read_crypto_meta(self, header, check):
+        crypto_meta = None
+        if (is_success(self._get_status_int()) or
+                self._get_status_int() in (304, 412)):
+            try:
+                crypto_meta = self.get_crypto_meta(header, check)
+            except EncryptionException as err:
+                self.logger.error(_('Error decrypting object: %s'), err)
+                raise HTTPInternalServerError(
+                    body='Error decrypting object', content_type='text/plain')
+        return crypto_meta
+
     def handle_get(self, req, start_response):
         app_resp = self._app_call(req.environ)
 
-        keys = self.get_decryption_keys(req)
-        if keys is None:
+        put_crypto_meta = self._read_crypto_meta(
+            'X-Object-Sysmeta-Crypto-Body-Meta', True)
+        put_keys = self.get_decryption_keys(req, put_crypto_meta)
+        post_crypto_meta = self._read_crypto_meta(
+            'X-Object-Transient-Sysmeta-Crypto-Meta', False)
+        post_keys = self.get_decryption_keys(req, post_crypto_meta)
+        if put_keys is None and post_keys is None:
             # skip decryption
             start_response(self._response_status, self._response_headers,
                            self._response_exc_info)
             return app_resp
 
-        mod_resp_headers = self.decrypt_resp_headers(keys)
+        mod_resp_headers = self.decrypt_resp_headers(put_keys, post_keys)
 
-        crypto_meta = None
-        if is_success(self._get_status_int()):
-            try:
-                crypto_meta = self.get_crypto_meta(
-                    'X-Object-Sysmeta-Crypto-Body-Meta')
-            except EncryptionException as err:
-                self.logger.error(_('Error decrypting object: %s'), err)
-                raise HTTPInternalServerError(
-                    body='Error decrypting object', content_type='text/plain')
-
-        if crypto_meta:
+        if put_crypto_meta and is_success(self._get_status_int()):
             # 2xx response and encrypted body
-            body_key = self.get_unwrapped_key(crypto_meta, keys['object'])
+            body_key = self.get_unwrapped_key(
+                put_crypto_meta, put_keys['object'])
             content_type, content_type_attrs = parse_content_type(
                 self._response_header_value('Content-Type'))
 
@@ -305,7 +322,7 @@ class DecrypterObjContext(BaseDecrypterContext):
                     content_type == 'multipart/byteranges'):
                 boundary = dict(content_type_attrs)["boundary"]
                 resp_iter = self.multipart_response_iter(
-                    app_resp, boundary, body_key, crypto_meta)
+                    app_resp, boundary, body_key, put_crypto_meta)
             else:
                 offset = 0
                 content_range = self._response_header_value('Content-Range')
@@ -313,7 +330,7 @@ class DecrypterObjContext(BaseDecrypterContext):
                     # Determine offset within the whole object if ranged GET
                     offset, end, total = parse_content_range(content_range)
                 resp_iter = self.response_iter(
-                    app_resp, body_key, crypto_meta, offset)
+                    app_resp, body_key, put_crypto_meta, offset)
         else:
             # don't decrypt body of unencrypted or non-2xx responses
             resp_iter = app_resp
@@ -326,15 +343,19 @@ class DecrypterObjContext(BaseDecrypterContext):
 
     def handle_head(self, req, start_response):
         app_resp = self._app_call(req.environ)
+        put_crypto_meta = self._read_crypto_meta(
+            'X-Object-Sysmeta-Crypto-Body-Meta', True)
+        put_keys = self.get_decryption_keys(req, put_crypto_meta)
+        post_crypto_meta = self._read_crypto_meta(
+            'X-Object-Transient-Sysmeta-Crypto-Meta', False)
+        post_keys = self.get_decryption_keys(req, post_crypto_meta)
 
-        keys = self.get_decryption_keys(req)
-
-        if keys is None:
+        if put_keys is None and post_keys is None:
             # skip decryption
             start_response(self._response_status, self._response_headers,
                            self._response_exc_info)
         else:
-            mod_resp_headers = self.decrypt_resp_headers(keys)
+            mod_resp_headers = self.decrypt_resp_headers(put_keys, post_keys)
             mod_resp_headers = purge_crypto_sysmeta_headers(mod_resp_headers)
             start_response(self._response_status, mod_resp_headers,
                            self._response_exc_info)
@@ -352,22 +373,18 @@ class DecrypterContContext(BaseDecrypterContext):
 
         if is_success(self._get_status_int()):
             # only decrypt body of 2xx responses
-            out_content_type = get_listing_content_type(req)
-            if out_content_type == 'application/json':
-                handler = self.process_json_resp
-                keys = self.get_decryption_keys(req)
-            elif out_content_type.endswith('/xml'):
-                handler = self.process_xml_resp
-                keys = self.get_decryption_keys(req)
-            else:
-                handler = keys = None
+            handler = None
+            for header, value in self._response_headers:
+                if header.lower() == 'content-type' and \
+                        value.split(';', 1)[0] == 'application/json':
+                    handler = self.process_json_resp
 
-            if handler and keys:
+            if handler:
                 try:
-                    app_resp = handler(keys['container'], app_resp)
+                    app_resp = handler(req, app_resp)
                 except EncryptionException as err:
                     self.logger.error(
-                        _("Error decrypting container listing: %s"),
+                        "Error decrypting container listing: %s",
                         err)
                     raise HTTPInternalServerError(
                         body='Error decrypting container listing',
@@ -379,13 +396,7 @@ class DecrypterContContext(BaseDecrypterContext):
 
         return app_resp
 
-    def update_content_length(self, new_total_len):
-        self._response_headers = [
-            (h, v) for h, v in self._response_headers
-            if h.lower() != 'content-length']
-        self._response_headers.append(('Content-Length', str(new_total_len)))
-
-    def process_json_resp(self, key, resp_iter):
+    def process_json_resp(self, req, resp_iter):
         """
         Parses json body listing and decrypt encrypted entries. Updates
         Content-Length header with new body length and return a body iter.
@@ -393,35 +404,35 @@ class DecrypterContContext(BaseDecrypterContext):
         with closing_if_possible(resp_iter):
             resp_body = ''.join(resp_iter)
         body_json = json.loads(resp_body)
-        new_body = json.dumps([self.decrypt_obj_dict(obj_dict, key)
+        new_body = json.dumps([self.decrypt_obj_dict(req, obj_dict)
                                for obj_dict in body_json])
         self.update_content_length(len(new_body))
         return [new_body]
 
-    def decrypt_obj_dict(self, obj_dict, key):
+    def decrypt_obj_dict(self, req, obj_dict):
         if 'hash' in obj_dict:
-            ciphertext = obj_dict['hash']
-            obj_dict['hash'] = self.decrypt_value_with_meta(ciphertext, key)
+            # each object's etag may have been encrypted with a different key
+            # so fetch keys based on its crypto meta
+            ciphertext, crypto_meta = extract_crypto_meta(obj_dict['hash'])
+            bad_keys = set()
+            if crypto_meta:
+                try:
+                    self.crypto.check_crypto_meta(crypto_meta)
+                    keys = self.get_decryption_keys(req, crypto_meta)
+                    obj_dict['hash'] = self.decrypt_value(
+                        ciphertext, keys['container'], crypto_meta)
+                except EncryptionException as err:
+                    if not isinstance(err, UnknownSecretIdError) or \
+                            err.args[0] not in bad_keys:
+                        # Only warn about an unknown key once per listing
+                        self.logger.error(
+                            "Error decrypting container listing: %s",
+                            err)
+                    if isinstance(err, UnknownSecretIdError):
+                        bad_keys.add(err.args[0])
+                    obj_dict['hash'] = '<unknown>'
         return obj_dict
 
-    def process_xml_resp(self, key, resp_iter):
-        """
-        Parses xml body listing and decrypt encrypted entries. Updates
-        Content-Length header with new body length and return a body iter.
-        """
-        with closing_if_possible(resp_iter):
-            resp_body = ''.join(resp_iter)
-        tree = ElementTree.fromstring(resp_body)
-        for elem in tree.iter('hash'):
-            ciphertext = elem.text.encode('utf8')
-            plain = self.decrypt_value_with_meta(ciphertext, key)
-            elem.text = plain.decode('utf8')
-        new_body = ElementTree.tostring(tree, encoding='UTF-8').replace(
-            "<?xml version='1.0' encoding='UTF-8'?>",
-            '<?xml version="1.0" encoding="UTF-8"?>', 1)
-        self.update_content_length(len(new_body))
-        return [new_body]
-
 
 class Decrypter(object):
     """Middleware for decrypting data and user metadata."""
