@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-"""
+r"""
 FormPost Middleware
 
 Translates a browser form post into a regular Swift object PUT.
@@ -86,7 +86,7 @@ sample code for computing the signature::
     max_file_count = 10
     expires = int(time() + 600)
     key = 'mykey'
-    hmac_body = '%s\\n%s\\n%s\\n%s\\n%s' % (path, redirect,
+    hmac_body = '%s\n%s\n%s\n%s\n%s' % (path, redirect,
         max_file_size, max_file_count, expires)
     signature = hmac.new(key, hmac_body, sha1).hexdigest()
 
@@ -113,15 +113,15 @@ the file are simply ignored).
 __all__ = ['FormPost', 'filter_factory', 'READ_CHUNK_SIZE', 'MAX_VALUE_LENGTH']
 
 import hmac
-import rfc822
 from hashlib import sha1
 from time import time
-from urllib import quote
 
+from six.moves.urllib.parse import quote
 from swift.common.exceptions import MimeInvalid
 from swift.common.middleware.tempurl import get_tempurl_keys_from_metadata
 from swift.common.utils import streq_const_time, register_swift_info, \
-    parse_content_disposition, iter_multipart_mime_documents
+    parse_content_disposition, parse_mime_headers, \
+    iter_multipart_mime_documents, reiterate, close_if_possible
 from swift.common.wsgi import make_pre_authed_env
 from swift.common.swob import HTTPUnauthorized
 from swift.proxy.controllers.base import get_account_info, get_container_info
@@ -254,9 +254,9 @@ class FormPost(object):
         file_count = 0
         for fp in iter_multipart_mime_documents(
                 env['wsgi.input'], boundary, read_chunk_size=READ_CHUNK_SIZE):
-            hdrs = rfc822.Message(fp, 0)
+            hdrs = parse_mime_headers(fp)
             disp, attrs = parse_content_disposition(
-                hdrs.getheader('Content-Disposition', ''))
+                hdrs.get('Content-Disposition', ''))
             if disp == 'form-data' and attrs.get('filename'):
                 file_count += 1
                 try:
@@ -270,9 +270,9 @@ class FormPost(object):
                 if 'content-type' not in attributes and 'content-type' in hdrs:
                     attributes['content-type'] = \
                         hdrs['Content-Type'] or 'application/octet-stream'
-                status, subheaders, message = \
+                status, subheaders = \
                     self._perform_subrequest(env, attributes, fp, keys)
-                if status[:1] != '2':
+                if not status.startswith('2'):
                     break
             else:
                 data = ''
@@ -323,7 +323,7 @@ class FormPost(object):
         :param attributes: dict of the attributes of the form so far.
         :param fp: The file-like object containing the request body.
         :param keys: The account keys to validate the signature with.
-        :returns: (status_line, headers_list, message)
+        :returns: (status_line, headers_list)
         """
         if not keys:
             raise FormUnauthorized('invalid signature')
@@ -337,7 +337,7 @@ class FormPost(object):
             del subenv['QUERY_STRING']
         subenv['HTTP_TRANSFER_ENCODING'] = 'chunked'
         subenv['wsgi.input'] = _CappedFileLikeObject(fp, max_file_size)
-        if subenv['PATH_INFO'][-1] != '/' and \
+        if not subenv['PATH_INFO'].endswith('/') and \
                 subenv['PATH_INFO'].count('/') < 4:
             subenv['PATH_INFO'] += '/'
         subenv['PATH_INFO'] += attributes['filename'] or 'filename'
@@ -357,8 +357,6 @@ class FormPost(object):
         if 'content-type' in attributes:
             subenv['CONTENT_TYPE'] = \
                 attributes['content-type'] or 'application/octet-stream'
-        elif 'CONTENT_TYPE' in subenv:
-            del subenv['CONTENT_TYPE']
         try:
             if int(attributes.get('expires') or 0) < time():
                 raise FormUnauthorized('form expired')
@@ -392,12 +390,10 @@ class FormPost(object):
             substatus[0] = status
             subheaders[0] = headers
 
-        i = iter(self.app(subenv, _start_response))
-        try:
-            next(i)
-        except StopIteration:
-            pass
-        return substatus[0], subheaders[0], ''
+        # reiterate to ensure the response started,
+        # but drop any data on the floor
+        close_if_possible(reiterate(self.app(subenv, _start_response)))
+        return substatus[0], subheaders[0]
 
     def _get_keys(self, env):
         """
