@@ -14,8 +14,8 @@
 # limitations under the License.
 
 import array
-import cPickle as pickle
-import inspect
+import six.moves.cPickle as pickle
+import json
 from collections import defaultdict
 from gzip import GzipFile
 from os.path import getmtime
@@ -26,72 +26,124 @@ from io import BufferedReader
 from hashlib import md5
 from itertools import chain
 from tempfile import NamedTemporaryFile
+import sys
 
-from swift.common.utils import hash_path, validate_configuration, json
+from six.moves import range
+
+from swift.common.exceptions import RingLoadError
+from swift.common.utils import hash_path, validate_configuration
 from swift.common.ring.utils import tiers_for_dev
 
 
+def calc_replica_count(replica2part2dev_id):
+    base = len(replica2part2dev_id) - 1
+    extra = 1.0 * len(replica2part2dev_id[-1]) / len(replica2part2dev_id[0])
+    return base + extra
+
+
 class RingData(object):
     """Partitioned consistent hashing ring data (used for serialization)."""
 
-    def __init__(self, replica2part2dev_id, devs, part_shift):
+    def __init__(self, replica2part2dev_id, devs, part_shift,
+                 next_part_power=None):
         self.devs = devs
         self._replica2part2dev_id = replica2part2dev_id
         self._part_shift = part_shift
+        self.next_part_power = next_part_power
 
         for dev in self.devs:
             if dev is not None:
                 dev.setdefault("region", 1)
 
+    @property
+    def replica_count(self):
+        """Number of replicas (full or partial) used in the ring."""
+        return calc_replica_count(self._replica2part2dev_id)
+
     @classmethod
-    def deserialize_v1(cls, gz_file):
+    def deserialize_v1(cls, gz_file, metadata_only=False):
+        """
+        Deserialize a v1 ring file into a dictionary with `devs`, `part_shift`,
+        and `replica2part2dev_id` keys.
+
+        If the optional kwarg `metadata_only` is True, then the
+        `replica2part2dev_id` is not loaded and that key in the returned
+        dictionary just has the value `[]`.
+
+        :param file gz_file: An opened file-like object which has already
+                             consumed the 6 bytes of magic and version.
+        :param bool metadata_only: If True, only load `devs` and `part_shift`
+        :returns: A dict containing `devs`, `part_shift`, and
+                  `replica2part2dev_id`
+        """
+
         json_len, = struct.unpack('!I', gz_file.read(4))
-        ring_dict = json.loads(gz_file.read(json_len))
+        ring_dict = json.loads(gz_file.read(json_len).decode('ascii'))
         ring_dict['replica2part2dev_id'] = []
+
+        if metadata_only:
+            return ring_dict
+
+        byteswap = (ring_dict.get('byteorder', sys.byteorder) != sys.byteorder)
+
         partition_count = 1 << (32 - ring_dict['part_shift'])
-        for x in xrange(ring_dict['replica_count']):
-            ring_dict['replica2part2dev_id'].append(
-                array.array('H', gz_file.read(2 * partition_count)))
+        for x in range(ring_dict['replica_count']):
+            part2dev = array.array('H', gz_file.read(2 * partition_count))
+            if byteswap:
+                part2dev.byteswap()
+            ring_dict['replica2part2dev_id'].append(part2dev)
+
         return ring_dict
 
     @classmethod
-    def load(cls, filename):
+    def load(cls, filename, metadata_only=False):
         """
         Load ring data from a file.
 
         :param filename: Path to a file serialized by the save() method.
+        :param bool metadata_only: If True, only load `devs` and `part_shift`.
         :returns: A RingData instance containing the loaded data.
         """
-        gz_file = GzipFile(filename, 'rb')
-        # Python 2.6 GzipFile doesn't support BufferedIO
-        if hasattr(gz_file, '_checkReadable'):
-            gz_file = BufferedReader(gz_file)
+        gz_file = BufferedReader(GzipFile(filename, 'rb'))
 
         # See if the file is in the new format
         magic = gz_file.read(4)
-        if magic == 'R1NG':
-            version, = struct.unpack('!H', gz_file.read(2))
-            if version == 1:
-                ring_data = cls.deserialize_v1(gz_file)
+        if magic == b'R1NG':
+            format_version, = struct.unpack('!H', gz_file.read(2))
+            if format_version == 1:
+                ring_data = cls.deserialize_v1(
+                    gz_file, metadata_only=metadata_only)
             else:
-                raise Exception('Unknown ring format version %d' % version)
+                raise Exception('Unknown ring format version %d' %
+                                format_version)
         else:
             # Assume old-style pickled ring
             gz_file.seek(0)
             ring_data = pickle.load(gz_file)
+
         if not hasattr(ring_data, 'devs'):
             ring_data = RingData(ring_data['replica2part2dev_id'],
-                                 ring_data['devs'], ring_data['part_shift'])
+                                 ring_data['devs'], ring_data['part_shift'],
+                                 ring_data.get('next_part_power'))
         return ring_data
 
     def serialize_v1(self, file_obj):
         # Write out new-style serialization magic and version:
-        file_obj.write(struct.pack('!4sH', 'R1NG', 1))
+        file_obj.write(struct.pack('!4sH', b'R1NG', 1))
         ring = self.to_dict()
-        json_encoder = json.JSONEncoder(sort_keys=True)
-        json_text = json_encoder.encode(
-            {'devs': ring['devs'], 'part_shift': ring['part_shift'],
-             'replica_count': len(ring['replica2part2dev_id'])})
+
+        # Only include next_part_power if it is set in the
+        # builder, otherwise just ignore it
+        _text = {'devs': ring['devs'], 'part_shift': ring['part_shift'],
+                 'replica_count': len(ring['replica2part2dev_id']),
+                 'byteorder': sys.byteorder}
+
+        next_part_power = ring.get('next_part_power')
+        if next_part_power is not None:
+            _text['next_part_power'] = next_part_power
+
+        json_text = json.dumps(_text, sort_keys=True,
+                               ensure_ascii=True).encode('ascii')
         json_len = len(json_text)
         file_obj.write(struct.pack('!I', json_len))
         file_obj.write(json_text)
@@ -109,15 +161,8 @@ class RingData(object):
         # Override the timestamp so that the same ring data creates
         # the same bytes on disk. This makes a checksum comparison a
         # good way to see if two rings are identical.
-        #
-        # This only works on Python 2.7; on 2.6, we always get the
-        # current time in the gzip output.
         tempf = NamedTemporaryFile(dir=".", prefix=filename, delete=False)
-        if 'mtime' in inspect.getargspec(GzipFile.__init__).args:
-            gz_file = GzipFile(filename, mode='wb', fileobj=tempf,
-                               mtime=mtime)
-        else:
-            gz_file = GzipFile(filename, mode='wb', fileobj=tempf)
+        gz_file = GzipFile(filename, mode='wb', fileobj=tempf, mtime=mtime)
         self.serialize_v1(gz_file)
         gz_file.close()
         tempf.flush()
@@ -129,7 +174,8 @@ class RingData(object):
     def to_dict(self):
         return {'devs': self.devs,
                 'replica2part2dev_id': self._replica2part2dev_id,
-                'part_shift': self._part_shift}
+                'part_shift': self._part_shift,
+                'next_part_power': self.next_part_power}
 
 
 class Ring(object):
@@ -138,9 +184,14 @@ class Ring(object):
 
     :param serialized_path: path to serialized RingData instance
     :param reload_time: time interval in seconds to check for a ring change
+    :param ring_name: ring name string (basically specified from policy)
+    :param validation_hook: hook point to validate ring configuration ontime
+
+    :raises RingLoadError: if the loaded ring data violates its constraint
     """
 
-    def __init__(self, serialized_path, reload_time=15, ring_name=None):
+    def __init__(self, serialized_path, reload_time=15, ring_name=None,
+                 validation_hook=lambda ring_data: None):
         # can't use the ring unless HASH_PATH_SUFFIX is set
         validate_configuration()
         if ring_name:
@@ -149,12 +200,24 @@ class Ring(object):
         else:
             self.serialized_path = os.path.join(serialized_path)
         self.reload_time = reload_time
+        self._validation_hook = validation_hook
         self._reload(force=True)
 
     def _reload(self, force=False):
         self._rtime = time() + self.reload_time
         if force or self.has_changed():
             ring_data = RingData.load(self.serialized_path)
+
+            try:
+                self._validation_hook(ring_data)
+            except RingLoadError:
+                if force:
+                    raise
+                else:
+                    # In runtime reload at working server, it's ok to use old
+                    # ring data if the new ring data is invalid.
+                    return
+
             self._mtime = getmtime(self.serialized_path)
             self._devs = ring_data.devs
             # NOTE(akscram): Replication parameters like replication_ip
@@ -177,20 +240,39 @@ class Ring(object):
 
             # Do this now, when we know the data has changed, rather than
             # doing it on every call to get_more_nodes().
+            #
+            # Since this is to speed up the finding of handoffs, we only
+            # consider devices with at least one partition assigned. This
+            # way, a region, zone, or server with no partitions assigned
+            # does not count toward our totals, thereby keeping the early
+            # bailouts in get_more_nodes() working.
+            dev_ids_with_parts = set()
+            for part2dev_id in self._replica2part2dev_id:
+                for dev_id in part2dev_id:
+                    dev_ids_with_parts.add(dev_id)
+
             regions = set()
             zones = set()
-            ip_ports = set()
+            ips = set()
             self._num_devs = 0
             for dev in self._devs:
-                if dev:
+                if dev and dev['id'] in dev_ids_with_parts:
                     regions.add(dev['region'])
                     zones.add((dev['region'], dev['zone']))
-                    ip_ports.add((dev['region'], dev['zone'],
-                                  dev['ip'], dev['port']))
+                    ips.add((dev['region'], dev['zone'], dev['ip']))
                     self._num_devs += 1
             self._num_regions = len(regions)
             self._num_zones = len(zones)
-            self._num_ip_ports = len(ip_ports)
+            self._num_ips = len(ips)
+            self._next_part_power = ring_data.next_part_power
+
+    @property
+    def next_part_power(self):
+        return self._next_part_power
+
+    @property
+    def part_power(self):
+        return 32 - self._part_shift
 
     def _rebuild_tier_data(self):
         self.tier2devs = defaultdict(list)
@@ -211,7 +293,7 @@ class Ring(object):
     @property
     def replica_count(self):
         """Number of replicas (full or partial) used in the ring."""
-        return len(self._replica2part2dev_id)
+        return calc_replica_count(self._replica2part2dev_id)
 
     @property
     def partition_count(self):
@@ -243,7 +325,7 @@ class Ring(object):
                 if dev_id not in seen_ids:
                     part_nodes.append(self.devs[dev_id])
                     seen_ids.add(dev_id)
-        return part_nodes
+        return [dict(node, index=i) for i, node in enumerate(part_nodes)]
 
     def get_part(self, account, container=None, obj=None):
         """
@@ -291,6 +373,7 @@ class Ring(object):
 
         ======  ===============================================================
         id      unique integer identifier amongst devices
+        index   offset into the primary node list for the partition
         weight  a float of the relative weight of this device as compared to
                 others; this indicates how many partitions the builder will try
                 to assign to this device
@@ -328,19 +411,19 @@ class Ring(object):
         used = set(d['id'] for d in primary_nodes)
         same_regions = set(d['region'] for d in primary_nodes)
         same_zones = set((d['region'], d['zone']) for d in primary_nodes)
-        same_ip_ports = set((d['region'], d['zone'], d['ip'], d['port'])
-                            for d in primary_nodes)
+        same_ips = set(
+            (d['region'], d['zone'], d['ip']) for d in primary_nodes)
 
         parts = len(self._replica2part2dev_id[0])
-        start = struct.unpack_from(
-            '>I', md5(str(part)).digest())[0] >> self._part_shift
+        part_hash = md5(str(part).encode('ascii')).digest()
+        start = struct.unpack_from('>I', part_hash)[0] >> self._part_shift
         inc = int(parts / 65536) or 1
         # Multiple loops for execution speed; the checks and bookkeeping get
         # simpler as you go along
         hit_all_regions = len(same_regions) == self._num_regions
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_regions:
                 # At this point, there are no regions left untouched, so we
                 # can stop looking.
@@ -355,17 +438,17 @@ class Ring(object):
                         used.add(dev_id)
                         same_regions.add(region)
                         zone = dev['zone']
-                        ip_port = (region, zone, dev['ip'], dev['port'])
+                        ip = (region, zone, dev['ip'])
                         same_zones.add((region, zone))
-                        same_ip_ports.add(ip_port)
+                        same_ips.add(ip)
                         if len(same_regions) == self._num_regions:
                             hit_all_regions = True
                             break
 
         hit_all_zones = len(same_zones) == self._num_zones
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_zones:
                 # Much like we stopped looking for fresh regions before, we
                 # can now stop looking for fresh zones; there are no more.
@@ -379,17 +462,17 @@ class Ring(object):
                         yield dev
                         used.add(dev_id)
                         same_zones.add(zone)
-                        ip_port = zone + (dev['ip'], dev['port'])
-                        same_ip_ports.add(ip_port)
+                        ip = zone + (dev['ip'],)
+                        same_ips.add(ip)
                         if len(same_zones) == self._num_zones:
                             hit_all_zones = True
                             break
 
-        hit_all_ip_ports = len(same_ip_ports) == self._num_ip_ports
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
-            if hit_all_ip_ports:
+        hit_all_ips = len(same_ips) == self._num_ips
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
+            if hit_all_ips:
                 # We've exhausted the pool of unused backends, so stop
                 # looking.
                 break
@@ -397,20 +480,19 @@ class Ring(object):
                 if handoff_part < len(part2dev_id):
                     dev_id = part2dev_id[handoff_part]
                     dev = self._devs[dev_id]
-                    ip_port = (dev['region'], dev['zone'],
-                               dev['ip'], dev['port'])
-                    if dev_id not in used and ip_port not in same_ip_ports:
+                    ip = (dev['region'], dev['zone'], dev['ip'])
+                    if dev_id not in used and ip not in same_ips:
                         yield dev
                         used.add(dev_id)
-                        same_ip_ports.add(ip_port)
-                        if len(same_ip_ports) == self._num_ip_ports:
-                            hit_all_ip_ports = True
+                        same_ips.add(ip)
+                        if len(same_ips) == self._num_ips:
+                            hit_all_ips = True
                             break
 
         hit_all_devs = len(used) == self._num_devs
-        for handoff_part in chain(xrange(start, parts, inc),
-                                  xrange(inc - ((parts - start) % inc),
-                                         start, inc)):
+        for handoff_part in chain(range(start, parts, inc),
+                                  range(inc - ((parts - start) % inc),
+                                        start, inc)):
             if hit_all_devs:
                 # We've used every device we have, so let's stop looking for
                 # unused devices now.
