@@ -18,11 +18,9 @@
 from __future__ import print_function
 
 import errno
-import inspect
 import os
 import signal
 import time
-import mimetools
 from swift import gettext_ as _
 from textwrap import dedent
 
@@ -35,6 +33,8 @@ import six
 from six import BytesIO
 from six import StringIO
 from six.moves.urllib.parse import unquote
+if six.PY2:
+    import mimetools
 
 from swift.common import utils, constraints
 from swift.common.storage_policy import BindPortsCache
@@ -42,7 +42,10 @@ from swift.common.swob import Request
 from swift.common.utils import capture_stdio, disable_fallocate, \
     drop_privileges, get_logger, NullLogger, config_true_value, \
     validate_configuration, get_hub, config_auto_int_value, \
-    CloseableChain
+    reiterate
+
+SIGNUM_TO_NAME = {getattr(signal, n): n for n in dir(signal)
+                  if n.startswith('SIG') and '_' not in n}
 
 # Set maximum line size of message headers to be accepted.
 wsgi.MAX_HEADER_LINE = constraints.MAX_HEADER_SIZE
@@ -64,6 +67,7 @@ class NamedConfigLoader(loadwsgi.ConfigLoader):
         context = super(NamedConfigLoader, self).get_context(
             object_type, name=name, global_conf=global_conf)
         context.name = name
+        context.local_conf['__name__'] = name
         return context
 
 
@@ -113,10 +117,12 @@ class ConfigString(NamedConfigLoader):
         self.filename = "string"
         defaults = {
             'here': "string",
-            '__file__': "string",
+            '__file__': self.contents,
         }
         self.parser = loadwsgi.NicerConfigParser("string", defaults=defaults)
         self.parser.optionxform = str  # Don't lower-case keys
+        # Defaults don't need interpolation (crazy PasteDeploy...)
+        self.parser.defaults = lambda: dict(self.parser._defaults, **defaults)
         self.parser.readfp(self.contents)
 
 
@@ -147,6 +153,9 @@ def monkey_patch_mimetools():
     mimetools.Message defaults content-type to "text/plain"
     This changes it to default to None, so we can detect missing headers.
     """
+    if six.PY3:
+        # The mimetools has been removed from Python 3
+        return
 
     orig_parsetype = mimetools.Message.parsetype
 
@@ -392,28 +401,139 @@ def loadapp(conf_file, global_conf=None, allow_modify_pipeline=True):
     return ctx.create()
 
 
+def load_app_config(conf_file):
+    """
+    Read the app config section from a config file.
+
+    :param conf_file: path to a config file
+    :return: a dict
+    """
+    app_conf = {}
+    try:
+        ctx = loadcontext(loadwsgi.APP, conf_file)
+    except LookupError:
+        pass
+    else:
+        app_conf.update(ctx.app_context.global_conf)
+        app_conf.update(ctx.app_context.local_conf)
+    return app_conf
+
+
+class SwiftHttpProtocol(wsgi.HttpProtocol):
+    default_request_version = "HTTP/1.0"
+
+    def log_request(self, *a):
+        """
+        Turn off logging requests by the underlying WSGI software.
+        """
+        pass
+
+    def log_message(self, f, *a):
+        """
+        Redirect logging other messages by the underlying WSGI software.
+        """
+        logger = getattr(self.server.app, 'logger', None)
+        if logger:
+            logger.error('ERROR WSGI: ' + f, *a)
+        else:
+            # eventlet<=0.17.4 doesn't have an error method, and in newer
+            # versions the output from error is same as info anyway
+            self.server.log.info('ERROR WSGI: ' + f, *a)
+
+
+class SwiftHttpProxiedProtocol(SwiftHttpProtocol):
+    """
+    Protocol object that speaks HTTP, including multiple requests, but with
+    a single PROXY line as the very first thing coming in over the socket.
+    This is so we can learn what the client's IP address is when Swift is
+    behind a TLS terminator, like hitch, that does not understand HTTP and
+    so cannot add X-Forwarded-For or other similar headers.
+
+    See http://www.haproxy.org/download/1.7/doc/proxy-protocol.txt for
+    protocol details.
+    """
+    def __init__(self, *a, **kw):
+        self.proxy_address = None
+        SwiftHttpProtocol.__init__(self, *a, **kw)
+
+    def handle_error(self, connection_line):
+        if not six.PY2:
+            connection_line = connection_line.decode('latin-1')
+
+        # No further processing will proceed on this connection under any
+        # circumstances.  We always send the request into the superclass to
+        # handle any cleanup - this ensures that the request will not be
+        # processed.
+        self.rfile.close()
+        # We don't really have any confidence that an HTTP Error will be
+        # processable by the client as our transmission broken down between
+        # ourselves and our gateway proxy before processing the client
+        # protocol request.  Hopefully the operator will know what to do!
+        msg = 'Invalid PROXY line %r' % connection_line
+        self.log_message(msg)
+        # Even assuming HTTP we don't even known what version of HTTP the
+        # client is sending?  This entire endeavor seems questionable.
+        self.request_version = self.default_request_version
+        # appease http.server
+        self.command = 'PROXY'
+        self.send_error(400, msg)
+
+    def handle(self):
+        """Handle multiple requests if necessary."""
+        # ensure the opening line for the connection is a valid PROXY protcol
+        # line; this is the only IO we do on this connection before any
+        # additional wrapping further pollutes the raw socket.
+        connection_line = self.rfile.readline(self.server.url_length_limit)
+
+        if not connection_line.startswith(b'PROXY '):
+            return self.handle_error(connection_line)
+
+        proxy_parts = connection_line.strip(b'\r\n').split(b' ')
+        if proxy_parts[1].startswith(b'UNKNOWN'):
+            # "UNKNOWN", in PROXY protocol version 1, means "not
+            # TCP4 or TCP6". This includes completely legitimate
+            # things like QUIC or Unix domain sockets. The PROXY
+            # protocol (section 2.1) states that the receiver
+            # (that's us) MUST ignore anything after "UNKNOWN" and
+            # before the CRLF, essentially discarding the first
+            # line.
+            pass
+        elif proxy_parts[1] in (b'TCP4', b'TCP6') and len(proxy_parts) == 6:
+            if six.PY2:
+                self.client_address = (proxy_parts[2], proxy_parts[4])
+                self.proxy_address = (proxy_parts[3], proxy_parts[5])
+            else:
+                self.client_address = (
+                    proxy_parts[2].decode('latin-1'),
+                    proxy_parts[4].decode('latin-1'))
+                self.proxy_address = (
+                    proxy_parts[3].decode('latin-1'),
+                    proxy_parts[5].decode('latin-1'))
+        else:
+            self.handle_error(connection_line)
+
+        return SwiftHttpProtocol.handle(self)
+
+    def get_environ(self):
+        environ = SwiftHttpProtocol.get_environ(self)
+        if self.proxy_address:
+            environ['SERVER_ADDR'] = self.proxy_address[0]
+            environ['SERVER_PORT'] = self.proxy_address[1]
+            if self.proxy_address[1] == '443':
+                environ['wsgi.url_scheme'] = 'https'
+                environ['HTTPS'] = 'on'
+        return environ
+
+
 def run_server(conf, logger, sock, global_conf=None):
     # Ensure TZ environment variable exists to avoid stat('/etc/localtime') on
-    # some platforms. This locks in reported times to the timezone in which
-    # the server first starts running in locations that periodically change
-    # timezones.
-    os.environ['TZ'] = time.strftime("%z", time.gmtime())
+    # some platforms. This locks in reported times to UTC.
+    os.environ['TZ'] = 'UTC+0'
+    time.tzset()
 
-    wsgi.HttpProtocol.default_request_version = "HTTP/1.0"
-    # Turn off logging requests by the underlying WSGI software.
-    wsgi.HttpProtocol.log_request = lambda *a: None
-    # Redirect logging other messages by the underlying WSGI software.
-    wsgi.HttpProtocol.log_message = \
-        lambda s, f, *a: logger.error('ERROR WSGI: ' + f % a)
     wsgi.WRITE_TIMEOUT = int(conf.get('client_timeout') or 60)
 
     eventlet.hubs.use_hub(get_hub())
-    # NOTE(sileht):
-    #     monkey-patching thread is required by python-keystoneclient;
-    #     monkey-patching select is required by oslo.messaging pika driver
-    #         if thread is monkey-patched.
-    eventlet.patcher.monkey_patch(all=False, socket=True, select=True,
-                                  thread=True)
     eventlet_debug = config_true_value(conf.get('eventlet_debug', 'no'))
     eventlet.debug.hub_exceptions(eventlet_debug)
     wsgi_logger = NullLogger()
@@ -430,15 +550,24 @@ def run_server(conf, logger, sock, global_conf=None):
     app = loadapp(conf['__file__'], global_conf=global_conf)
     max_clients = int(conf.get('max_clients', '1024'))
     pool = RestrictedGreenPool(size=max_clients)
+
+    # Select which protocol class to use (normal or one expecting PROXY
+    # protocol)
+    if config_true_value(conf.get('require_proxy_protocol', 'no')):
+        protocol_class = SwiftHttpProxiedProtocol
+    else:
+        protocol_class = SwiftHttpProtocol
+
+    server_kwargs = {
+        'custom_pool': pool,
+        'protocol': protocol_class,
+        # Disable capitalizing headers in Eventlet. This is necessary for
+        # the AWS SDK to work with s3api middleware (it needs an "ETag"
+        # header; "Etag" just won't do).
+        'capitalize_response_headers': False,
+    }
     try:
-        # Disable capitalizing headers in Eventlet if possible.  This is
-        # necessary for the AWS SDK to work with swift3 middleware.
-        argspec = inspect.getargspec(wsgi.server)
-        if 'capitalize_response_headers' in argspec.args:
-            wsgi.server(sock, app, wsgi_logger, custom_pool=pool,
-                        capitalize_response_headers=False)
-        else:
-            wsgi.server(sock, app, wsgi_logger, custom_pool=pool)
+        wsgi.server(sock, app, wsgi_logger, **server_kwargs)
     except socket.error as err:
         if err[0] != errno.EINVAL:
             raise
@@ -476,7 +605,7 @@ class WorkersStrategy(object):
 
         return 0.5
 
-    def bind_ports(self):
+    def do_bind_ports(self):
         """
         Bind the one listen socket for this strategy and drop privileges
         (since the parent process will never need to bind again).
@@ -541,7 +670,8 @@ class WorkersStrategy(object):
         :param int pid: The new worker process' PID
         """
 
-        self.logger.notice('Started child %s' % pid)
+        self.logger.notice('Started child %s from parent %s',
+                           pid, os.getpid())
         self.children.append(pid)
 
     def register_worker_exit(self, pid):
@@ -551,7 +681,8 @@ class WorkersStrategy(object):
         :param int pid: The PID of the worker that exited.
         """
 
-        self.logger.error('Removing dead child %s' % pid)
+        self.logger.error('Removing dead child %s from parent %s',
+                          pid, os.getpid())
         self.children.remove(pid)
 
     def shutdown_sockets(self):
@@ -644,10 +775,7 @@ class PortPidState(object):
         Yield all current listen sockets.
         """
 
-        # Use six.itervalues() instead of calling directly the .values() method
-        # on Python 2 to avoid a temporary list, because sock_data_by_port
-        # comes from users and can be large.
-        for orphan_data in six.itervalues(self.sock_data_by_port):
+        for orphan_data in self.sock_data_by_port.values():
             yield orphan_data['sock']
 
     def forget_port(self, port):
@@ -730,7 +858,7 @@ class ServersPerPortStrategy(object):
 
         return self.ring_check_interval
 
-    def bind_ports(self):
+    def do_bind_ports(self):
         """
         Bind one listen socket per unique local storage policy ring port.  Then
         do all the work of drop_privileges except the actual dropping of
@@ -877,6 +1005,9 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         print(e)
         return 1
 
+    # optional nice/ionice priority scheduling
+    utils.modify_priority(conf, logger)
+
     servers_per_port = int(conf.get('servers_per_port', '0') or 0)
 
     # NOTE: for now servers_per_port is object-server-only; future work could
@@ -888,11 +1019,8 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
     else:
         strategy = WorkersStrategy(conf, logger)
 
-    error_msg = strategy.bind_ports()
-    if error_msg:
-        logger.error(error_msg)
-        print(error_msg)
-        return 1
+    # patch event before loadapp
+    utils.eventlet_monkey_patch()
 
     # Ensure the configuration and application can be loaded before proceeding.
     global_conf = {'log_name': log_name}
@@ -904,7 +1032,15 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
     utils.FALLOCATE_RESERVE, utils.FALLOCATE_IS_PERCENT = \
         utils.config_fallocate_value(conf.get('fallocate_reserve', '1%'))
 
-    # redirect errors to logger and close stdio
+    # Start listening on bind_addr/port
+    error_msg = strategy.do_bind_ports()
+    if error_msg:
+        logger.error(error_msg)
+        print(error_msg)
+        return 1
+
+    # Redirect errors to logger and close stdio. Do this *after* binding ports;
+    # we use this to signal that the service is ready to accept connections.
     capture_stdio(logger)
 
     no_fork_sock = strategy.no_fork_sock()
@@ -912,24 +1048,17 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
         run_server(conf, logger, no_fork_sock, global_conf=global_conf)
         return 0
 
-    def kill_children(*args):
-        """Kills the entire process group."""
-        logger.error('SIGTERM received')
-        signal.signal(signal.SIGTERM, signal.SIG_IGN)
-        running[0] = False
-        os.killpg(0, signal.SIGTERM)
+    def stop_with_signal(signum, *args):
+        """Set running flag to False and capture the signum"""
+        running_context[0] = False
+        running_context[1] = signum
 
-    def hup(*args):
-        """Shuts down the server, but allows running requests to complete"""
-        logger.error('SIGHUP received')
-        signal.signal(signal.SIGHUP, signal.SIG_IGN)
-        running[0] = False
+    # context to hold boolean running state and stop signum
+    running_context = [True, None]
+    signal.signal(signal.SIGTERM, stop_with_signal)
+    signal.signal(signal.SIGHUP, stop_with_signal)
 
-    running = [True]
-    signal.signal(signal.SIGTERM, kill_children)
-    signal.signal(signal.SIGHUP, hup)
-
-    while running[0]:
+    while running_context[0]:
         for sock, sock_info in strategy.new_worker_socks():
             pid = os.fork()
             if pid == 0:
@@ -952,19 +1081,40 @@ def run_wsgi(conf_path, app_section, *args, **kwargs):
 
         with Timeout(loop_timeout, exception=False):
             try:
-                pid, status = green_os.wait()
-                if os.WIFEXITED(status) or os.WIFSIGNALED(status):
-                    strategy.register_worker_exit(pid)
-            except OSError as err:
-                if err.errno not in (errno.EINTR, errno.ECHILD):
-                    raise
+                try:
+                    pid, status = green_os.wait()
+                    if os.WIFEXITED(status) or os.WIFSIGNALED(status):
+                        strategy.register_worker_exit(pid)
+                except OSError as err:
+                    if err.errno not in (errno.EINTR, errno.ECHILD):
+                        raise
+                    if err.errno == errno.ECHILD:
+                        # If there are no children at all (ECHILD), then
+                        # there's nothing to actually wait on. We sleep
+                        # for a little bit to avoid a tight CPU spin
+                        # and still are able to catch any KeyboardInterrupt
+                        # events that happen. The value of 0.01 matches the
+                        # value in eventlet's waitpid().
+                        sleep(0.01)
             except KeyboardInterrupt:
                 logger.notice('User quit')
-                running[0] = False
+                running_context[0] = False
                 break
 
+    if running_context[1] is not None:
+        try:
+            signame = SIGNUM_TO_NAME[running_context[1]]
+        except KeyError:
+            logger.error('Stopping with unexpected signal %r' %
+                         running_context[1])
+        else:
+            logger.error('%s received', signame)
+    if running_context[1] == signal.SIGTERM:
+        os.killpg(0, signal.SIGTERM)
+
     strategy.shutdown_sockets()
-    logger.notice('Exited')
+    signal.signal(signal.SIGTERM, signal.SIG_IGN)
+    logger.notice('Exited (%s)', os.getpid())
     return 0
 
 
@@ -1044,16 +1194,11 @@ class WSGIContext(object):
         self._response_headers = None
         self._response_exc_info = None
         resp = self.app(env, self._start_response)
-        # if start_response has been called, just return the iter
-        if self._response_status is not None:
-            return resp
-        resp = iter(resp)
-        try:
-            first_chunk = next(resp)
-        except StopIteration:
-            return iter([])
-        else:  # We got a first_chunk
-            return CloseableChain([first_chunk], resp)
+        # if start_response has not been called, iterate until we've got a
+        # non-empty chunk, by which time the app *should* have called it
+        if self._response_status is None:
+            resp = reiterate(resp)
+        return resp
 
     def _get_status_int(self):
         """
@@ -1069,6 +1214,12 @@ class WSGIContext(object):
                 return val
         return None
 
+    def update_content_length(self, new_total_len):
+        self._response_headers = [
+            (h, v) for h, v in self._response_headers
+            if h.lower() != 'content-length']
+        self._response_headers.append(('Content-Length', str(new_total_len)))
+
 
 def make_env(env, method=None, path=None, agent='Swift', query_string=None,
              swift_source=None):
@@ -1104,8 +1255,7 @@ def make_env(env, method=None, path=None, agent='Swift', query_string=None,
                  'SERVER_PROTOCOL', 'swift.cache', 'swift.source',
                  'swift.trans_id', 'swift.authorize_override',
                  'swift.authorize', 'HTTP_X_USER_ID', 'HTTP_X_PROJECT_ID',
-                 'HTTP_REFERER', 'swift.orig_req_method', 'swift.log_info',
-                 'swift.infocache'):
+                 'HTTP_REFERER', 'swift.infocache'):
         if name in env:
             newenv[name] = env[name]
     if method:
