@@ -15,28 +15,32 @@
 
 import os
 import itertools
-import time
+import json
 from collections import defaultdict
 from eventlet import Timeout
 
+from swift.container.sync_store import ContainerSyncStore
 from swift.container.backend import ContainerBroker, DATADIR
 from swift.container.reconciler import (
     MISPLACED_OBJECTS_ACCOUNT, incorrect_policy_index,
     get_reconciler_container_name, get_row_to_q_entry_translator)
 from swift.common import db_replicator
 from swift.common.storage_policy import POLICIES
+from swift.common.swob import HTTPOk, HTTPAccepted
 from swift.common.exceptions import DeviceUnavailable
 from swift.common.http import is_success
-from swift.common.db import DatabaseAlreadyExists
-from swift.common.utils import (json, Timestamp, hash_path,
-                                storage_directory, quorum_size)
+from swift.common.utils import Timestamp, majority_size, get_db_files
 
 
 class ContainerReplicator(db_replicator.Replicator):
     server_type = 'container'
     brokerclass = ContainerBroker
     datadir = DATADIR
-    default_port = 6001
+    default_port = 6201
+
+    def __init__(self, conf, logger=None):
+        super(ContainerReplicator, self).__init__(conf, logger=logger)
+        self.reconciler_cleanups = self.sync_store = None
 
     def report_up_to_date(self, full_info):
         reported_key_map = {
@@ -60,12 +64,11 @@ class ContainerReplicator(db_replicator.Replicator):
         return sync_args
 
     def _handle_sync_response(self, node, response, info, broker, http,
-                              different_region):
-        parent = super(ContainerReplicator, self)
+                              different_region=False):
         if is_success(response.status):
             remote_info = json.loads(response.data)
             if incorrect_policy_index(info, remote_info):
-                status_changed_at = Timestamp(time.time())
+                status_changed_at = Timestamp.now()
                 broker.set_storage_policy_index(
                     remote_info['storage_policy_index'],
                     timestamp=status_changed_at.internal)
@@ -74,9 +77,59 @@ class ContainerReplicator(db_replicator.Replicator):
             if any(info[key] != remote_info[key] for key in sync_timestamps):
                 broker.merge_timestamps(*(remote_info[key] for key in
                                           sync_timestamps))
-        rv = parent._handle_sync_response(
+
+            if 'shard_max_row' in remote_info:
+                # Grab remote's shard ranges, too
+                self._fetch_and_merge_shard_ranges(http, broker)
+
+        return super(ContainerReplicator, self)._handle_sync_response(
             node, response, info, broker, http, different_region)
-        return rv
+
+    def _sync_shard_ranges(self, broker, http, local_id):
+        # TODO: currently the number of shard ranges is expected to be _much_
+        # less than normal objects so all are sync'd on each cycle. However, in
+        # future there should be sync points maintained much like for object
+        # syncing so that only new shard range rows are sync'd.
+        shard_range_data = broker.get_all_shard_range_data()
+        if shard_range_data:
+            if not self._send_replicate_request(
+                    http, 'merge_shard_ranges', shard_range_data, local_id):
+                return False
+            self.logger.debug('%s synced %s shard ranges to %s',
+                              broker.db_file, len(shard_range_data),
+                              '%(ip)s:%(port)s/%(device)s' % http.node)
+        return True
+
+    def _choose_replication_mode(self, node, rinfo, info, local_sync, broker,
+                                 http, different_region):
+        if 'shard_max_row' in rinfo:
+            # Always replicate shard ranges to new-enough swift
+            shard_range_success = self._sync_shard_ranges(
+                broker, http, info['id'])
+        else:
+            shard_range_success = False
+            self.logger.warning(
+                '%s is unable to replicate shard ranges to peer %s; '
+                'peer may need upgrading', broker.db_file,
+                '%(ip)s:%(port)s/%(device)s' % node)
+        if broker.sharding_initiated():
+            self.logger.warning(
+                '%s is able to shard -- refusing to replicate objects to peer '
+                '%s; have shard ranges and will wait for cleaving',
+                broker.db_file,
+                '%(ip)s:%(port)s/%(device)s' % node)
+            self.stats['deferred'] += 1
+            return shard_range_success
+
+        success = super(ContainerReplicator, self)._choose_replication_mode(
+            node, rinfo, info, local_sync, broker, http,
+            different_region)
+        return shard_range_success and success
+
+    def _fetch_and_merge_shard_ranges(self, http, broker):
+        response = http.replicate('get_shard_ranges')
+        if is_success(response.status):
+            broker.merge_shard_ranges(json.loads(response.data))
 
     def find_local_handoff_for_part(self, part):
         """
@@ -113,15 +166,10 @@ class ContainerReplicator(db_replicator.Replicator):
             raise DeviceUnavailable(
                 'No mounted devices found suitable to Handoff reconciler '
                 'container %s in partition %s' % (container, part))
-        hsh = hash_path(account, container)
-        db_dir = storage_directory(DATADIR, part, hsh)
-        db_path = os.path.join(self.root, node['device'], db_dir, hsh + '.db')
-        broker = ContainerBroker(db_path, account=account, container=container)
-        if not os.path.exists(broker.db_file):
-            try:
-                broker.initialize(timestamp, 0)
-            except DatabaseAlreadyExists:
-                pass
+        broker = ContainerBroker.create_broker(
+            os.path.join(self.root, node['device']), part, account, container,
+            logger=self.logger, put_timestamp=timestamp,
+            storage_policy_index=0)
         if self.reconciler_containers is not None:
             self.reconciler_containers[container] = part, broker, node['id']
         return broker
@@ -188,17 +236,36 @@ class ContainerReplicator(db_replicator.Replicator):
     def _post_replicate_hook(self, broker, info, responses):
         if info['account'] == MISPLACED_OBJECTS_ACCOUNT:
             return
+
+        try:
+            self.sync_store.update_sync_store(broker)
+        except Exception:
+            self.logger.exception('Failed to update sync_store %s' %
+                                  broker.db_file)
+
         point = broker.get_reconciler_sync()
         if not broker.has_multiple_policies() and info['max_row'] != point:
             broker.update_reconciler_sync(info['max_row'])
             return
         max_sync = self.dump_to_reconciler(broker, point)
-        success = responses.count(True) >= quorum_size(len(responses))
+        success = responses.count(True) >= majority_size(len(responses))
         if max_sync > point and success:
-            # to be safe, only slide up the sync point with a quorum on
+            # to be safe, only slide up the sync point with a majority on
             # replication
             broker.update_reconciler_sync(max_sync)
 
+    def cleanup_post_replicate(self, broker, orig_info, responses):
+        if broker.sharding_required():
+            # despite being a handoff, since we're sharding we're not going to
+            # do any cleanup so we can continue cleaving - this is still
+            # considered "success"
+            self.logger.debug(
+                'Not deleting db %s (requires sharding, state %s)',
+                broker.db_file, broker.get_db_state())
+            return True
+        return super(ContainerReplicator, self).cleanup_post_replicate(
+            broker, orig_info, responses)
+
     def delete_db(self, broker):
         """
         Ensure that reconciler databases are only cleaned up at the end of the
@@ -209,6 +276,14 @@ class ContainerReplicator(db_replicator.Replicator):
             # this container shouldn't be here, make sure it's cleaned up
             self.reconciler_cleanups[broker.container] = broker
             return
+        if self.sync_store:
+            try:
+                # DB is going to get deleted. Be preemptive about it
+                self.sync_store.remove_synced_container(broker)
+            except Exception:
+                self.logger.exception('Failed to remove sync_store entry %s' %
+                                      broker.db_file)
+
         return super(ContainerReplicator, self).delete_db(broker)
 
     def replicate_reconcilers(self):
@@ -236,6 +311,9 @@ class ContainerReplicator(db_replicator.Replicator):
     def run_once(self, *args, **kwargs):
         self.reconciler_containers = {}
         self.reconciler_cleanups = {}
+        self.sync_store = ContainerSyncStore(self.root,
+                                             self.logger,
+                                             self.mount_check)
         rv = super(ContainerReplicator, self).run_once(*args, **kwargs)
         if any([self.reconciler_containers, self.reconciler_cleanups]):
             self.replicate_reconcilers()
@@ -244,6 +322,9 @@ class ContainerReplicator(db_replicator.Replicator):
 
 class ContainerReplicatorRpc(db_replicator.ReplicatorRpc):
 
+    def _db_file_exists(self, db_path):
+        return bool(get_db_files(db_path))
+
     def _parse_sync_args(self, args):
         parent = super(ContainerReplicatorRpc, self)
         remote_info = parent._parse_sync_args(args)
@@ -265,9 +346,33 @@ class ContainerReplicatorRpc(db_replicator.ReplicatorRpc):
         """
         info = broker.get_replication_info()
         if incorrect_policy_index(info, remote_info):
-            status_changed_at = Timestamp(time.time()).internal
+            status_changed_at = Timestamp.now().internal
             broker.set_storage_policy_index(
                 remote_info['storage_policy_index'],
                 timestamp=status_changed_at)
             info = broker.get_replication_info()
         return info
+
+    def _abort_rsync_then_merge(self, db_file, old_filename):
+        if super(ContainerReplicatorRpc, self)._abort_rsync_then_merge(
+                db_file, old_filename):
+            return True
+        # if the local db has started sharding since the original 'sync'
+        # request then abort object replication now; instantiate a fresh broker
+        # each time this check if performed so to get latest state
+        broker = ContainerBroker(db_file)
+        return broker.sharding_initiated()
+
+    def _post_rsync_then_merge_hook(self, existing_broker, new_broker):
+        # Note the following hook will need to change to using a pointer and
+        # limit in the future.
+        new_broker.merge_shard_ranges(
+            existing_broker.get_all_shard_range_data())
+
+    def merge_shard_ranges(self, broker, args):
+        broker.merge_shard_ranges(args[0])
+        return HTTPAccepted()
+
+    def get_shard_ranges(self, broker, args):
+        return HTTPOk(headers={'Content-Type': 'application/json'},
+                      body=json.dumps(broker.get_all_shard_range_data()))
