@@ -15,6 +15,7 @@
 
 """ Object Server for Swift """
 
+import six
 import six.moves.cPickle as pickle
 import json
 import os
@@ -26,14 +27,15 @@ import math
 from swift import gettext_ as _
 from hashlib import md5
 
-from eventlet import sleep, wsgi, Timeout
+from eventlet import sleep, wsgi, Timeout, tpool
 from eventlet.greenthread import spawn
 
 from swift.common.utils import public, get_logger, \
     config_true_value, timing_stats, replication, \
     normalize_delete_at_timestamp, get_log_line, Timestamp, \
     get_expirer_container, parse_mime_headers, \
-    iter_multipart_mime_documents, extract_swift_bytes, safe_json_loads
+    iter_multipart_mime_documents, extract_swift_bytes, safe_json_loads, \
+    config_auto_int_value, split_path, get_redirect_data, normalize_timestamp
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_object_creation, \
     valid_timestamp, check_utf8
@@ -42,7 +44,7 @@ from swift.common.exceptions import ConnectionTimeout, DiskFileQuarantined, \
     DiskFileDeviceUnavailable, DiskFileExpired, ChunkReadTimeout, \
     ChunkReadError, DiskFileXattrNotSupported
 from swift.obj import ssync_receiver
-from swift.common.http import is_success
+from swift.common.http import is_success, HTTP_MOVED_PERMANENTLY
 from swift.common.base_storage_server import BaseStorageServer
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.request_helpers import get_name_and_placement, \
@@ -54,7 +56,7 @@ from swift.common.swob import HTTPAccepted, HTTPBadRequest, HTTPCreated, \
     HTTPClientDisconnect, HTTPMethodNotAllowed, Request, Response, \
     HTTPInsufficientStorage, HTTPForbidden, HTTPException, HTTPConflict, \
     HTTPServerError
-from swift.obj.diskfile import DATAFILE_SYSTEM_META, DiskFileRouter
+from swift.obj.diskfile import RESERVED_DATAFILE_META, DiskFileRouter
 
 
 def iter_mime_headers_and_bodies(wsgi_input, mime_boundary, read_chunk_size):
@@ -139,6 +141,10 @@ class ObjectController(BaseStorageServer):
             x-delete-at,
             x-object-manifest,
             x-static-large-object,
+            cache-control,
+            content-language,
+            expires,
+            x-robots-tag
         '''
         extra_allowed_headers = [
             header.strip().lower() for header in conf.get(
@@ -147,7 +153,7 @@ class ObjectController(BaseStorageServer):
         ]
         self.allowed_headers = set()
         for header in extra_allowed_headers:
-            if header not in DATAFILE_SYSTEM_META:
+            if header not in RESERVED_DATAFILE_META:
                 self.allowed_headers.add(header)
         self.auto_create_account_prefix = \
             conf.get('auto_create_account_prefix') or '.'
@@ -169,7 +175,9 @@ class ObjectController(BaseStorageServer):
         # disk_chunk_size parameter. However, it affects all created sockets
         # using this class so we have chosen to tie it to the
         # network_chunk_size parameter value instead.
-        socket._fileobject.default_bufsize = self.network_chunk_size
+        if six.PY2:
+            socket._fileobject.default_bufsize = self.network_chunk_size
+        # TODO: find a way to enable similar functionality in py3
 
         # Provide further setup specific to an object server implementation.
         self.setup(conf)
@@ -198,6 +206,34 @@ class ObjectController(BaseStorageServer):
         self.replication_failure_ratio = float(
             conf.get('replication_failure_ratio') or 1.0)
 
+        servers_per_port = int(conf.get('servers_per_port', '0') or 0)
+        if servers_per_port:
+            # The typical servers-per-port deployment also uses one port per
+            # disk, so you really get N servers per disk. In that case,
+            # having a pool of 20 threads per server per disk is far too
+            # much. For example, given a 60-disk chassis and 4 servers per
+            # disk, the default configuration will give us 21 threads per
+            # server (the main thread plus the twenty tpool threads), for a
+            # total of around 60 * 21 * 4 = 5040 threads. This is clearly
+            # too high.
+            #
+            # Instead, we use a tpool size of 1, giving us 2 threads per
+            # process. In the example above, that's 60 * 2 * 4 = 480
+            # threads, which is reasonable since there are 240 processes.
+            default_tpool_size = 1
+        else:
+            # If we're not using servers-per-port, then leave the tpool size
+            # alone. The default (20) is typically good enough for one
+            # object server handling requests for many disks.
+            default_tpool_size = None
+
+        tpool_size = config_auto_int_value(
+            conf.get('eventlet_tpool_num_threads'),
+            default_tpool_size)
+
+        if tpool_size:
+            tpool.set_num_threads(tpool_size)
+
     def get_diskfile(self, device, partition, account, container, obj,
                      policy, **kwargs):
         """
@@ -213,7 +249,7 @@ class ObjectController(BaseStorageServer):
 
     def async_update(self, op, account, container, obj, host, partition,
                      contdevice, headers_out, objdevice, policy,
-                     logger_thread_locals=None):
+                     logger_thread_locals=None, container_path=None):
         """
         Sends or saves an async update.
 
@@ -231,11 +267,21 @@ class ObjectController(BaseStorageServer):
         :param logger_thread_locals: The thread local values to be set on the
                                      self.logger to retain transaction
                                      logging information.
+        :param container_path: optional path in the form `<account/container>`
+            to which the update should be sent. If given this path will be used
+            instead of constructing a path from the ``account`` and
+            ``container`` params.
         """
         if logger_thread_locals:
             self.logger.thread_locals = logger_thread_locals
         headers_out['user-agent'] = 'object-server %s' % os.getpid()
-        full_path = '/%s/%s/%s' % (account, container, obj)
+        if container_path:
+            # use explicitly specified container path
+            full_path = '/%s/%s' % (container_path, obj)
+        else:
+            full_path = '/%s/%s/%s' % (account, container, obj)
+
+        redirect_data = None
         if all([host, partition, contdevice]):
             try:
                 with ConnectionTimeout(self.conn_timeout):
@@ -245,15 +291,23 @@ class ObjectController(BaseStorageServer):
                 with Timeout(self.node_timeout):
                     response = conn.getresponse()
                     response.read()
-                    if is_success(response.status):
-                        return
-                    else:
-                        self.logger.error(_(
-                            'ERROR Container update failed '
-                            '(saving for async update later): %(status)d '
-                            'response from %(ip)s:%(port)s/%(dev)s'),
-                            {'status': response.status, 'ip': ip, 'port': port,
-                             'dev': contdevice})
+                if is_success(response.status):
+                    return
+
+                if response.status == HTTP_MOVED_PERMANENTLY:
+                    try:
+                        redirect_data = get_redirect_data(response)
+                    except ValueError as err:
+                        self.logger.error(
+                            'Container update failed for %r; problem with '
+                            'redirect location: %s' % (obj, err))
+                else:
+                    self.logger.error(_(
+                        'ERROR Container update failed '
+                        '(saving for async update later): %(status)d '
+                        'response from %(ip)s:%(port)s/%(dev)s'),
+                        {'status': response.status, 'ip': ip, 'port': port,
+                         'dev': contdevice})
             except (Exception, Timeout):
                 self.logger.exception(_(
                     'ERROR container update failed with '
@@ -261,6 +315,13 @@ class ObjectController(BaseStorageServer):
                     {'ip': ip, 'port': port, 'dev': contdevice})
         data = {'op': op, 'account': account, 'container': container,
                 'obj': obj, 'headers': headers_out}
+        if redirect_data:
+            self.logger.debug(
+                'Update to %(path)s redirected to %(redirect)s',
+                {'path': full_path, 'redirect': redirect_data[0]})
+            container_path = redirect_data[0]
+        if container_path:
+            data['container_path'] = container_path
         timestamp = headers_out.get('x-meta-timestamp',
                                     headers_out.get('x-timestamp'))
         self._diskfile_router[policy].pickle_async_update(
@@ -287,6 +348,7 @@ class ObjectController(BaseStorageServer):
         contdevices = [d.strip() for d in
                        headers_in.get('X-Container-Device', '').split(',')]
         contpartition = headers_in.get('X-Container-Partition', '')
+        contpath = headers_in.get('X-Backend-Container-Path')
 
         if len(conthosts) != len(contdevices):
             # This shouldn't happen unless there's a bug in the proxy,
@@ -299,6 +361,21 @@ class ObjectController(BaseStorageServer):
                     'devices': headers_in.get('X-Container-Device', '')})
             return
 
+        if contpath:
+            try:
+                # TODO: this is very late in request handling to be validating
+                # a header - if we did *not* check and the header was bad
+                # presumably the update would fail and we would fall back to an
+                # async update to the root container, which might be best
+                # course of action rather than aborting update altogether?
+                split_path('/' + contpath, minsegs=2, maxsegs=2)
+            except ValueError:
+                self.logger.error(
+                    "Invalid X-Backend-Container-Path, should be of the form "
+                    "'account/container' but got %r." % contpath)
+                # fall back to updating root container
+                contpath = None
+
         if contpartition:
             updates = zip(conthosts, contdevices)
         else:
@@ -312,7 +389,8 @@ class ObjectController(BaseStorageServer):
             gt = spawn(self.async_update, op, account, container, obj,
                        conthost, contpartition, contdevice, headers_out,
                        objdevice, policy,
-                       logger_thread_locals=self.logger.thread_locals)
+                       logger_thread_locals=self.logger.thread_locals,
+                       container_path=contpath)
             update_greenthreads.append(gt)
         # Wait a little bit to see if the container updates are successful.
         # If we immediately return after firing off the greenthread above, then
@@ -360,19 +438,26 @@ class ObjectController(BaseStorageServer):
             'x-trans-id': headers_in.get('x-trans-id', '-'),
             'referer': request.as_referer()})
         if op != 'DELETE':
+            hosts = headers_in.get('X-Delete-At-Host', None)
+            if hosts is None:
+                # If header is missing, no update needed as sufficient other
+                # object servers should perform the required update.
+                return
             delete_at_container = headers_in.get('X-Delete-At-Container', None)
             if not delete_at_container:
+                # older proxy servers did not send X-Delete-At-Container so for
+                # backwards compatibility calculate the value here, but also
+                # log a warning because this is prone to inconsistent
+                # expiring_objects_container_divisor configurations.
+                # See https://bugs.launchpad.net/swift/+bug/1187200
                 self.logger.warning(
                     'X-Delete-At-Container header must be specified for '
                     'expiring objects background %s to work properly. Making '
                     'best guess as to the container name for now.' % op)
-                # TODO(gholt): In a future release, change the above warning to
-                # a raised exception and remove the guess code below.
                 delete_at_container = get_expirer_container(
                     delete_at, self.expiring_objects_container_divisor,
                     account, container, obj)
             partition = headers_in.get('X-Delete-At-Partition', None)
-            hosts = headers_in.get('X-Delete-At-Host', '')
             contdevices = headers_in.get('X-Delete-At-Device', '')
             updates = [upd for upd in
                        zip((h.strip() for h in hosts.split(',')),
@@ -384,6 +469,11 @@ class ObjectController(BaseStorageServer):
             headers_out['x-content-type'] = 'text/plain'
             headers_out['x-etag'] = 'd41d8cd98f00b204e9800998ecf8427e'
         else:
+            if not config_true_value(
+                request.headers.get(
+                    'X-Backend-Clean-Expiring-Object-Queue', 't')):
+                return
+
             # DELETEs of old expiration data have no way of knowing what the
             # old X-Delete-At-Container was at the time of the initial setting
             # of the data, so a best guess is made here.
@@ -407,7 +497,10 @@ class ObjectController(BaseStorageServer):
     def _make_timeout_reader(self, file_like):
         def timeout_reader():
             with ChunkReadTimeout(self.client_timeout):
-                return file_like.read(self.network_chunk_size)
+                try:
+                    return file_like.read(self.network_chunk_size)
+                except (IOError, ValueError):
+                    raise ChunkReadError
         return timeout_reader
 
     def _read_put_commit_message(self, mime_documents_iter):
@@ -494,11 +587,6 @@ class ObjectController(BaseStorageServer):
                     override = key.lower().replace(override_prefix, 'x-')
                     update_headers[override] = val
 
-    def _preserve_slo_manifest(self, update_metadata, orig_metadata):
-        if 'X-Static-Large-Object' in orig_metadata:
-            update_metadata['X-Static-Large-Object'] = \
-                orig_metadata['X-Static-Large-Object']
-
     @public
     @timing_stats()
     def POST(self, request):
@@ -507,17 +595,20 @@ class ObjectController(BaseStorageServer):
             get_name_and_placement(request, 5, 5, True)
         req_timestamp = valid_timestamp(request)
         new_delete_at = int(request.headers.get('X-Delete-At') or 0)
-        if new_delete_at and new_delete_at < time.time():
+        if new_delete_at and new_delete_at < req_timestamp:
             return HTTPBadRequest(body='X-Delete-At in past', request=request,
                                   content_type='text/plain')
+        next_part_power = request.headers.get('X-Backend-Next-Part-Power')
         try:
             disk_file = self.get_diskfile(
                 device, partition, account, container, obj,
-                policy=policy)
+                policy=policy, open_expired=config_true_value(
+                    request.headers.get('x-backend-replication', 'false')),
+                next_part_power=next_part_power)
         except DiskFileDeviceUnavailable:
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
-            orig_metadata = disk_file.read_metadata()
+            orig_metadata = disk_file.read_metadata(current_time=req_timestamp)
         except DiskFileXattrNotSupported:
             return HTTPInsufficientStorage(drive=device, request=request)
         except (DiskFileNotExist, DiskFileQuarantined):
@@ -538,7 +629,6 @@ class ObjectController(BaseStorageServer):
 
         if req_timestamp > orig_timestamp:
             metadata = {'X-Timestamp': req_timestamp.internal}
-            self._preserve_slo_manifest(metadata, orig_metadata)
             metadata.update(val for val in request.headers.items()
                             if (is_user_meta('object', val[0]) or
                                 is_object_transient_sysmeta(val[0])))
@@ -650,9 +740,6 @@ class ObjectController(BaseStorageServer):
         if error_response:
             return error_response
         new_delete_at = int(request.headers.get('X-Delete-At') or 0)
-        if new_delete_at and new_delete_at < time.time():
-            return HTTPBadRequest(body='X-Delete-At in past', request=request,
-                                  content_type='text/plain')
         try:
             fsize = request.message_length()
         except ValueError as e:
@@ -674,19 +761,21 @@ class ObjectController(BaseStorageServer):
         # nodes; handoff nodes should 409 subrequests to over-write an
         # existing data fragment until they offloaded the existing fragment
         frag_index = request.headers.get('X-Backend-Ssync-Frag-Index')
+        next_part_power = request.headers.get('X-Backend-Next-Part-Power')
         try:
             disk_file = self.get_diskfile(
                 device, partition, account, container, obj,
-                policy=policy, frag_index=frag_index)
+                policy=policy, frag_index=frag_index,
+                next_part_power=next_part_power)
         except DiskFileDeviceUnavailable:
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
-            orig_metadata = disk_file.read_metadata()
+            orig_metadata = disk_file.read_metadata(current_time=req_timestamp)
             orig_timestamp = disk_file.data_timestamp
         except DiskFileXattrNotSupported:
             return HTTPInsufficientStorage(drive=device, request=request)
         except DiskFileDeleted as e:
-            orig_metadata = e.metadata
+            orig_metadata = {}
             orig_timestamp = e.timestamp
         except (DiskFileNotExist, DiskFileQuarantined):
             orig_metadata = {}
@@ -818,9 +907,9 @@ class ObjectController(BaseStorageServer):
                         send_hundred_continue_response()
                     if not self._read_put_commit_message(mime_documents_iter):
                         return HTTPServerError(request=request)
-                    # got 2nd phase confirmation, write a timestamp.durable
-                    # state file to indicate a successful PUT
 
+                # got 2nd phase confirmation (when required), call commit to
+                # indicate a successful PUT
                 writer.commit(request.timestamp)
 
                 # Drain any remaining MIME docs from the socket. There
@@ -869,16 +958,21 @@ class ObjectController(BaseStorageServer):
         """Handle HTTP GET requests for the Swift Object Server."""
         device, partition, account, container, obj, policy = \
             get_name_and_placement(request, 5, 5, True)
+        request.headers.setdefault('X-Timestamp',
+                                   normalize_timestamp(time.time()))
+        req_timestamp = valid_timestamp(request)
         frag_prefs = safe_json_loads(
             request.headers.get('X-Backend-Fragment-Preferences'))
         try:
             disk_file = self.get_diskfile(
                 device, partition, account, container, obj,
-                policy=policy, frag_prefs=frag_prefs)
+                policy=policy, frag_prefs=frag_prefs,
+                open_expired=config_true_value(
+                    request.headers.get('x-backend-replication', 'false')))
         except DiskFileDeviceUnavailable:
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
-            with disk_file.open():
+            with disk_file.open(current_time=req_timestamp):
                 metadata = disk_file.get_metadata()
                 obj_size = int(metadata['Content-Length'])
                 file_x_ts = Timestamp(metadata['X-Timestamp'])
@@ -931,16 +1025,21 @@ class ObjectController(BaseStorageServer):
         """Handle HTTP HEAD requests for the Swift Object Server."""
         device, partition, account, container, obj, policy = \
             get_name_and_placement(request, 5, 5, True)
+        request.headers.setdefault('X-Timestamp',
+                                   normalize_timestamp(time.time()))
+        req_timestamp = valid_timestamp(request)
         frag_prefs = safe_json_loads(
             request.headers.get('X-Backend-Fragment-Preferences'))
         try:
             disk_file = self.get_diskfile(
                 device, partition, account, container, obj,
-                policy=policy, frag_prefs=frag_prefs)
+                policy=policy, frag_prefs=frag_prefs,
+                open_expired=config_true_value(
+                    request.headers.get('x-backend-replication', 'false')))
         except DiskFileDeviceUnavailable:
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
-            metadata = disk_file.read_metadata()
+            metadata = disk_file.read_metadata(current_time=req_timestamp)
         except DiskFileXattrNotSupported:
             return HTTPInsufficientStorage(drive=device, request=request)
         except (DiskFileNotExist, DiskFileQuarantined) as e:
@@ -986,14 +1085,15 @@ class ObjectController(BaseStorageServer):
         device, partition, account, container, obj, policy = \
             get_name_and_placement(request, 5, 5, True)
         req_timestamp = valid_timestamp(request)
+        next_part_power = request.headers.get('X-Backend-Next-Part-Power')
         try:
             disk_file = self.get_diskfile(
                 device, partition, account, container, obj,
-                policy=policy)
+                policy=policy, next_part_power=next_part_power)
         except DiskFileDeviceUnavailable:
             return HTTPInsufficientStorage(drive=device, request=request)
         try:
-            orig_metadata = disk_file.read_metadata()
+            orig_metadata = disk_file.read_metadata(current_time=req_timestamp)
         except DiskFileXattrNotSupported:
             return HTTPInsufficientStorage(drive=device, request=request)
         except DiskFileExpired as e:
@@ -1015,10 +1115,10 @@ class ObjectController(BaseStorageServer):
             else:
                 response_class = HTTPConflict
         response_timestamp = max(orig_timestamp, req_timestamp)
-        orig_delete_at = int(orig_metadata.get('X-Delete-At') or 0)
+        orig_delete_at = Timestamp(orig_metadata.get('X-Delete-At') or 0)
         try:
             req_if_delete_at_val = request.headers['x-if-delete-at']
-            req_if_delete_at = int(req_if_delete_at_val)
+            req_if_delete_at = Timestamp(req_if_delete_at_val)
         except KeyError:
             pass
         except ValueError:
@@ -1031,6 +1131,9 @@ class ObjectController(BaseStorageServer):
             if not orig_timestamp:
                 # no object found at all
                 return HTTPNotFound()
+            if orig_timestamp >= req_timestamp:
+                # Found a newer object -- return 409 as work item is stale
+                return HTTPConflict()
             if orig_delete_at != req_if_delete_at:
                 return HTTPPreconditionFailed(
                     request=request,
@@ -1110,6 +1213,7 @@ class ObjectController(BaseStorageServer):
                     ' %(path)s '), {'method': req.method, 'path': req.path})
                 res = HTTPInternalServerError(body=traceback.format_exc())
         trans_time = time.time() - start_time
+        res.fix_conditional_response()
         if self.log_requests:
             log_line = get_log_line(req, res, trans_time, '')
             if req.method in ('REPLICATE', 'SSYNC') or \
