@@ -17,6 +17,7 @@ import hashlib
 import json
 import os
 import random
+import sys
 import socket
 import time
 
@@ -122,24 +123,47 @@ class Connection(object):
         self.username = config['username']
         self.password = config['password']
 
-        self.storage_host = None
-        self.storage_port = None
-        self.storage_url = None
-
+        self.storage_netloc = None
+        self.storage_path = None
         self.conn_class = None
 
+    @property
+    def storage_url(self):
+        return '%s://%s/%s' % (self.storage_scheme, self.storage_netloc,
+                               self.storage_path)
+
+    @storage_url.setter
+    def storage_url(self, value):
+        url = urllib.parse.urlparse(value)
+
+        if url.scheme == 'http':
+            self.conn_class = http_client.HTTPConnection
+        elif url.scheme == 'https':
+            self.conn_class = http_client.HTTPSConnection
+        else:
+            raise ValueError('unexpected protocol %s' % (url.scheme))
+
+        self.storage_netloc = url.netloc
+        # Make sure storage_path is a string and not unicode, since
+        # keystoneclient (called by swiftclient) returns them in
+        # unicode and this would cause troubles when doing
+        # no_safe_quote query.
+        x = url.path.split('/')
+        self.storage_path = str('/%s/%s' % (x[1], x[2]))
+        self.account_name = str(x[2])
+
+    @property
+    def storage_scheme(self):
+        if self.conn_class is None:
+            return None
+        if issubclass(self.conn_class, http_client.HTTPSConnection):
+            return 'https'
+        return 'http'
+
     def get_account(self):
         return Account(self, self.account)
 
-    def authenticate(self, clone_conn=None):
-        if clone_conn:
-            self.conn_class = clone_conn.conn_class
-            self.storage_host = clone_conn.storage_host
-            self.storage_url = clone_conn.storage_url
-            self.storage_port = clone_conn.storage_port
-            self.storage_token = clone_conn.storage_token
-            return
-
+    def authenticate(self):
         if self.auth_version == "1":
             auth_path = '%sv1.0' % (self.auth_prefix)
             if self.account:
@@ -153,6 +177,16 @@ class Connection(object):
         auth_netloc = "%s:%d" % (self.auth_host, self.auth_port)
         auth_url = auth_scheme + auth_netloc + auth_path
 
+        if self.insecure:
+            try:
+                import requests
+                from requests.packages.urllib3.exceptions import \
+                    InsecureRequestWarning
+            except ImportError:
+                pass
+            else:
+                requests.packages.urllib3.disable_warnings(
+                    InsecureRequestWarning)
         authargs = dict(snet=False, tenant_name=self.account,
                         auth_version=self.auth_version, os_options={},
                         insecure=self.insecure)
@@ -162,26 +196,7 @@ class Connection(object):
         if not (storage_url and storage_token):
             raise AuthenticationFailed()
 
-        x = storage_url.split('/')
-
-        if x[0] == 'http:':
-            self.conn_class = http_client.HTTPConnection
-            self.storage_port = 80
-        elif x[0] == 'https:':
-            self.conn_class = http_client.HTTPSConnection
-            self.storage_port = 443
-        else:
-            raise ValueError('unexpected protocol %s' % (x[0]))
-
-        self.storage_host = x[2].split(':')[0]
-        if ':' in x[2]:
-            self.storage_port = int(x[2].split(':')[1])
-        # Make sure storage_url is a string and not unicode, since
-        # keystoneclient (called by swiftclient) returns them in
-        # unicode and this would cause troubles when doing
-        # no_safe_quote query.
-        self.storage_url = str('/%s/%s' % (x[3], x[4]))
-        self.account_name = str(x[4])
+        self.storage_url = storage_url
         self.auth_user = auth_user
         # With v2 keystone, storage_token is unicode.
         # We want it to be string otherwise this would cause
@@ -191,7 +206,7 @@ class Connection(object):
         self.user_acl = '%s:%s' % (self.account, self.username)
 
         self.http_connect()
-        return self.storage_url, self.storage_token
+        return self.storage_path, self.storage_token
 
     def cluster_info(self):
         """
@@ -206,9 +221,14 @@ class Connection(object):
         return json.loads(self.response.read())
 
     def http_connect(self):
-        self.connection = self.conn_class(self.storage_host,
-                                          port=self.storage_port)
-        # self.connection.set_debuglevel(3)
+        if self.storage_scheme == 'https' and \
+                self.insecure and sys.version_info >= (2, 7, 9):
+            import ssl
+            self.connection = self.conn_class(
+                self.storage_netloc,
+                context=ssl._create_unverified_context())
+        else:
+            self.connection = self.conn_class(self.storage_netloc)
 
     def make_path(self, path=None, cfg=None):
         if path is None:
@@ -217,16 +237,16 @@ class Connection(object):
             cfg = {}
 
         if cfg.get('version_only_path'):
-            return '/' + self.storage_url.split('/')[1]
+            return '/' + self.storage_path.split('/')[1]
 
         if path:
             quote = urllib.parse.quote
             if cfg.get('no_quote') or cfg.get('no_path_quote'):
                 quote = lambda x: x
-            return '%s/%s' % (self.storage_url,
+            return '%s/%s' % (self.storage_path,
                               '/'.join([quote(i) for i in path]))
         else:
-            return self.storage_url
+            return self.storage_path
 
     def make_headers(self, hdrs, cfg=None):
         if cfg is None:
@@ -335,9 +355,6 @@ class Connection(object):
                           for (x, y) in parms.items()]
             path = '%s?%s' % (path, '&'.join(query_args))
 
-        self.connection = self.conn_class(self.storage_host,
-                                          port=self.storage_port)
-        # self.connection.set_debuglevel(3)
         self.connection.putrequest('PUT', path)
         for key, value in headers.items():
             self.connection.putheader(key, value)
@@ -354,6 +371,8 @@ class Connection(object):
             self.connection.send('0\r\n\r\n')
 
         self.response = self.connection.getresponse()
+        # Hope it isn't big!
+        self.response.body = self.response.read()
         self.connection.close()
         return self.response.status
 
@@ -366,26 +385,39 @@ class Base(object):
         if optional_fields is None:
             optional_fields = ()
 
+        def is_int_header(header):
+            if header.startswith('x-account-storage-policy-') and \
+                    header.endswith(('-bytes-used', '-object-count')):
+                return True
+            return header in (
+                'content-length',
+                'x-account-container-count',
+                'x-account-object-count',
+                'x-account-bytes-used',
+                'x-container-object-count',
+                'x-container-bytes-used',
+            )
+
         headers = dict(self.conn.response.getheaders())
         ret = {}
 
-        for field in required_fields:
-            if field[1] not in headers:
+        for return_key, header in required_fields:
+            if header not in headers:
                 raise ValueError("%s was not found in response header" %
-                                 (field[1]))
+                                 (header,))
 
-            try:
-                ret[field[0]] = int(headers[field[1]])
-            except ValueError:
-                ret[field[0]] = headers[field[1]]
+            if is_int_header(header):
+                ret[return_key] = int(headers[header])
+            else:
+                ret[return_key] = headers[header]
 
-        for field in optional_fields:
-            if field[1] not in headers:
+        for return_key, header in optional_fields:
+            if header not in headers:
                 continue
-            try:
-                ret[field[0]] = int(headers[field[1]])
-            except ValueError:
-                ret[field[0]] = headers[field[1]]
+            if is_int_header(header):
+                ret[return_key] = int(headers[header])
+            else:
+                ret[return_key] = headers[header]
 
         return ret
 
@@ -439,7 +471,7 @@ class Account(Base):
                 tree = minidom.parseString(self.conn.response.read())
                 for x in tree.getElementsByTagName('container'):
                     cont = {}
-                    for key in ['name', 'count', 'bytes']:
+                    for key in ['name', 'count', 'bytes', 'last_modified']:
                         cont[key] = x.getElementsByTagName(key)[0].\
                             childNodes[0].nodeValue
                     conts.append(cont)
@@ -482,8 +514,11 @@ class Account(Base):
         fields = [['object_count', 'x-account-object-count'],
                   ['container_count', 'x-account-container-count'],
                   ['bytes_used', 'x-account-bytes-used']]
+        optional_fields = [
+            ['temp-url-key', 'x-account-meta-temp-url-key'],
+            ['temp-url-key-2', 'x-account-meta-temp-url-key-2']]
 
-        return self.header_fields(fields)
+        return self.header_fields(fields, optional_fields=optional_fields)
 
     @property
     def path(self):
@@ -534,7 +569,7 @@ class Container(Base):
     def delete_files(self):
         for f in listing_items(self.files):
             file_item = self.file(f)
-            if not file_item.delete():
+            if not file_item.delete(tolerate_missing=True):
                 return False
 
         return listing_empty(self.files)
@@ -565,27 +600,38 @@ class Container(Base):
                 files = json.loads(self.conn.response.read())
 
                 for file_item in files:
-                    file_item['name'] = file_item['name'].encode('utf-8')
-                    file_item['content_type'] = file_item['content_type'].\
-                        encode('utf-8')
+                    for key in ('name', 'subdir', 'content_type'):
+                        if key in file_item:
+                            file_item[key] = file_item[key].encode('utf-8')
                 return files
             elif format_type == 'xml':
                 files = []
                 tree = minidom.parseString(self.conn.response.read())
-                for x in tree.getElementsByTagName('object'):
+                container = tree.getElementsByTagName('container')[0]
+                for x in container.childNodes:
                     file_item = {}
-                    for key in ['name', 'hash', 'bytes', 'content_type',
-                                'last_modified']:
-
-                        file_item[key] = x.getElementsByTagName(key)[0].\
-                            childNodes[0].nodeValue
+                    if x.tagName == 'object':
+                        for key in ['name', 'hash', 'bytes', 'content_type',
+                                    'last_modified']:
+                            file_item[key] = x.getElementsByTagName(key)[0].\
+                                childNodes[0].nodeValue
+                    elif x.tagName == 'subdir':
+                        file_item['subdir'] = x.getElementsByTagName(
+                            'name')[0].childNodes[0].nodeValue
+                    else:
+                        raise ValueError('Found unexpected element %s'
+                                         % x.tagName)
                     files.append(file_item)
 
                 for file_item in files:
-                    file_item['name'] = file_item['name'].encode('utf-8')
-                    file_item['content_type'] = file_item['content_type'].\
-                        encode('utf-8')
-                    file_item['bytes'] = int(file_item['bytes'])
+                    if 'subdir' in file_item:
+                        file_item['subdir'] = file_item['subdir'].\
+                            encode('utf-8')
+                    else:
+                        file_item['name'] = file_item['name'].encode('utf-8')
+                        file_item['content_type'] = file_item['content_type'].\
+                            encode('utf-8')
+                        file_item['bytes'] = int(file_item['bytes'])
                 return files
             else:
                 content = self.conn.response.read()
@@ -614,9 +660,15 @@ class Container(Base):
 
         if self.conn.response.status == 204:
             required_fields = [['bytes_used', 'x-container-bytes-used'],
-                               ['object_count', 'x-container-object-count']]
+                               ['object_count', 'x-container-object-count'],
+                               ['last_modified', 'last-modified']]
             optional_fields = [
+                # N.B. swift doesn't return both x-versions-location
+                # and x-history-location at a response so that this is safe
+                # using same variable "versions" for both and it means
+                # versioning is enabled.
                 ['versions', 'x-versions-location'],
+                ['versions', 'x-history-location'],
                 ['tempurl_key', 'x-container-meta-temp-url-key'],
                 ['tempurl_key2', 'x-container-meta-temp-url-key-2']]
 
@@ -703,8 +755,11 @@ class File(Base):
         if 'Destination' in headers:
             headers['Destination'] = urllib.parse.quote(headers['Destination'])
 
-        return self.conn.make_request('COPY', self.path, hdrs=headers,
-                                      parms=parms) == 201
+        if self.conn.make_request('COPY', self.path, hdrs=headers,
+                                  cfg=cfg, parms=parms) != 201:
+            raise ResponseError(self.conn.response, 'COPY',
+                                self.conn.make_path(self.path))
+        return True
 
     def copy_account(self, dest_account, dest_cont, dest_file,
                      hdrs=None, parms=None, cfg=None):
@@ -729,17 +784,25 @@ class File(Base):
         if 'Destination' in headers:
             headers['Destination'] = urllib.parse.quote(headers['Destination'])
 
-        return self.conn.make_request('COPY', self.path, hdrs=headers,
-                                      parms=parms) == 201
+        if self.conn.make_request('COPY', self.path, hdrs=headers,
+                                  cfg=cfg, parms=parms) != 201:
+            raise ResponseError(self.conn.response, 'COPY',
+                                self.conn.make_path(self.path))
+        return True
 
-    def delete(self, hdrs=None, parms=None, cfg=None):
+    def delete(self, hdrs=None, parms=None, cfg=None, tolerate_missing=False):
         if hdrs is None:
             hdrs = {}
         if parms is None:
             parms = {}
-        if self.conn.make_request('DELETE', self.path, hdrs=hdrs,
-                                  cfg=cfg, parms=parms) != 204:
+        if tolerate_missing:
+            allowed_statuses = (204, 404)
+        else:
+            allowed_statuses = (204,)
 
+        if self.conn.make_request(
+                'DELETE', self.path, hdrs=hdrs, cfg=cfg,
+                parms=parms) not in allowed_statuses:
             raise ResponseError(self.conn.response, 'DELETE',
                                 self.conn.make_path(self.path))
 
@@ -762,11 +825,12 @@ class File(Base):
                   ['content_type', 'content-type'],
                   ['last_modified', 'last-modified'],
                   ['etag', 'etag']]
-        optional_fields = [['x_object_manifest', 'x-object-manifest']]
+        optional_fields = [['x_object_manifest', 'x-object-manifest'],
+                           ['x_symlink_target', 'x-symlink-target']]
 
         header_fields = self.header_fields(fields,
                                            optional_fields=optional_fields)
-        header_fields['etag'] = header_fields['etag'].strip('"')
+        header_fields['etag'] = header_fields['etag']
         return header_fields
 
     def initialize(self, hdrs=None, parms=None):
@@ -791,7 +855,7 @@ class File(Base):
             if hdr[0].lower().startswith('x-object-meta-'):
                 self.metadata[hdr[0][14:]] = hdr[1]
             if hdr[0].lower() == 'etag':
-                self.etag = hdr[1].strip('"')
+                self.etag = hdr[1]
             if hdr[0].lower() == 'content-length':
                 self.size = int(hdr[1])
             if hdr[0].lower() == 'last-modified':
@@ -882,12 +946,10 @@ class File(Base):
             fobj.close()
 
     def sync_metadata(self, metadata=None, cfg=None, parms=None):
-        if metadata is None:
-            metadata = {}
         if cfg is None:
             cfg = {}
 
-        self.metadata.update(metadata)
+        self.metadata = self.metadata if metadata is None else metadata
 
         if self.metadata:
             headers = self.make_headers(cfg=cfg)
