@@ -17,11 +17,10 @@
 
 import json
 import unittest2
-from unittest2 import SkipTest
 from uuid import uuid4
 
 from test.functional import check_response, cluster_info, retry, \
-    requires_acls, load_constraint, requires_policies
+    requires_acls, load_constraint, requires_policies, SkipTest
 import test.functional as tf
 
 from six.moves import range
@@ -69,9 +68,9 @@ class TestContainer(unittest2.TestCase):
             return check_response(conn)
 
         def delete(url, token, parsed, conn, container, obj):
-            conn.request(
-                'DELETE', '/'.join([parsed.path, container, obj['name']]), '',
-                {'X-Auth-Token': token})
+            path = '/'.join([parsed.path, container,
+                             obj['name'].encode('utf8')])
+            conn.request('DELETE', path, '', {'X-Auth-Token': token})
             return check_response(conn)
 
         for container in (self.name, self.container):
@@ -438,6 +437,9 @@ class TestContainer(unittest2.TestCase):
         if tf.skip:
             raise SkipTest
 
+        if tf.in_process:
+            tf.skip_if_no_xattrs()
+
         def post(url, token, parsed, conn, extra_headers):
             headers = {'X-Auth-Token': token}
             headers.update(extra_headers)
@@ -580,6 +582,9 @@ class TestContainer(unittest2.TestCase):
     def test_cross_account_public_container(self):
         if tf.skip or tf.skip2:
             raise SkipTest
+
+        if tf.in_process:
+            tf.skip_if_no_xattrs()
         # Obtain the first account's string
         first_account = ['unknown']
 
@@ -627,7 +632,7 @@ class TestContainer(unittest2.TestCase):
         resp.read()
         self.assertEqual(resp.status, 403)
 
-        # Now make the container also writeable by the second account
+        # Now make the container also writable by the second account
         def post(url, token, parsed, conn):
             conn.request('POST', parsed.path + '/' + self.name, '',
                          {'X-Auth-Token': token,
@@ -649,6 +654,9 @@ class TestContainer(unittest2.TestCase):
     def test_nonadmin_user(self):
         if tf.skip or tf.skip3:
             raise SkipTest
+
+        if tf.in_process:
+            tf.skip_if_no_xattrs()
         # Obtain the first account's string
         first_account = ['unknown']
 
@@ -696,7 +704,7 @@ class TestContainer(unittest2.TestCase):
         resp.read()
         self.assertEqual(resp.status, 403)
 
-        # Now make the container also writeable by the third account
+        # Now make the container also writable by the third account
         def post(url, token, parsed, conn):
             conn.request('POST', parsed.path + '/' + self.name, '',
                          {'X-Auth-Token': token,
@@ -1562,6 +1570,9 @@ class TestContainer(unittest2.TestCase):
         if 'container_quotas' not in cluster_info:
             raise SkipTest('Container quotas not enabled')
 
+        if tf.in_process:
+            tf.skip_if_no_xattrs()
+
         def post(url, token, parsed, conn, name, value):
             conn.request('POST', parsed.path + '/' + self.name, '',
                          {'X-Auth-Token': token, name: value})
