@@ -16,14 +16,17 @@
 import json
 import mock
 import os
+import random
 import re
 import tempfile
 import time
 import unittest
 import shutil
+import string
 import sys
+import six
 
-from eventlet.green import urllib2, socket
+from eventlet.green import socket
 from six import StringIO
 from six.moves import urllib
 
@@ -34,6 +37,13 @@ from swift.common.ring import utils as ring_utils
 from swift.common.storage_policy import StoragePolicy, POLICIES
 from test.unit import patch_policies
 
+if six.PY3:
+    from eventlet.green.urllib import request as urllib2
+    GREEN_URLLIB_URLOPEN = 'eventlet.green.urllib.request.urlopen'
+else:
+    from eventlet.green import urllib2
+    GREEN_URLLIB_URLOPEN = 'eventlet.green.urllib2.urlopen'
+
 
 class TestHelpers(unittest.TestCase):
     def test_seconds2timeunit(self):
@@ -60,7 +70,7 @@ class TestScout(unittest.TestCase):
         self.url = 'http://127.0.0.1:8080/recon/type'
         self.server_type_url = 'http://127.0.0.1:8080/'
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_ok(self, mock_urlopen):
         mock_urlopen.return_value.read = lambda: json.dumps([])
         url, content, status, ts_start, ts_end = self.scout_instance.scout(
@@ -69,7 +79,7 @@ class TestScout(unittest.TestCase):
         self.assertEqual(content, [])
         self.assertEqual(status, 200)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_url_error(self, mock_urlopen):
         mock_urlopen.side_effect = urllib2.URLError("")
         url, content, status, ts_start, ts_end = self.scout_instance.scout(
@@ -78,7 +88,7 @@ class TestScout(unittest.TestCase):
         self.assertEqual(url, self.url)
         self.assertEqual(status, -1)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_http_error(self, mock_urlopen):
         mock_urlopen.side_effect = urllib2.HTTPError(
             self.url, 404, "Internal error", None, None)
@@ -88,7 +98,7 @@ class TestScout(unittest.TestCase):
         self.assertIsInstance(content, urllib2.HTTPError)
         self.assertEqual(status, 404)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_socket_timeout(self, mock_urlopen):
         mock_urlopen.side_effect = socket.timeout("timeout")
         url, content, status, ts_start, ts_end = self.scout_instance.scout(
@@ -97,19 +107,19 @@ class TestScout(unittest.TestCase):
         self.assertEqual(url, self.url)
         self.assertEqual(status, -1)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_server_type_ok(self, mock_urlopen):
         def getheader(name):
             d = {'Server': 'server-type'}
             return d.get(name)
-        mock_urlopen.return_value.info.return_value.getheader = getheader
+        mock_urlopen.return_value.info.return_value.get = getheader
         url, content, status = self.scout_instance.scout_server_type(
             ("127.0.0.1", "8080"))
         self.assertEqual(url, self.server_type_url)
         self.assertEqual(content, 'server-type')
         self.assertEqual(status, 200)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_server_type_url_error(self, mock_urlopen):
         mock_urlopen.side_effect = urllib2.URLError("")
         url, content, status = self.scout_instance.scout_server_type(
@@ -118,7 +128,7 @@ class TestScout(unittest.TestCase):
         self.assertEqual(url, self.server_type_url)
         self.assertEqual(status, -1)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_server_type_http_error(self, mock_urlopen):
         mock_urlopen.side_effect = urllib2.HTTPError(
             self.server_type_url, 404, "Internal error", None, None)
@@ -128,7 +138,7 @@ class TestScout(unittest.TestCase):
         self.assertIsInstance(content, urllib2.HTTPError)
         self.assertEqual(status, 404)
 
-    @mock.patch('eventlet.green.urllib2.urlopen')
+    @mock.patch(GREEN_URLLIB_URLOPEN)
     def test_scout_server_type_socket_timeout(self, mock_urlopen):
         mock_urlopen.side_effect = socket.timeout("timeout")
         url, content, status = self.scout_instance.scout_server_type(
@@ -141,6 +151,7 @@ class TestScout(unittest.TestCase):
 @patch_policies
 class TestRecon(unittest.TestCase):
     def setUp(self, *_args, **_kwargs):
+        self.swift_conf_file = utils.SWIFT_CONF_FILE
         self.recon_instance = recon.SwiftRecon()
         self.swift_dir = tempfile.mkdtemp()
         self.ring_name = POLICIES.legacy.ring_name
@@ -150,10 +161,25 @@ class TestRecon(unittest.TestCase):
         self.tmpfile_name2 = os.path.join(
             self.swift_dir, self.ring_name2 + '.ring.gz')
 
-        utils.HASH_PATH_SUFFIX = 'endcap'
-        utils.HASH_PATH_PREFIX = 'startcap'
+        swift_conf = os.path.join(self.swift_dir, 'swift.conf')
+        self.policy_name = ''.join(random.sample(string.ascii_letters, 20))
+        swift_conf_data = '''
+[swift-hash]
+swift_hash_path_suffix = changeme
+
+[storage-policy:0]
+name = default
+default = yes
+
+[storage-policy:1]
+name = unu
+aliases = %s
+''' % self.policy_name
+        with open(swift_conf, "wb") as sc:
+            sc.write(swift_conf_data.encode('utf8'))
 
     def tearDown(self, *_args, **_kwargs):
+        utils.SWIFT_CONF_FILE = self.swift_conf_file
         shutil.rmtree(self.swift_dir, ignore_errors=True)
 
     def _make_object_rings(self):
@@ -257,6 +283,26 @@ class TestRecon(unittest.TestCase):
         self.assertEqual(set([('127.0.0.1', 10001),
                               ('127.0.0.2', 10004)]), ips)
 
+    def test_get_error_ringnames(self):
+        # create invalid ring name files
+        invalid_ring_file_names = ('object.sring.gz',
+                                   'object-1.sring.gz',
+                                   'broken')
+        for invalid_ring in invalid_ring_file_names:
+            ring_path = os.path.join(self.swift_dir, invalid_ring)
+            with open(ring_path, 'w'):
+                pass
+
+        hosts = [("127.0.0.1", "8080")]
+        self.recon_instance.verbose = True
+        self.recon_instance.server_type = 'object'
+        stdout = StringIO()
+        with mock.patch('sys.stdout', new=stdout), \
+                mock.patch('swift.common.utils.md5'):
+            self.recon_instance.get_ringmd5(hosts, self.swift_dir)
+        output = stdout.getvalue()
+        self.assertNotIn('On disk ', output)
+
     def test_get_ringmd5(self):
         for server_type in ('account', 'container', 'object', 'object-1'):
             ring_name = '%s.ring.gz' % server_type
@@ -264,6 +310,7 @@ class TestRecon(unittest.TestCase):
             open(ring_file, 'w')
 
         empty_file_hash = 'd41d8cd98f00b204e9800998ecf8427e'
+        bad_file_hash = '00000000000000000000000000000000'
         hosts = [("127.0.0.1", "8080")]
         with mock.patch('swift.cli.recon.Scout') as mock_scout:
             scout_instance = mock.MagicMock()
@@ -277,10 +324,62 @@ class TestRecon(unittest.TestCase):
             status = 200
             scout_instance.scout.return_value = (url, response, status, 0, 0)
             mock_scout.return_value = scout_instance
-            stdout = StringIO()
             mock_hash = mock.MagicMock()
+
+            # Check correct account, container and object ring hashes
+            for server_type in ('account', 'container', 'object'):
+                self.recon_instance.server_type = server_type
+                stdout = StringIO()
+                with mock.patch('sys.stdout', new=stdout), \
+                        mock.patch('swift.common.utils.md5', new=mock_hash):
+                    mock_hash.return_value.hexdigest.return_value = \
+                        empty_file_hash
+                    self.recon_instance.get_ringmd5(hosts, self.swift_dir)
+                output = stdout.getvalue()
+                expected = '1/1 hosts matched'
+                found = False
+                for line in output.splitlines():
+                    if '!!' in line:
+                        self.fail('Unexpected Error in output: %r' % line)
+                    if expected in line:
+                        found = True
+                if not found:
+                    self.fail('Did not find expected substring %r '
+                              'in output:\n%s' % (expected, output))
+
+            # Check bad container ring hash
+            self.recon_instance.server_type = 'container'
+            response = {
+                '/etc/swift/account.ring.gz': empty_file_hash,
+                '/etc/swift/container.ring.gz': bad_file_hash,
+                '/etc/swift/object.ring.gz': empty_file_hash,
+                '/etc/swift/object-1.ring.gz': empty_file_hash,
+            }
+            scout_instance.scout.return_value = (url, response, status, 0, 0)
+            mock_scout.return_value = scout_instance
+            stdout = StringIO()
             with mock.patch('sys.stdout', new=stdout), \
-                    mock.patch('swift.cli.recon.md5', new=mock_hash):
+                    mock.patch('swift.common.utils.md5', new=mock_hash):
+                mock_hash.return_value.hexdigest.return_value = \
+                    empty_file_hash
+                self.recon_instance.get_ringmd5(hosts, self.swift_dir)
+            output = stdout.getvalue()
+            expected = '0/1 hosts matched'
+            found = False
+            for line in output.splitlines():
+                if '!!' in line:
+                    self.assertIn('doesn\'t match on disk md5sum', line)
+                if expected in line:
+                    found = True
+            if not found:
+                self.fail('Did not find expected substring %r '
+                          'in output:\n%s' % (expected, output))
+
+            # Check object ring, container mismatch should be ignored
+            self.recon_instance.server_type = 'object'
+            stdout = StringIO()
+            with mock.patch('sys.stdout', new=stdout), \
+                    mock.patch('swift.common.utils.md5', new=mock_hash):
                 mock_hash.return_value.hexdigest.return_value = \
                     empty_file_hash
                 self.recon_instance.get_ringmd5(hosts, self.swift_dir)
@@ -290,11 +389,13 @@ class TestRecon(unittest.TestCase):
                 if '!!' in line:
                     self.fail('Unexpected Error in output: %r' % line)
                 if expected in line:
-                    break
-            else:
+                    found = True
+            if not found:
                 self.fail('Did not find expected substring %r '
                           'in output:\n%s' % (expected, output))
 
+        # Cleanup
+        self.recon_instance.server_type = 'object'
         for ring in ('account', 'container', 'object', 'object-1'):
             os.remove(os.path.join(self.swift_dir, "%s.ring.gz" % ring))
 
@@ -413,7 +514,7 @@ class TestRecon(unittest.TestCase):
             self.recon_instance.umount_check(hosts)
 
         output = stdout.getvalue()
-        r = re.compile("\Not mounted:|Device errors: .*")
+        r = re.compile("^Not mounted:|Device errors: .*")
         lines = output.splitlines()
         self.assertTrue(lines)
         for line in lines:
@@ -509,7 +610,7 @@ class TestRecon(unittest.TestCase):
 
         self.assertEqual(expected, discovered_hosts)
 
-    def test_main_object_hosts_default_unu(self):
+    def _test_main_object_hosts_policy_name(self, policy_name='unu'):
         self._make_object_rings()
         discovered_hosts = set()
 
@@ -521,7 +622,7 @@ class TestRecon(unittest.TestCase):
 
         with mock.patch.object(sys, 'argv', [
                 "prog", "object", "--swiftdir=%s" % self.swift_dir,
-                "--validate-servers", '--policy=unu']):
+                "--validate-servers", '--policy', policy_name]):
 
             self.recon_instance.main()
 
@@ -531,6 +632,12 @@ class TestRecon(unittest.TestCase):
         ])
         self.assertEqual(expected, discovered_hosts)
 
+    def test_main_object_hosts_default_unu(self):
+        self._test_main_object_hosts_policy_name()
+
+    def test_main_object_hosts_default_alias(self):
+        self._test_main_object_hosts_policy_name(self.policy_name)
+
     def test_main_object_hosts_default_invalid(self):
         self._make_object_rings()
         stdout = StringIO()
@@ -564,10 +671,11 @@ class TestReconCommands(unittest.TestCase):
             response_body = resps[(host, port, path[7:])]
 
             resp = mock.MagicMock()
-            resp.read = mock.MagicMock(side_effect=[response_body])
+            resp.read = mock.MagicMock(side_effect=[
+                response_body if six.PY2 else response_body.encode('utf8')])
             return resp
 
-        return mock.patch('eventlet.green.urllib2.urlopen', fake_urlopen)
+        return mock.patch(GREEN_URLLIB_URLOPEN, fake_urlopen)
 
     def test_server_type_check(self):
         hosts = [('127.0.0.1', 6010), ('127.0.0.1', 6011),
@@ -668,7 +776,8 @@ class TestReconCommands(unittest.TestCase):
 
         printed = []
         with self.mock_responses(responses):
-            with mock.patch.object(self.recon, '_md5_file', lambda _: cksum):
+            with mock.patch('swift.cli.recon.md5_hash_for_file',
+                            lambda _: cksum):
                 self.recon.get_swiftconfmd5(hosts, printfn=printed.append)
 
         output = '\n'.join(printed) + '\n'
@@ -687,7 +796,8 @@ class TestReconCommands(unittest.TestCase):
 
         printed = []
         with self.mock_responses(responses):
-            with mock.patch.object(self.recon, '_md5_file', lambda _: cksum):
+            with mock.patch('swift.cli.recon.md5_hash_for_file',
+                            lambda _: cksum):
                 self.recon.get_swiftconfmd5(hosts, printfn=printed.append)
 
         output = '\n'.join(printed) + '\n'
@@ -764,7 +874,9 @@ class TestReconCommands(unittest.TestCase):
             mock.call('Disk usage: space used: 260 of 300'),
             mock.call('Disk usage: space free: 40 of 300'),
             mock.call('Disk usage: lowest: 85.0%, ' +
-                      'highest: 90.0%, avg: 86.6666666667%'),
+                      'highest: 90.0%%, avg: %s' %
+                      ('86.6666666667%' if six.PY2 else
+                       '86.66666666666667%')),
             mock.call('=' * 79),
         ]
 
@@ -894,12 +1006,12 @@ class TestReconCommands(unittest.TestCase):
 
         def dummy_request(*args, **kwargs):
             return [
-                ('http://127.0.0.1:6010/recon/load',
+                ('http://127.0.0.1:6010/recon/time',
                  now,
                  200,
                  now - 0.5,
                  now + 0.5),
-                ('http://127.0.0.1:6020/recon/load',
+                ('http://127.0.0.1:6020/recon/time',
                  now,
                  200,
                  now,
@@ -944,7 +1056,7 @@ class TestReconCommands(unittest.TestCase):
         default_calls = [
             mock.call("!! http://127.0.0.1:6010/recon/time current time is "
                       "2015-04-25 22:13:21, but remote is "
-                      "2015-04-25 22:13:20, differs by 1.30 sec"),
+                      "2015-04-25 22:13:20, differs by 1.3000 sec"),
             mock.call('1/2 hosts matched, 0 error[s] while checking hosts.'),
         ]
 
@@ -954,6 +1066,136 @@ class TestReconCommands(unittest.TestCase):
         # that is returned from the recon middleware, thus can't rely on it
         mock_print.assert_has_calls(default_calls, any_order=True)
 
+    @mock.patch('six.moves.builtins.print')
+    @mock.patch('time.time')
+    def test_time_check_jitter(self, mock_now, mock_print):
+        now = 1430000000.0
+        mock_now.return_value = now
+
+        def dummy_request(*args, **kwargs):
+            return [
+                ('http://127.0.0.1:6010/recon/time',
+                 now - 2,
+                 200,
+                 now,
+                 now + 3),
+                ('http://127.0.0.1:6020/recon/time',
+                 now + 2,
+                 200,
+                 now - 3,
+                 now),
+            ]
+
+        cli = recon.SwiftRecon()
+        cli.pool.imap = dummy_request
+
+        default_calls = [
+            mock.call('2/2 hosts matched, 0 error[s] while checking hosts.')
+        ]
+
+        cli.time_check([('127.0.0.1', 6010), ('127.0.0.1', 6020)], 3)
+        # We need any_order=True because the order of calls depends on the dict
+        # that is returned from the recon middleware, thus can't rely on it
+        mock_print.assert_has_calls(default_calls, any_order=True)
+
+    @mock.patch('six.moves.builtins.print')
+    def test_version_check(self, mock_print):
+        version = "2.7.1.dev144"
+
+        def dummy_request(*args, **kwargs):
+            return [
+                ('http://127.0.0.1:6010/recon/version',
+                 {'version': version},
+                 200,
+                 0,
+                 0),
+                ('http://127.0.0.1:6020/recon/version',
+                 {'version': version},
+                 200,
+                 0,
+                 0),
+            ]
+
+        cli = recon.SwiftRecon()
+        cli.pool.imap = dummy_request
+
+        default_calls = [
+            mock.call("Versions matched (%s), "
+                      "0 error[s] while checking hosts." % version)
+        ]
+
+        cli.version_check([('127.0.0.1', 6010), ('127.0.0.1', 6020)])
+        # We need any_order=True because the order of calls depends on the dict
+        # that is returned from the recon middleware, thus can't rely on it
+        mock_print.assert_has_calls(default_calls, any_order=True)
+
+    @mock.patch('six.moves.builtins.print')
+    @mock.patch('time.time')
+    def test_time_check_jitter_mismatch(self, mock_now, mock_print):
+        now = 1430000000.0
+        mock_now.return_value = now
+
+        def dummy_request(*args, **kwargs):
+            return [
+                ('http://127.0.0.1:6010/recon/time',
+                 now - 4,
+                 200,
+                 now,
+                 now + 2),
+                ('http://127.0.0.1:6020/recon/time',
+                 now + 4,
+                 200,
+                 now - 2,
+                 now),
+            ]
+
+        cli = recon.SwiftRecon()
+        cli.pool.imap = dummy_request
+
+        default_calls = [
+            mock.call("!! http://127.0.0.1:6010/recon/time current time is "
+                      "2015-04-25 22:13:22, but remote is "
+                      "2015-04-25 22:13:16, differs by 6.0000 sec"),
+            mock.call("!! http://127.0.0.1:6020/recon/time current time is "
+                      "2015-04-25 22:13:20, but remote is "
+                      "2015-04-25 22:13:24, differs by 4.0000 sec"),
+            mock.call('0/2 hosts matched, 0 error[s] while checking hosts.'),
+        ]
+
+        cli.time_check([('127.0.0.1', 6010), ('127.0.0.1', 6020)], 3)
+        # We need any_order=True because the order of calls depends on the dict
+        # that is returned from the recon middleware, thus can't rely on it
+        mock_print.assert_has_calls(default_calls, any_order=True)
+
+    @mock.patch('six.moves.builtins.print')
+    def test_version_check_differs(self, mock_print):
+        def dummy_request(*args, **kwargs):
+            return [
+                ('http://127.0.0.1:6010/recon/version',
+                 {'version': "2.7.1.dev144"},
+                 200,
+                 0,
+                 0),
+                ('http://127.0.0.1:6020/recon/version',
+                 {'version': "2.7.1.dev145"},
+                 200,
+                 0,
+                 0),
+            ]
+
+        cli = recon.SwiftRecon()
+        cli.pool.imap = dummy_request
+
+        default_calls = [
+            mock.call("Versions not matched (2.7.1.dev144, 2.7.1.dev145), "
+                      "0 error[s] while checking hosts.")
+        ]
+
+        cli.version_check([('127.0.0.1', 6010), ('127.0.0.1', 6020)])
+        # We need any_order=True because the order of calls depends on the dict
+        # that is returned from the recon middleware, thus can't rely on it
+        mock_print.assert_has_calls(default_calls, any_order=True)
+
     @mock.patch('six.moves.builtins.print')
     @mock.patch('swift.cli.recon.SwiftRecon.get_hosts')
     def test_multiple_server_types(self, mock_get_hosts, mock_print):
