@@ -18,10 +18,10 @@ import hmac
 import json
 import os
 import unittest
-import urllib
 
 import mock
 
+from six.moves.urllib import parse as urlparse
 from swift.common.middleware.crypto import encrypter
 from swift.common.middleware.crypto.crypto_utils import (
     CRYPTO_KEY_CALLBACK, Crypto)
@@ -52,7 +52,7 @@ class TestEncrypter(unittest.TestCase):
         param = param.strip()
         self.assertTrue(param.startswith('swift_meta='))
         actual_meta = json.loads(
-            urllib.unquote_plus(param[len('swift_meta='):]))
+            urlparse.unquote_plus(param[len('swift_meta='):]))
         self.assertEqual(Crypto.cipher, actual_meta['cipher'])
         meta_iv = base64.b64decode(actual_meta['iv'])
         self.assertEqual(FAKE_IV, meta_iv)
@@ -61,7 +61,7 @@ class TestEncrypter(unittest.TestCase):
             enc_val)
         # if there is any encrypted user metadata then this header should exist
         self.assertIn('X-Object-Transient-Sysmeta-Crypto-Meta', req_hdrs)
-        common_meta = json.loads(urllib.unquote_plus(
+        common_meta = json.loads(urlparse.unquote_plus(
             req_hdrs['X-Object-Transient-Sysmeta-Crypto-Meta']))
         self.assertDictEqual({'cipher': Crypto.cipher,
                               'key_id': {'v': 'fake', 'path': '/a/c/fake'}},
@@ -101,7 +101,7 @@ class TestEncrypter(unittest.TestCase):
 
         # verify body crypto meta
         actual = req_hdrs['X-Object-Sysmeta-Crypto-Body-Meta']
-        actual = json.loads(urllib.unquote_plus(actual))
+        actual = json.loads(urlparse.unquote_plus(actual))
         self.assertEqual(Crypto().cipher, actual['cipher'])
         self.assertEqual(FAKE_IV, base64.b64decode(actual['iv']))
 
@@ -120,7 +120,7 @@ class TestEncrypter(unittest.TestCase):
             req_hdrs['X-Object-Sysmeta-Crypto-Etag'].partition('; swift_meta=')
         # verify crypto_meta was appended to this etag
         self.assertTrue(etag_meta)
-        actual_meta = json.loads(urllib.unquote_plus(etag_meta))
+        actual_meta = json.loads(urlparse.unquote_plus(etag_meta))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
 
         # verify encrypted version of plaintext etag
@@ -147,7 +147,7 @@ class TestEncrypter(unittest.TestCase):
         crypto_meta_tag = 'swift_meta='
         self.assertTrue(param.startswith(crypto_meta_tag), param)
         actual_meta = json.loads(
-            urllib.unquote_plus(param[len(crypto_meta_tag):]))
+            urlparse.unquote_plus(param[len(crypto_meta_tag):]))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
         self.assertEqual(fetch_crypto_keys()['id'], actual_meta['key_id'])
 
@@ -278,7 +278,7 @@ class TestEncrypter(unittest.TestCase):
         encrypted_etag, _junk, etag_meta = \
             req_hdrs['X-Object-Sysmeta-Crypto-Etag'].partition('; swift_meta=')
         self.assertTrue(etag_meta)
-        actual_meta = json.loads(urllib.unquote_plus(etag_meta))
+        actual_meta = json.loads(urlparse.unquote_plus(etag_meta))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
 
         self.assertEqual(ciphertext_etag, req_hdrs['Etag'])
@@ -298,7 +298,7 @@ class TestEncrypter(unittest.TestCase):
         crypto_meta_tag = 'swift_meta='
         self.assertTrue(param.startswith(crypto_meta_tag), param)
         actual_meta = json.loads(
-            urllib.unquote_plus(param[len(crypto_meta_tag):]))
+            urlparse.unquote_plus(param[len(crypto_meta_tag):]))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
 
         cont_key = fetch_crypto_keys()['container']
@@ -309,7 +309,7 @@ class TestEncrypter(unittest.TestCase):
 
         # verify body crypto meta
         actual = req_hdrs['X-Object-Sysmeta-Crypto-Body-Meta']
-        actual = json.loads(urllib.unquote_plus(actual))
+        actual = json.loads(urlparse.unquote_plus(actual))
         self.assertEqual(Crypto().cipher, actual['cipher'])
         self.assertEqual(FAKE_IV, base64.b64decode(actual['iv']))
 
@@ -368,7 +368,7 @@ class TestEncrypter(unittest.TestCase):
         crypto_meta_tag = 'swift_meta='
         self.assertTrue(param.startswith(crypto_meta_tag), param)
         actual_meta = json.loads(
-            urllib.unquote_plus(param[len(crypto_meta_tag):]))
+            urlparse.unquote_plus(param[len(crypto_meta_tag):]))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
         self.assertEqual(fetch_crypto_keys()['id'], actual_meta['key_id'])
 
@@ -491,7 +491,7 @@ class TestEncrypter(unittest.TestCase):
         crypto_meta_tag = 'swift_meta='
         self.assertTrue(param.startswith(crypto_meta_tag), param)
         actual_meta = json.loads(
-            urllib.unquote_plus(param[len(crypto_meta_tag):]))
+            urlparse.unquote_plus(param[len(crypto_meta_tag):]))
         self.assertEqual(Crypto().cipher, actual_meta['cipher'])
         self.assertEqual(fetch_crypto_keys()['id'], actual_meta['key_id'])
 
@@ -604,13 +604,20 @@ class TestEncrypter(unittest.TestCase):
             # verify etags have been supplemented with masked values
             self.assertIn(match_header_name, actual_headers)
             actual_etags = set(actual_headers[match_header_name].split(', '))
+            # masked values for secret_id None
             key = fetch_crypto_keys()['object']
             masked_etags = [
                 '"%s"' % base64.b64encode(hmac.new(
                     key, etag.strip('"'), hashlib.sha256).digest())
                 for etag in plain_etags if etag not in ('*', '')]
+            # masked values for secret_id myid
+            key = fetch_crypto_keys(key_id={'secret_id': 'myid'})['object']
+            masked_etags_myid = [
+                '"%s"' % base64.b64encode(hmac.new(
+                    key, etag.strip('"'), hashlib.sha256).digest())
+                for etag in plain_etags if etag not in ('*', '')]
             expected_etags = set((expected_plain_etags or plain_etags) +
-                                 masked_etags)
+                                 masked_etags + masked_etags_myid)
             self.assertEqual(expected_etags, actual_etags)
             # check that the request environ was returned to original state
             self.assertEqual(set(plain_etags),
@@ -798,7 +805,7 @@ class TestEncrypter(unittest.TestCase):
         self.assertEqual('Unable to retrieve encryption keys.', resp.body)
 
     def test_PUT_error_in_key_callback(self):
-        def raise_exc():
+        def raise_exc(*args, **kwargs):
             raise Exception('Testing')
 
         body = 'FAKE APP'
