@@ -15,11 +15,10 @@ import unittest
 
 from swift.common.swob import Request, wsgify, HTTPForbidden
 
-from swift.common.middleware import account_quotas
+from swift.common.middleware import account_quotas, copy
 
-from swift.proxy.controllers.base import _get_cache_key, \
-    headers_to_account_info, get_object_env_key, \
-    headers_to_object_info
+from swift.proxy.controllers.base import get_cache_key, \
+    headers_to_account_info, headers_to_object_info
 
 
 class FakeCache(object):
@@ -57,16 +56,18 @@ class FakeApp(object):
                 return aresp(env, start_response)
         if env['REQUEST_METHOD'] == "HEAD" and \
                 env['PATH_INFO'] == '/v1/a/c2/o2':
-            env_key = get_object_env_key('a', 'c2', 'o2')
-            env[env_key] = headers_to_object_info(self.headers, 200)
+            cache_key = get_cache_key('a', 'c2', 'o2')
+            env.setdefault('swift.infocache', {})[cache_key] = \
+                headers_to_object_info(self.headers, 200)
             start_response('200 OK', self.headers)
         elif env['REQUEST_METHOD'] == "HEAD" and \
                 env['PATH_INFO'] == '/v1/a/c2/o3':
             start_response('404 Not Found', [])
         else:
             # Cache the account_info (same as a real application)
-            cache_key, env_key = _get_cache_key('a', None)
-            env[env_key] = headers_to_account_info(self.headers, 200)
+            cache_key = get_cache_key('a')
+            env.setdefault('swift.infocache', {})[cache_key] = \
+                headers_to_account_info(self.headers, 200)
             start_response('200 OK', self.headers)
         return []
 
@@ -97,7 +98,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache})
         res = req.get_response(app)
         # Response code of 200 because authentication itself is not done here
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_no_quotas(self):
         headers = [('x-account-bytes-used', '1000'), ]
@@ -107,7 +108,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_obj_request_ignores_attempt_to_set_quotas(self):
         # If you try to set X-Account-Meta-* on an object, it's ignored, so
@@ -121,7 +122,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_container_request_ignores_attempt_to_set_quotas(self):
         # As with an object, if you try to set X-Account-Meta-* on a
@@ -134,7 +135,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_bogus_quota_is_ignored(self):
         # This can happen if the metadata was set by a user prior to the
@@ -147,7 +148,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -158,8 +159,8 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
-        self.assertEquals(res.body, 'Upload exceeds quota.')
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
 
     def test_exceed_quota_not_authorized(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -171,7 +172,7 @@ class TestAccountQuota(unittest.TestCase):
                             headers={'x-auth-token': 'bad-secret'},
                             environ={'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_exceed_quota_authorized(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -183,7 +184,7 @@ class TestAccountQuota(unittest.TestCase):
                             headers={'x-auth-token': 'secret'},
                             environ={'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
+        self.assertEqual(res.status_int, 413)
 
     def test_under_quota_not_authorized(self):
         headers = [('x-account-bytes-used', '0'),
@@ -195,7 +196,7 @@ class TestAccountQuota(unittest.TestCase):
                             headers={'x-auth-token': 'bad-secret'},
                             environ={'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_under_quota_authorized(self):
         headers = [('x-account-bytes-used', '0'),
@@ -207,7 +208,7 @@ class TestAccountQuota(unittest.TestCase):
                             headers={'x-auth-token': 'secret'},
                             environ={'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_container_create_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -219,7 +220,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_CONTAINER_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_container_post_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -231,7 +232,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_CONTAINER_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_over_quota_obj_post_still_works(self):
         headers = [('x-account-bytes-used', '1001'),
@@ -243,85 +244,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_OBJECT_META_BERT': 'ernie',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
-
-    def test_exceed_bytes_quota_copy_from(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
-        self.assertEquals(res.body, 'Upload exceeds quota.')
-
-    def test_exceed_bytes_quota_copy_verb(self):
-        headers = [('x-account-bytes-used', '500'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 413)
-        self.assertEquals(res.body, 'Upload exceeds quota.')
-
-    def test_not_exceed_bytes_quota_copy_from(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o2'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
-
-    def test_not_exceed_bytes_quota_copy_verb(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000'),
-                   ('content-length', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c2/o2',
-                            environ={'REQUEST_METHOD': 'COPY',
-                                     'swift.cache': cache},
-                            headers={'Destination': '/c/o'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
-
-    def test_quota_copy_from_no_src(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': '/c2/o3'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
-
-    def test_quota_copy_from_bad_src(self):
-        headers = [('x-account-bytes-used', '0'),
-                   ('x-account-meta-quota-bytes', '1000')]
-        app = account_quotas.AccountQuotaMiddleware(FakeApp(headers))
-        cache = FakeCache(None)
-        req = Request.blank('/v1/a/c/o',
-                            environ={'REQUEST_METHOD': 'PUT',
-                                     'swift.cache': cache},
-                            headers={'x-copy-from': 'bad_path'})
-        res = req.get_response(app)
-        self.assertEquals(res.status_int, 412)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -333,7 +256,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_from(self):
         headers = [('x-account-bytes-used', '500'),
@@ -347,7 +270,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True},
                             headers={'x-copy-from': 'c2/o2'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_exceed_bytes_quota_reseller_copy_verb(self):
         headers = [('x-account-bytes-used', '500'),
@@ -361,7 +284,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'reseller_request': True},
                             headers={'Destination': 'c/o'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_bad_application_quota(self):
         headers = []
@@ -371,7 +294,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 404)
+        self.assertEqual(res.status_int, 404)
 
     def test_no_info_quota(self):
         headers = []
@@ -381,7 +304,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_not_exceed_bytes_quota(self):
         headers = [('x-account-bytes-used', '1000'),
@@ -392,7 +315,7 @@ class TestAccountQuota(unittest.TestCase):
                             environ={'REQUEST_METHOD': 'PUT',
                                      'swift.cache': cache})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_invalid_quotas(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -404,7 +327,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': 'abc',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 400)
+        self.assertEqual(res.status_int, 400)
 
     def test_valid_quotas_admin(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -415,7 +338,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '100'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_valid_quotas_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -427,7 +350,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '100',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_delete_quotas(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -438,7 +361,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache,
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': ''})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_with_remove_header(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -449,7 +372,7 @@ class TestAccountQuota(unittest.TestCase):
             'swift.cache': cache,
             'HTTP_X_REMOVE_ACCOUNT_META_QUOTA_BYTES': 'True'})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 403)
+        self.assertEqual(res.status_int, 403)
 
     def test_delete_quotas_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -459,7 +382,7 @@ class TestAccountQuota(unittest.TestCase):
                                      'HTTP_X_ACCOUNT_META_QUOTA_BYTES': '',
                                      'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_delete_quotas_with_remove_header_reseller(self):
         headers = [('x-account-bytes-used', '0'), ]
@@ -471,7 +394,7 @@ class TestAccountQuota(unittest.TestCase):
             'HTTP_X_REMOVE_ACCOUNT_META_QUOTA_BYTES': 'True',
             'reseller_request': True})
         res = req.get_response(app)
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
     def test_invalid_request_exception(self):
         headers = [('x-account-bytes-used', '1000'), ]
@@ -482,8 +405,93 @@ class TestAccountQuota(unittest.TestCase):
                                      'swift.cache': cache})
         res = req.get_response(app)
         # Response code of 200 because authentication itself is not done here
-        self.assertEquals(res.status_int, 200)
+        self.assertEqual(res.status_int, 200)
 
 
+class AccountQuotaCopyingTestCases(unittest.TestCase):
+
+    def setUp(self):
+        self.app = FakeApp()
+        self.aq_filter = account_quotas.filter_factory({})(self.app)
+        self.copy_filter = copy.filter_factory({})(self.aq_filter)
+
+    def test_exceed_bytes_quota_copy_from(self):
+        headers = [('x-account-bytes-used', '500'),
+                   ('x-account-meta-quota-bytes', '1000'),
+                   ('content-length', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_exceed_bytes_quota_copy_verb(self):
+        headers = [('x-account-bytes-used', '500'),
+                   ('x-account-meta-quota-bytes', '1000'),
+                   ('content-length', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 413)
+        self.assertEqual(res.body, 'Upload exceeds quota.')
+
+    def test_not_exceed_bytes_quota_copy_from(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000'),
+                   ('content-length', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o2'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_not_exceed_bytes_quota_copy_verb(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000'),
+                   ('content-length', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                                     'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_quota_copy_from_no_src(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': '/c2/o3'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 200)
+
+    def test_quota_copy_from_bad_src(self):
+        headers = [('x-account-bytes-used', '0'),
+                   ('x-account-meta-quota-bytes', '1000')]
+        self.app.headers = headers
+        cache = FakeCache(None)
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                                     'swift.cache': cache},
+                            headers={'x-copy-from': 'bad_path'})
+        res = req.get_response(self.copy_filter)
+        self.assertEqual(res.status_int, 412)
+
 if __name__ == '__main__':
     unittest.main()
