@@ -62,7 +62,10 @@ class VersionedWritesBaseTestCase(unittest.TestCase):
         conf = {'allow_versioned_writes': 'true'}
         self.vw = versioned_writes.filter_factory(conf)(self.app)
 
-    def call_app(self, req, app=None, expect_exception=False):
+    def tearDown(self):
+        self.assertEqual(self.app.unclosed_requests, {})
+
+    def call_app(self, req, app=None):
         if app is None:
             app = self.app
 
@@ -84,24 +87,13 @@ class VersionedWritesBaseTestCase(unittest.TestCase):
             headers[0] = h
 
         body_iter = app(req.environ, start_response)
-        body = ''
-        caught_exc = None
-        try:
-            for chunk in body_iter:
-                body += chunk
-        except Exception as exc:
-            if expect_exception:
-                caught_exc = exc
-            else:
-                raise
+        with utils.closing_if_possible(body_iter):
+            body = b''.join(body_iter)
 
-        if expect_exception:
-            return status[0], headers[0], body, caught_exc
-        else:
-            return status[0], headers[0], body
+        return status[0], headers[0], body
 
-    def call_vw(self, req, **kwargs):
-        return self.call_app(req, app=self.vw, **kwargs)
+    def call_vw(self, req):
+        return self.call_app(req, app=self.vw)
 
     def assertRequestEqual(self, req, other):
         self.assertEqual(req.method, other.method)
@@ -338,23 +330,6 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
 
-    def test_put_object_post_as_copy(self):
-        # PUTs due to a post-as-copy should NOT cause a versioning op
-        self.app.register(
-            'PUT', '/v1/a/c/o', swob.HTTPCreated, {}, 'passed')
-
-        cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
-        req = Request.blank(
-            '/v1/a/c/o',
-            environ={'REQUEST_METHOD': 'PUT', 'swift.cache': cache,
-                     'CONTENT_LENGTH': '100',
-                     'swift.post_as_copy': True})
-        status, headers, body = self.call_vw(req)
-        self.assertEqual(status, '201 Created')
-        self.assertEqual(len(self.authorized), 1)
-        self.assertRequestEqual(req, self.authorized[0])
-        self.assertEqual(1, self.app.call_count)
-
     def test_put_first_object_success(self):
         self.app.register(
             'PUT', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
@@ -379,6 +354,30 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(['VW', None], self.app.swift_sources)
         self.assertEqual({'fake_trans_id'}, set(self.app.txn_ids))
 
+    def test_put_versioned_object_including_url_encoded_name_success(self):
+        self.app.register(
+            'PUT', '/v1/a/c/%ff', swob.HTTPOk, {}, 'passed')
+        self.app.register(
+            'GET', '/v1/a/c/%ff', swob.HTTPNotFound, {}, None)
+
+        cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
+        req = Request.blank(
+            '/v1/a/c/%25ff',
+            environ={'REQUEST_METHOD': 'PUT', 'swift.cache': cache,
+                     'CONTENT_LENGTH': '100',
+                     'swift.trans_id': 'fake_trans_id'})
+        status, headers, body = self.call_vw(req)
+        self.assertEqual(status, '200 OK')
+        self.assertEqual(len(self.authorized), 2)
+        # Versioned writes middleware now calls auth on the incoming request
+        # before we try the GET and then at the proxy, so there are 2
+        # atuhorized for the same request.
+        self.assertRequestEqual(req, self.authorized[0])
+        self.assertRequestEqual(req, self.authorized[1])
+        self.assertEqual(2, self.app.call_count)
+        self.assertEqual(['VW', None], self.app.swift_sources)
+        self.assertEqual({'fake_trans_id'}, set(self.app.txn_ids))
+
     def test_put_object_no_versioning_with_container_config_true(self):
         # set False to versions_write and expect no GET occurred
         self.vw.conf = {'allow_versioned_writes': 'false'}
@@ -397,27 +396,42 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertNotIn('GET', called_method)
 
     def test_put_request_is_dlo_manifest_with_container_config_true(self):
-        # set x-object-manifest on request and expect no versioning occurred
-        # only the PUT for the original client request
         self.app.register(
             'PUT', '/v1/a/c/o', swob.HTTPCreated, {}, 'passed')
+        self.app.register(
+            'GET', '/v1/a/c/o', swob.HTTPOk,
+            {'last-modified': 'Thu, 1 Jan 1970 00:01:00 GMT'}, 'old version')
+        self.app.register(
+            'PUT', '/v1/a/ver_cont/001o/0000000060.00000', swob.HTTPCreated,
+            {}, '')
         cache = FakeCache({'versions': 'ver_cont'})
         req = Request.blank(
             '/v1/a/c/o',
+            headers={'X-Object-Manifest': 'req/manifest'},
             environ={'REQUEST_METHOD': 'PUT', 'swift.cache': cache,
                      'CONTENT_LENGTH': '100'})
-        req.headers['X-Object-Manifest'] = 'req/manifest'
         status, headers, body = self.call_vw(req)
         self.assertEqual(status, '201 Created')
-        self.assertEqual(len(self.authorized), 1)
+        self.assertEqual(len(self.authorized), 2)
         self.assertRequestEqual(req, self.authorized[0])
-        self.assertEqual(1, self.app.call_count)
+        self.assertRequestEqual(req, self.authorized[1])
+        self.assertEqual(3, self.app.call_count)
+        self.assertEqual([
+            ('GET', '/v1/a/c/o'),
+            ('PUT', '/v1/a/ver_cont/001o/0000000060.00000'),
+            ('PUT', '/v1/a/c/o'),
+        ], self.app.calls)
+        self.assertIn('x-object-manifest',
+                      self.app.calls_with_headers[2].headers)
 
     def test_put_version_is_dlo_manifest_with_container_config_true(self):
-        # set x-object-manifest on response and expect no versioning occurred
-        # only initial GET on source object ok followed by PUT
         self.app.register('GET', '/v1/a/c/o', swob.HTTPOk,
-                          {'X-Object-Manifest': 'resp/manifest'}, 'passed')
+                          {'X-Object-Manifest': 'resp/manifest',
+                           'last-modified': 'Thu, 1 Jan 1970 01:00:00 GMT'},
+                          'passed')
+        self.app.register(
+            'PUT', '/v1/a/ver_cont/001o/0000003600.00000', swob.HTTPCreated,
+            {}, '')
         self.app.register(
             'PUT', '/v1/a/c/o', swob.HTTPCreated, {}, 'passed')
         cache = FakeCache({'versions': 'ver_cont'})
@@ -427,13 +441,20 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
                      'CONTENT_LENGTH': '100'})
         status, headers, body = self.call_vw(req)
         self.assertEqual(status, '201 Created')
-        # The middleware now auths the request before the inital GET, the
+        # The middleware now auths the request before the initial GET, the
         # same GET that gets the X-Object-Manifest back. So a second auth is
         # now done.
         self.assertEqual(len(self.authorized), 2)
         self.assertRequestEqual(req, self.authorized[0])
         self.assertRequestEqual(req, self.authorized[1])
-        self.assertEqual(2, self.app.call_count)
+        self.assertEqual(3, self.app.call_count)
+        self.assertEqual([
+            ('GET', '/v1/a/c/o'),
+            ('PUT', '/v1/a/ver_cont/001o/0000003600.00000'),
+            ('PUT', '/v1/a/c/o'),
+        ], self.app.calls)
+        self.assertIn('x-object-manifest',
+                      self.app.calls_with_headers[1].headers)
 
     def test_delete_object_no_versioning_with_container_config_true(self):
         # set False to versions_write obviously and expect no GET versioning
@@ -570,7 +591,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPNotFound, {}, None)
 
         cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
@@ -586,7 +607,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(['VW', None], self.app.swift_sources)
         self.assertEqual({'fake_trans_id'}, set(self.app.txn_ids))
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('DELETE', '/v1/a/c/o'),
@@ -597,7 +618,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {}, '[]')
 
         cache = FakeCache({'sysmeta': {'versions-location': 'ver_cont'}})
@@ -610,7 +631,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('DELETE', '/v1/a/c/o'),
@@ -619,7 +640,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
     def test_delete_latest_version_no_marker_success(self):
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -658,7 +679,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         req_headers = self.app.headers[-1]
         self.assertNotIn('x-if-delete-at', [h.lower() for h in req_headers])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', '/v1/a/ver_cont/001o/2'),
@@ -669,7 +690,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
     def test_delete_latest_version_restores_marker_success(self):
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "x", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -717,7 +738,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         # in the base versioned container.
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -752,7 +773,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('HEAD', '/v1/a/c/o'),
@@ -773,7 +794,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
 
     def test_delete_latest_version_doubled_up_markers_success(self):
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/'
+            'GET', '/v1/a/ver_cont?prefix=001o/'
             '&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "x", '
@@ -891,7 +912,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -917,7 +938,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(len(self.authorized), 1)
         self.assertRequestEqual(req, self.authorized[0])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', '/v1/a/ver_cont/001o/1'),
@@ -928,7 +949,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
     def test_DELETE_on_expired_versioned_object(self):
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -965,7 +986,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
         self.assertEqual(5, self.app.call_count)
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', '/v1/a/ver_cont/001o/2'),
@@ -978,7 +999,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         authorize_call = []
         self.app.register(
             'GET',
-            '/v1/a/ver_cont?format=json&prefix=001o/&marker=&reverse=on',
+            '/v1/a/ver_cont?prefix=001o/&marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "y", '
             '"last_modified": "2014-11-21T14:23:02.206740", '
@@ -1007,7 +1028,7 @@ class VersionedWritesTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(len(authorize_call), 1)
         self.assertRequestEqual(req, authorize_call[0])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
         ])
@@ -1044,7 +1065,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "x", '
@@ -1058,7 +1079,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
             '"name": "001o/2", '
             '"content_type": "text/plain"}]')
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/'
+            'GET', '/v1/a/ver_cont?prefix=001o/'
             '&marker=001o/2',
             swob.HTTPNotFound, {}, None)
         self.app.register(
@@ -1089,7 +1110,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         req_headers = self.app.headers[-1]
         self.assertNotIn('x-if-delete-at', [h.lower() for h in req_headers])
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', prefix_listing_prefix + 'marker=001o/2'),
@@ -1100,7 +1121,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
     def test_DELETE_on_expired_versioned_object(self):
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "x", '
@@ -1114,7 +1135,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
             '"name": "001o/2", '
             '"content_type": "text/plain"}]')
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/'
+            'GET', '/v1/a/ver_cont?prefix=001o/'
             '&marker=001o/2',
             swob.HTTPNotFound, {}, None)
 
@@ -1142,7 +1163,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.assertRequestEqual(req, self.authorized[0])
         self.assertEqual(6, self.app.call_count)
 
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', prefix_listing_prefix + 'marker=001o/2'),
@@ -1157,7 +1178,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.app.register(
             'DELETE', '/v1/a/c/o', swob.HTTPOk, {}, 'passed')
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&reverse=on',
             swob.HTTPOk, {},
             '[{"hash": "x", '
@@ -1171,7 +1192,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
             '"name": "001o/2", '
             '"content_type": "text/plain"}]')
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/'
+            'GET', '/v1/a/ver_cont?prefix=001o/'
             '&marker=001o/2',
             swob.HTTPNotFound, {}, None)
         self.app.register(
@@ -1192,7 +1213,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
         self.assertEqual(status, '403 Forbidden')
         self.assertEqual(len(authorize_call), 1)
         self.assertRequestEqual(req, authorize_call[0])
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', prefix_listing_prefix + 'marker=001o/2'),
@@ -1209,7 +1230,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         # first container server can reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&reverse=on',
             swob.HTTPOk, {}, json.dumps(list(reversed(old_versions[2:]))))
         # but all objects are already gone
@@ -1225,21 +1246,21 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         # second container server can't reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/2&reverse=on',
             swob.HTTPOk, {}, json.dumps(old_versions[3:]))
 
         # subsequent requests shouldn't reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&end_marker=001o/2',
             swob.HTTPOk, {}, json.dumps(old_versions[:1]))
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/0&end_marker=001o/2',
             swob.HTTPOk, {}, json.dumps(old_versions[1:2]))
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/1&end_marker=001o/2',
             swob.HTTPOk, {}, '[]')
         self.app.register(
@@ -1258,7 +1279,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
                      'CONTENT_LENGTH': '0'})
         status, headers, body = self.call_vw(req)
         self.assertEqual(status, '204 No Content')
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', '/v1/a/ver_cont/001o/4'),
@@ -1284,7 +1305,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         # first container server can reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&reverse=on',
             swob.HTTPOk, {}, json.dumps(list(reversed(old_versions[-2:]))))
         # but both objects are already gone
@@ -1297,21 +1318,21 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
 
         # second container server can't reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/3&reverse=on',
             swob.HTTPOk, {}, json.dumps(old_versions[4:]))
 
         # subsequent requests shouldn't reverse
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=&end_marker=001o/3',
             swob.HTTPOk, {}, json.dumps(old_versions[:2]))
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/1&end_marker=001o/3',
             swob.HTTPOk, {}, json.dumps(old_versions[2:3]))
         self.app.register(
-            'GET', '/v1/a/ver_cont?format=json&prefix=001o/&'
+            'GET', '/v1/a/ver_cont?prefix=001o/&'
             'marker=001o/2&end_marker=001o/3',
             swob.HTTPOk, {}, '[]')
         self.app.register(
@@ -1330,7 +1351,7 @@ class VersionedWritesOldContainersTestCase(VersionedWritesBaseTestCase):
                      'CONTENT_LENGTH': '0'})
         status, headers, body = self.call_vw(req)
         self.assertEqual(status, '204 No Content')
-        prefix_listing_prefix = '/v1/a/ver_cont?format=json&prefix=001o/&'
+        prefix_listing_prefix = '/v1/a/ver_cont?prefix=001o/&'
         self.assertEqual(self.app.calls, [
             ('GET', prefix_listing_prefix + 'marker=&reverse=on'),
             ('GET', '/v1/a/ver_cont/001o/4'),
