@@ -21,11 +21,11 @@ from contextlib import closing
 from gzip import GzipFile
 from shutil import rmtree
 from tempfile import mkdtemp
-from test.unit import FakeLogger
+from test.unit import debug_logger, mock_check_drive
 
 from eventlet import spawn, Timeout
 
-from swift.common import utils
+from swift.common import exceptions, utils
 from swift.container import updater as container_updater
 from swift.container.backend import ContainerBroker, DATADIR
 from swift.common.ring import RingData
@@ -55,33 +55,13 @@ class TestContainerUpdater(unittest.TestCase):
         os.mkdir(self.devices_dir)
         self.sda1 = os.path.join(self.devices_dir, 'sda1')
         os.mkdir(self.sda1)
+        self.logger = debug_logger('test')
 
     def tearDown(self):
         rmtree(os.path.dirname(self.testdir), ignore_errors=1)
 
-    def test_creation(self):
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'false',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '2',
-            'node_timeout': '5.5',
-        })
-        self.assertTrue(hasattr(cu, 'logger'))
-        self.assertTrue(cu.logger is not None)
-        self.assertEqual(cu.devices, self.devices_dir)
-        self.assertEqual(cu.interval, 1)
-        self.assertEqual(cu.concurrency, 2)
-        self.assertEqual(cu.node_timeout, 5.5)
-        self.assertTrue(cu.get_account_ring() is not None)
-
-    @mock.patch.object(container_updater, 'ismount')
-    @mock.patch.object(container_updater.ContainerUpdater, 'container_sweep')
-    def test_run_once_with_device_unmounted(self, mock_sweep, mock_ismount):
-
-        mock_ismount.return_value = False
-        cu = container_updater.ContainerUpdater({
+    def _get_container_updater(self, conf_updates=None):
+        conf = {
             'devices': self.devices_dir,
             'mount_check': 'false',
             'swift_dir': self.testdir,
@@ -89,7 +69,68 @@ class TestContainerUpdater(unittest.TestCase):
             'concurrency': '1',
             'node_timeout': '15',
             'account_suppression_time': 0
-        })
+        }
+        if conf_updates:
+            conf.update(conf_updates)
+        return container_updater.ContainerUpdater(conf, logger=self.logger)
+
+    def test_creation(self):
+        cu = self._get_container_updater({'concurrency': '2',
+                                          'node_timeout': '5.5'})
+        self.assertTrue(hasattr(cu, 'logger'))
+        self.assertTrue(cu.logger is not None)
+        self.assertEqual(cu.devices, self.devices_dir)
+        self.assertEqual(cu.interval, 1)
+        self.assertEqual(cu.concurrency, 2)
+        self.assertEqual(cu.node_timeout, 5.5)
+        self.assertEqual(cu.account_suppression_time, 0)
+        self.assertTrue(cu.get_account_ring() is not None)
+
+    def test_conf_params(self):
+        # defaults
+        daemon = container_updater.ContainerUpdater({})
+        self.assertEqual(daemon.devices, '/srv/node')
+        self.assertEqual(daemon.mount_check, True)
+        self.assertEqual(daemon.swift_dir, '/etc/swift')
+        self.assertEqual(daemon.interval, 300)
+        self.assertEqual(daemon.concurrency, 4)
+        self.assertEqual(daemon.max_containers_per_second, 50.0)
+
+        # non-defaults
+        conf = {
+            'devices': '/some/where/else',
+            'mount_check': 'huh?',
+            'swift_dir': '/not/here',
+            'interval': '600',
+            'concurrency': '2',
+            'containers_per_second': '10.5',
+        }
+        daemon = container_updater.ContainerUpdater(conf)
+        self.assertEqual(daemon.devices, '/some/where/else')
+        self.assertEqual(daemon.mount_check, False)
+        self.assertEqual(daemon.swift_dir, '/not/here')
+        self.assertEqual(daemon.interval, 600)
+        self.assertEqual(daemon.concurrency, 2)
+        self.assertEqual(daemon.max_containers_per_second, 10.5)
+
+        # check deprecated option
+        daemon = container_updater.ContainerUpdater({'slowdown': '0.04'})
+        self.assertEqual(daemon.max_containers_per_second, 20.0)
+
+        def check_bad(conf):
+            with self.assertRaises(ValueError):
+                container_updater.ContainerUpdater(conf)
+
+        check_bad({'interval': 'foo'})
+        check_bad({'interval': '300.0'})
+        check_bad({'concurrency': 'bar'})
+        check_bad({'concurrency': '1.0'})
+        check_bad({'slowdown': 'baz'})
+        check_bad({'containers_per_second': 'quux'})
+
+    @mock.patch.object(container_updater.ContainerUpdater, 'container_sweep')
+    def test_run_once_with_device_unmounted(self, mock_sweep):
+        cu = self._get_container_updater()
         containers_dir = os.path.join(self.sda1, DATADIR)
         os.mkdir(containers_dir)
         partition_dir = os.path.join(containers_dir, "a")
@@ -102,35 +143,61 @@ class TestContainerUpdater(unittest.TestCase):
         self.assertTrue(mock_sweep.called)
 
         mock_sweep.reset_mock()
-
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'true',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '1',
-            'node_timeout': '15',
-            'account_suppression_time': 0
-        })
-        cu.logger = FakeLogger()
-        cu.run_once()
-        log_lines = cu.logger.get_lines_for_level('warning')
-        self.assertTrue(len(log_lines) > 0)
-        msg = 'sda1 is not mounted'
+        cu = self._get_container_updater({'mount_check': 'true'})
+        with mock_check_drive():
+            cu.run_once()
+        log_lines = self.logger.get_lines_for_level('warning')
+        self.assertGreater(len(log_lines), 0)
+        msg = '%s is not mounted' % self.sda1
         self.assertEqual(log_lines[0], msg)
         # Ensure that the container_sweep did not run
         self.assertFalse(mock_sweep.called)
 
+    @mock.patch('swift.container.updater.dump_recon_cache')
+    def test_run_once_with_get_info_timeout(self, mock_dump_recon):
+        cu = self._get_container_updater()
+        containers_dir = os.path.join(self.sda1, DATADIR)
+        os.mkdir(containers_dir)
+        subdir = os.path.join(containers_dir, 'subdir')
+        os.mkdir(subdir)
+        db_file = os.path.join(subdir, 'hash.db')
+        cb = ContainerBroker(db_file, account='a', container='c')
+        cb.initialize(normalize_timestamp(1), 0)
+
+        timeout = exceptions.LockTimeout(10, db_file)
+        timeout.cancel()
+        with mock.patch('swift.container.updater.ContainerBroker.get_info',
+                        side_effect=timeout):
+            cu.run_once()
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertIn('Failed to get container info (Lock timeout: '
+                      '10 seconds: %s); skipping.' % db_file, log_lines)
+
+    @mock.patch('swift.container.updater.dump_recon_cache')
+    @mock.patch('swift.container.updater.ContainerUpdater.process_container',
+                side_effect=Exception('Boom!'))
+    def test_error_in_process(self, mock_process, mock_dump_recon):
+        cu = self._get_container_updater()
+        containers_dir = os.path.join(self.sda1, DATADIR)
+        os.mkdir(containers_dir)
+        subdir = os.path.join(containers_dir, 'subdir')
+        os.mkdir(subdir)
+        cb = ContainerBroker(os.path.join(subdir, 'hash.db'), account='a',
+                             container='c', pending_timeout=1)
+        cb.initialize(normalize_timestamp(1), 0)
+
+        cu.run_once()
+
+        log_lines = self.logger.get_lines_for_level('error')
+        self.assertTrue(log_lines)
+        self.assertIn('Error processing container ', log_lines[0])
+        self.assertIn('devices/sda1/containers/subdir/hash.db', log_lines[0])
+        self.assertIn('Boom!', log_lines[0])
+        self.assertFalse(log_lines[1:])
+        self.assertEqual(1, len(mock_dump_recon.mock_calls))
+
     def test_run_once(self):
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'false',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '1',
-            'node_timeout': '15',
-            'account_suppression_time': 0
-        })
+        cu = self._get_container_updater()
         cu.run_once()
         containers_dir = os.path.join(self.sda1, DATADIR)
         os.mkdir(containers_dir)
@@ -141,6 +208,7 @@ class TestContainerUpdater(unittest.TestCase):
         cb = ContainerBroker(os.path.join(subdir, 'hash.db'), account='a',
                              container='c')
         cb.initialize(normalize_timestamp(1), 0)
+        self.assertTrue(cb.is_root_container())
         cu.run_once()
         info = cb.get_info()
         self.assertEqual(info['object_count'], 0)
@@ -210,49 +278,24 @@ class TestContainerUpdater(unittest.TestCase):
     def test_listdir_with_exception(self, mock_listdir):
         e = OSError('permission_denied')
         mock_listdir.side_effect = e
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'false',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '1',
-            'node_timeout': '15',
-            'account_suppression_time': 0
-        })
-        cu.logger = FakeLogger()
+        cu = self._get_container_updater()
         paths = cu.get_paths()
         self.assertEqual(paths, [])
-        log_lines = cu.logger.get_lines_for_level('error')
+        log_lines = self.logger.get_lines_for_level('error')
         msg = ('ERROR:  Failed to get paths to drive partitions: '
                'permission_denied')
         self.assertEqual(log_lines[0], msg)
 
     @mock.patch('os.listdir', return_value=['foo', 'bar'])
     def test_listdir_without_exception(self, mock_listdir):
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'false',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '1',
-            'node_timeout': '15',
-            'account_suppression_time': 0
-        })
-        cu.logger = FakeLogger()
+        cu = self._get_container_updater()
         path = cu._listdir('foo/bar/')
         self.assertEqual(path, ['foo', 'bar'])
-        log_lines = cu.logger.get_lines_for_level('error')
+        log_lines = self.logger.get_lines_for_level('error')
         self.assertEqual(len(log_lines), 0)
 
     def test_unicode(self):
-        cu = container_updater.ContainerUpdater({
-            'devices': self.devices_dir,
-            'mount_check': 'false',
-            'swift_dir': self.testdir,
-            'interval': '1',
-            'concurrency': '1',
-            'node_timeout': '15',
-        })
+        cu = self._get_container_updater()
         containers_dir = os.path.join(self.sda1, DATADIR)
         os.mkdir(containers_dir)
         subdir = os.path.join(containers_dir, 'subdir')
@@ -302,5 +345,94 @@ class TestContainerUpdater(unittest.TestCase):
         self.assertEqual(info['reported_object_count'], 1)
         self.assertEqual(info['reported_bytes_used'], 3)
 
+    def test_shard_container(self):
+        cu = self._get_container_updater()
+        cu.run_once()
+        containers_dir = os.path.join(self.sda1, DATADIR)
+        os.mkdir(containers_dir)
+        cu.run_once()
+        self.assertTrue(os.path.exists(containers_dir))
+        subdir = os.path.join(containers_dir, 'subdir')
+        os.mkdir(subdir)
+        cb = ContainerBroker(os.path.join(subdir, 'hash.db'),
+                             account='.shards_a', container='c')
+        cb.initialize(normalize_timestamp(1), 0)
+        cb.set_sharding_sysmeta('Root', 'a/c')
+        self.assertFalse(cb.is_root_container())
+        cu.run_once()
+        info = cb.get_info()
+        self.assertEqual(info['object_count'], 0)
+        self.assertEqual(info['bytes_used'], 0)
+        self.assertEqual(info['reported_put_timestamp'], '0')
+        self.assertEqual(info['reported_delete_timestamp'], '0')
+        self.assertEqual(info['reported_object_count'], 0)
+        self.assertEqual(info['reported_bytes_used'], 0)
+
+        cb.put_object('o', normalize_timestamp(2), 3, 'text/plain',
+                      '68b329da9893e34099c7d8ad5cb9c940')
+        # Fake us having already reported *bad* stats under swift 2.18.0
+        cb.reported('0', '0', 1, 3)
+
+        # Should fail with a bunch of connection-refused
+        cu.run_once()
+        info = cb.get_info()
+        self.assertEqual(info['object_count'], 1)
+        self.assertEqual(info['bytes_used'], 3)
+        self.assertEqual(info['reported_put_timestamp'], '0')
+        self.assertEqual(info['reported_delete_timestamp'], '0')
+        self.assertEqual(info['reported_object_count'], 1)
+        self.assertEqual(info['reported_bytes_used'], 3)
+
+        def accept(sock, addr, return_code):
+            try:
+                with Timeout(3):
+                    inc = sock.makefile('rb')
+                    out = sock.makefile('wb')
+                    out.write('HTTP/1.1 %d OK\r\nContent-Length: 0\r\n\r\n' %
+                              return_code)
+                    out.flush()
+                    self.assertEqual(inc.readline(),
+                                     'PUT /sda1/2/.shards_a/c HTTP/1.1\r\n')
+                    headers = {}
+                    line = inc.readline()
+                    while line and line != '\r\n':
+                        headers[line.split(':')[0].lower()] = \
+                            line.split(':')[1].strip()
+                        line = inc.readline()
+                    self.assertTrue('x-put-timestamp' in headers)
+                    self.assertTrue('x-delete-timestamp' in headers)
+                    self.assertTrue('x-object-count' in headers)
+                    self.assertTrue('x-bytes-used' in headers)
+            except BaseException as err:
+                import traceback
+                traceback.print_exc()
+                return err
+            return None
+        bindsock = listen_zero()
+
+        def spawn_accepts():
+            events = []
+            for _junk in range(2):
+                sock, addr = bindsock.accept()
+                events.append(spawn(accept, sock, addr, 201))
+            return events
+
+        spawned = spawn(spawn_accepts)
+        for dev in cu.get_account_ring().devs:
+            if dev is not None:
+                dev['port'] = bindsock.getsockname()[1]
+        cu.run_once()
+        for event in spawned.wait():
+            err = event.wait()
+            if err:
+                raise err
+        info = cb.get_info()
+        self.assertEqual(info['object_count'], 1)
+        self.assertEqual(info['bytes_used'], 3)
+        self.assertEqual(info['reported_put_timestamp'], '0000000001.00000')
+        self.assertEqual(info['reported_delete_timestamp'], '0')
+        self.assertEqual(info['reported_object_count'], 0)
+        self.assertEqual(info['reported_bytes_used'], 0)
+
 if __name__ == '__main__':
     unittest.main()
