@@ -12,20 +12,23 @@
 # implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import json
 
-from test import unit
 import unittest
 import mock
 import os
+import sys
+import signal
 import time
 import string
+import xattr
 from shutil import rmtree
 from hashlib import md5
 from tempfile import mkdtemp
 import textwrap
 from os.path import dirname, basename
-from test.unit import (FakeLogger, patch_policies, make_timestamp_iter,
-                       DEFAULT_TEST_EC_TYPE)
+from test.unit import (debug_logger, patch_policies, make_timestamp_iter,
+                       DEFAULT_TEST_EC_TYPE, skip_if_no_xattrs)
 from swift.obj import auditor, replicator
 from swift.obj.diskfile import (
     DiskFile, write_metadata, invalidate_hash, get_data_dir,
@@ -62,10 +65,11 @@ def works_only_once(callable_thing, exception):
 class TestAuditor(unittest.TestCase):
 
     def setUp(self):
+        skip_if_no_xattrs()
         self.testdir = os.path.join(mkdtemp(), 'tmp_test_object_auditor')
         self.devices = os.path.join(self.testdir, 'node')
         self.rcache = os.path.join(self.testdir, 'object.recon')
-        self.logger = FakeLogger()
+        self.logger = debug_logger()
         rmtree(self.testdir, ignore_errors=1)
         mkdirs(os.path.join(self.devices, 'sda'))
         os.mkdir(os.path.join(self.devices, 'sdb'))
@@ -117,7 +121,6 @@ class TestAuditor(unittest.TestCase):
 
     def tearDown(self):
         rmtree(os.path.dirname(self.testdir), ignore_errors=1)
-        unit.xattr_data = {}
 
     def test_worker_conf_parms(self):
         def check_common_defaults():
@@ -188,6 +191,74 @@ class TestAuditor(unittest.TestCase):
         run_tests(self.disk_file_p1)
         run_tests(self.disk_file_ec)
 
+    def test_object_audit_adds_metadata_checksums(self):
+        disk_file = self.df_mgr.get_diskfile('sda', '0', 'a', 'c', 'o-md',
+                                             policy=POLICIES.legacy)
+
+        # simulate a PUT
+        now = time.time()
+        data = b'boots and cats and ' * 1024
+        hasher = md5()
+        with disk_file.create() as writer:
+            writer.write(data)
+            hasher.update(data)
+            etag = hasher.hexdigest()
+            metadata = {
+                'ETag': etag,
+                'X-Timestamp': str(normalize_timestamp(now)),
+                'Content-Length': len(data),
+                'Content-Type': 'the old type',
+            }
+            writer.put(metadata)
+            writer.commit(Timestamp(now))
+
+        # simulate a subsequent POST
+        post_metadata = metadata.copy()
+        post_metadata['Content-Type'] = 'the new type'
+        post_metadata['X-Object-Meta-Biff'] = 'buff'
+        post_metadata['X-Timestamp'] = str(normalize_timestamp(now + 1))
+        disk_file.write_metadata(post_metadata)
+
+        file_paths = [os.path.join(disk_file._datadir, fname)
+                      for fname in os.listdir(disk_file._datadir)
+                      if fname not in ('.', '..')]
+        file_paths.sort()
+
+        # sanity check: make sure we have a .data and a .meta file
+        self.assertEqual(len(file_paths), 2)
+        self.assertTrue(file_paths[0].endswith(".data"))
+        self.assertTrue(file_paths[1].endswith(".meta"))
+
+        # Go remove the xattr "user.swift.metadata_checksum" as if this
+        # object were written before Swift supported metadata checksums.
+        for file_path in file_paths:
+            xattr.removexattr(file_path, "user.swift.metadata_checksum")
+
+        # Run the auditor...
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        auditor_worker.object_audit(
+            AuditLocation(disk_file._datadir, 'sda', '0',
+                          policy=disk_file.policy))
+        self.assertEqual(auditor_worker.quarantines, 0)  # sanity
+
+        # ...and the checksums are back
+        for file_path in file_paths:
+            metadata = xattr.getxattr(file_path, "user.swift.metadata")
+            i = 1
+            while True:
+                try:
+                    metadata += xattr.getxattr(
+                        file_path, "user.swift.metadata%d" % i)
+                    i += 1
+                except (IOError, OSError):
+                    break
+
+            checksum = xattr.getxattr(
+                file_path, "user.swift.metadata_checksum")
+
+            self.assertEqual(checksum, md5(metadata).hexdigest())
+
     def test_object_audit_diff_data(self):
         auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
@@ -245,7 +316,8 @@ class TestAuditor(unittest.TestCase):
                 writer.put(metadata)
                 writer.commit(Timestamp(timestamp))
 
-            auditor_worker = auditor.AuditorWorker(self.conf, FakeLogger(),
+            self.logger.clear()
+            auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                    self.rcache, self.devices)
             self.assertEqual(0, auditor_worker.quarantines)  # sanity check
             auditor_worker.object_audit(
@@ -599,23 +671,93 @@ class TestAuditor(unittest.TestCase):
         self.assertEqual(auditor_worker.stats_buckets['OVER'], 2)
 
     def test_object_run_logging(self):
-        logger = FakeLogger()
-        auditor_worker = auditor.AuditorWorker(self.conf, logger,
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
         auditor_worker.audit_all_objects(device_dirs=['sda'])
-        log_lines = logger.get_lines_for_level('info')
-        self.assertTrue(len(log_lines) > 0)
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertGreater(len(log_lines), 0)
         self.assertTrue(log_lines[0].index('ALL - parallel, sda'))
 
-        logger = FakeLogger()
-        auditor_worker = auditor.AuditorWorker(self.conf, logger,
+        self.logger.clear()
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices,
                                                zero_byte_only_at_fps=50)
         auditor_worker.audit_all_objects(device_dirs=['sda'])
-        log_lines = logger.get_lines_for_level('info')
-        self.assertTrue(len(log_lines) > 0)
+        log_lines = self.logger.get_lines_for_level('info')
+        self.assertGreater(len(log_lines), 0)
         self.assertTrue(log_lines[0].index('ZBF - sda'))
 
+    def test_object_run_recon_cache(self):
+        ts = Timestamp(time.time())
+        data = 'test_data'
+
+        with self.disk_file.create() as writer:
+            writer.write(data)
+            metadata = {
+                'ETag': md5(data).hexdigest(),
+                'X-Timestamp': ts.normal,
+                'Content-Length': str(os.fstat(writer._fd).st_size),
+            }
+            writer.put(metadata)
+            writer.commit(ts)
+
+        # all devices
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        auditor_worker.audit_all_objects()
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        expected = {'object_auditor_stats_ALL':
+                    {'passes': 1, 'errors': 0, 'audit_time': mock.ANY,
+                     'start_time': mock.ANY, 'quarantined': 0,
+                     'bytes_processed': 9}}
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        self.assertEqual(expected, actual_rcache)
+
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices,
+                                               zero_byte_only_at_fps=50)
+        auditor_worker.audit_all_objects()
+        self.assertEqual(expected, actual_rcache)
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        expected.update({
+            'object_auditor_stats_ZBF':
+            {'passes': 1, 'errors': 0, 'audit_time': mock.ANY,
+             'start_time': mock.ANY, 'quarantined': 0,
+             'bytes_processed': 0}})
+        self.assertEqual(expected, actual_rcache)
+
+        # specific devices
+        os.unlink(self.rcache)
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices)
+        auditor_worker.audit_all_objects(device_dirs=['sda'])
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        expected = {'object_auditor_stats_ALL':
+                    {'sda': {'passes': 1, 'errors': 0, 'audit_time': mock.ANY,
+                             'start_time': mock.ANY, 'quarantined': 0,
+                             'bytes_processed': 9}}}
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        self.assertEqual(expected, actual_rcache)
+
+        auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
+                                               self.rcache, self.devices,
+                                               zero_byte_only_at_fps=50)
+        auditor_worker.audit_all_objects(device_dirs=['sda'])
+        self.assertEqual(expected, actual_rcache)
+        with open(self.rcache) as fd:
+            actual_rcache = json.load(fd)
+        expected.update({
+            'object_auditor_stats_ZBF':
+            {'sda': {'passes': 1, 'errors': 0, 'audit_time': mock.ANY,
+                     'start_time': mock.ANY, 'quarantined': 0,
+                     'bytes_processed': 0}}})
+        self.assertEqual(expected, actual_rcache)
+
     def test_object_run_once_no_sda(self):
         auditor_worker = auditor.AuditorWorker(self.conf, self.logger,
                                                self.rcache, self.devices)
@@ -710,13 +852,13 @@ class TestAuditor(unittest.TestCase):
         self.auditor.run_audit(**kwargs)
         self.assertFalse(os.path.isdir(quarantine_path))
         del(kwargs['zero_byte_fps'])
-        clear_auditor_status(self.devices)
+        clear_auditor_status(self.devices, 'objects')
         self.auditor.run_audit(**kwargs)
         self.assertTrue(os.path.isdir(quarantine_path))
 
     def setup_bad_zero_byte(self, timestamp=None):
         if timestamp is None:
-            timestamp = Timestamp(time.time())
+            timestamp = Timestamp.now()
         self.auditor = auditor.ObjectAuditor(self.conf)
         self.auditor.log_time = 0
         etag = md5()
@@ -834,7 +976,7 @@ class TestAuditor(unittest.TestCase):
             self.disk_file.delete(ts_tomb)
             # this get_hashes call will truncate the invalid hashes entry
             self.disk_file.manager.get_hashes(
-                self.devices + '/sda', '0', [], self.disk_file.policy)
+                'sda', '0', [], self.disk_file.policy)
         suffix = basename(dirname(self.disk_file._datadir))
         part_dir = dirname(dirname(self.disk_file._datadir))
         # sanity checks...
@@ -912,8 +1054,7 @@ class TestAuditor(unittest.TestCase):
 
     def _test_expired_object_is_ignored(self, zero_byte_fps):
         # verify that an expired object does not get mistaken for a tombstone
-        audit = auditor.ObjectAuditor(self.conf)
-        audit.logger = FakeLogger()
+        audit = auditor.ObjectAuditor(self.conf, logger=self.logger)
         audit.log_time = 0
         now = time.time()
         write_diskfile(self.disk_file, Timestamp(now - 20),
@@ -939,7 +1080,7 @@ class TestAuditor(unittest.TestCase):
 
         # this get_hashes call will truncate the invalid hashes entry
         self.disk_file.manager.get_hashes(
-            self.devices + '/sda', '0', [], self.disk_file.policy)
+            'sda', '0', [], self.disk_file.policy)
         with open(hash_invalid, 'rb') as fp:
             self.assertEqual('', fp.read().strip('\n'))  # sanity check
 
@@ -1113,6 +1254,12 @@ class TestAuditor(unittest.TestCase):
                 self.wait_called += 1
                 return (self.wait_called, 0)
 
+            def mock_signal(self, sig, action):
+                pass
+
+            def mock_exit(self):
+                pass
+
         for i in string.ascii_letters[2:26]:
             mkdirs(os.path.join(self.devices, 'sd%s' % i))
 
@@ -1128,8 +1275,12 @@ class TestAuditor(unittest.TestCase):
         my_auditor.run_audit = mocker.mock_run
         was_fork = os.fork
         was_wait = os.wait
+        was_signal = signal.signal
+        was_exit = sys.exit
         os.fork = mocker.mock_fork
         os.wait = mocker.mock_wait
+        signal.signal = mocker.mock_signal
+        sys.exit = mocker.mock_exit
         try:
             my_auditor._sleep = mocker.mock_sleep_stop
             my_auditor.run_once(zero_byte_fps=50)
@@ -1141,6 +1292,12 @@ class TestAuditor(unittest.TestCase):
                 'ERROR auditing: %s', loop_error)
             my_auditor.audit_loop = real_audit_loop
 
+            # sleep between ZBF scanner forks
+            self.assertRaises(StopForever, my_auditor.fork_child, True, True)
+
+            mocker.fork_called = 0
+            signal.signal = was_signal
+            sys.exit = was_exit
             self.assertRaises(StopForever,
                               my_auditor.run_forever, zero_byte_fps=50)
             self.assertEqual(mocker.check_kwargs['zero_byte_fps'], 50)
@@ -1167,11 +1324,11 @@ class TestAuditor(unittest.TestCase):
 
             mocker.fork_called = 0
             self.assertRaises(StopForever, my_auditor.run_forever)
-            # Fork is called 2 times since the zbf process is forked just
-            # once before self._sleep() is called and StopForever is raised
-            # Also wait is called just once before StopForever is raised
-            self.assertEqual(mocker.fork_called, 2)
-            self.assertEqual(mocker.wait_called, 1)
+            # Fork or Wait are called greate than or equal to 2 times in the
+            # main process. 2 times if zbf run once and 3 times if zbf run
+            # again
+            self.assertGreaterEqual(mocker.fork_called, 2)
+            self.assertGreaterEqual(mocker.wait_called, 2)
 
             my_auditor._sleep = mocker.mock_sleep_continue
             my_auditor.audit_loop = works_only_once(my_auditor.audit_loop,
@@ -1181,13 +1338,13 @@ class TestAuditor(unittest.TestCase):
             mocker.fork_called = 0
             mocker.wait_called = 0
             self.assertRaises(LetMeOut, my_auditor.run_forever)
-            # Fork is called no. of devices + (no. of devices)/2 + 1 times
-            # since zbf process is forked (no.of devices)/2 + 1 times
+            # Fork or Wait are called greater than or equal to
+            # no. of devices + (no. of devices)/2 + 1 times in main process
             no_devices = len(os.listdir(self.devices))
-            self.assertEqual(mocker.fork_called, no_devices + no_devices / 2
-                             + 1)
-            self.assertEqual(mocker.wait_called, no_devices + no_devices / 2
-                             + 1)
+            self.assertGreaterEqual(mocker.fork_called, no_devices +
+                                    no_devices / 2 + 1)
+            self.assertGreaterEqual(mocker.wait_called, no_devices +
+                                    no_devices / 2 + 1)
 
         finally:
             os.fork = was_fork
@@ -1237,6 +1394,53 @@ class TestAuditor(unittest.TestCase):
 
         self.assertEqual(sorted(forked_pids), [2, 1001])
 
+    def test_run_audit_once_zbfps(self):
+        my_auditor = auditor.ObjectAuditor(dict(devices=self.devices,
+                                                mount_check='false',
+                                                zero_byte_files_per_second=89,
+                                                concurrency=1,
+                                                recon_cache_path=self.testdir))
+
+        with mock.patch.object(my_auditor, '_sleep', lambda *a: None):
+            my_auditor.run_once(zero_byte_fps=50)
+
+        with open(self.rcache) as fd:
+            # there's no objects to audit so expect no stats; this assertion
+            # may change if https://bugs.launchpad.net/swift/+bug/1704858 is
+            # fixed
+            self.assertEqual({}, json.load(fd))
+
+        # check recon cache stays clean after a second run
+        with mock.patch.object(my_auditor, '_sleep', lambda *a: None):
+            my_auditor.run_once(zero_byte_fps=50)
+
+        with open(self.rcache) as fd:
+            self.assertEqual({}, json.load(fd))
+
+        ts = Timestamp(time.time())
+        with self.disk_file.create() as writer:
+            metadata = {
+                'ETag': md5('').hexdigest(),
+                'X-Timestamp': ts.normal,
+                'Content-Length': str(os.fstat(writer._fd).st_size),
+            }
+            writer.put(metadata)
+            writer.commit(ts)
+
+        # check recon cache stays clean after a second run
+        with mock.patch.object(my_auditor, '_sleep', lambda *a: None):
+            my_auditor.run_once(zero_byte_fps=50)
+        with open(self.rcache) as fd:
+            self.assertEqual({
+                'object_auditor_stats_ZBF': {
+                    'audit_time': 0,
+                    'bytes_processed': 0,
+                    'errors': 0,
+                    'passes': 1,
+                    'quarantined': 0,
+                    'start_time': mock.ANY}},
+                json.load(fd))
+
     def test_run_parallel_audit_once(self):
         my_auditor = auditor.ObjectAuditor(
             dict(devices=self.devices, mount_check='false',
