@@ -14,6 +14,7 @@
 # limitations under the License.
 
 import itertools
+import json
 from collections import defaultdict
 import unittest
 import mock
@@ -23,11 +24,14 @@ from swift.proxy.controllers.base import headers_to_container_info, \
     Controller, GetOrHeadHandler, bytes_to_skip
 from swift.common.swob import Request, HTTPException, RESPONSE_REASONS
 from swift.common import exceptions
-from swift.common.utils import split_path
+from swift.common.utils import split_path, ShardRange, Timestamp
 from swift.common.header_key_dict import HeaderKeyDict
 from swift.common.http import is_success
-from swift.common.storage_policy import StoragePolicy
-from test.unit import fake_http_connect, FakeRing, FakeMemcache
+from swift.common.storage_policy import StoragePolicy, StoragePolicyCollection
+from test.unit import (
+    fake_http_connect, FakeRing, FakeMemcache, PatchPolicies, FakeLogger,
+    make_timestamp_iter,
+    mocked_http_conn)
 from swift.proxy import server as proxy_server
 from swift.common.request_helpers import (
     get_sys_meta_prefix, get_object_transient_sysmeta
@@ -172,7 +176,8 @@ class TestFuncs(unittest.TestCase):
     def setUp(self):
         self.app = proxy_server.Application(None, FakeMemcache(),
                                             account_ring=FakeRing(),
-                                            container_ring=FakeRing())
+                                            container_ring=FakeRing(),
+                                            logger=FakeLogger())
 
     def test_get_info_zero_recheck(self):
         mock_cache = mock.Mock()
@@ -209,7 +214,8 @@ class TestFuncs(unittest.TestCase):
         self.assertEqual(info_c['object_count'], 1000)
         # Make sure the env cache is set
         exp_cached_info_c = {
-            k: str(v) if k in ('bytes', 'object_count') else v
+            k: str(v) if k in (
+                'bytes', 'object_count', 'storage_policy') else v
             for k, v in info_c.items()}
         self.assertEqual(env['swift.infocache'].get('account/a'),
                          exp_cached_info_a)
@@ -340,7 +346,7 @@ class TestFuncs(unittest.TestCase):
         req = Request.blank("/v1/AUTH_account/cont",
                             environ={'swift.cache': FakeCache({})})
         resp = get_container_info(req.environ, FakeApp())
-        self.assertEqual(resp['storage_policy'], '0')
+        self.assertEqual(resp['storage_policy'], 0)
         self.assertEqual(resp['bytes'], 6666)
         self.assertEqual(resp['object_count'], 1000)
 
@@ -365,7 +371,7 @@ class TestFuncs(unittest.TestCase):
         req = Request.blank("/v1/account/cont",
                             environ={'swift.cache': FakeCache(cache_stub)})
         resp = get_container_info(req.environ, FakeApp())
-        self.assertEqual(resp['storage_policy'], '0')
+        self.assertEqual(resp['storage_policy'], 0)
         self.assertEqual(resp['bytes'], 3333)
         self.assertEqual(resp['object_count'], 10)
         self.assertEqual(resp['status'], 404)
@@ -614,6 +620,27 @@ class TestFuncs(unittest.TestCase):
             resp,
             headers_to_account_info(headers.items(), 200))
 
+    def test_headers_to_account_info_storage_policies(self):
+        headers = {
+            'x-account-storage-policy-zero-object-count': '13',
+            'x-account-storage-policy-zero-container-count': '120',
+            'x-account-storage-policy-zero-bytes-used': '1002',
+            'x-account-storage-policy-one-object-count': '10',
+            'x-account-storage-policy-one-container-count': '20',
+        }
+        spc = StoragePolicyCollection([StoragePolicy(0, 'zero', True),
+                                       StoragePolicy(1, 'one', False)])
+        with PatchPolicies(spc):
+            resp = headers_to_account_info(headers.items(), 200)
+        self.assertEqual(resp['storage_policies'], {
+            0: {'object_count': 13,
+                'container_count': 120,
+                'bytes': 1002},
+            1: {'object_count': 10,
+                'container_count': 20,
+                'bytes': 0},
+        })
+
     def test_headers_to_object_info_missing(self):
         resp = headers_to_object_info({}, 404)
         self.assertEqual(resp['status'], 404)
@@ -866,6 +893,8 @@ class TestFuncs(unittest.TestCase):
                     return ''
 
             def getheader(self, header):
+                # content-length for the whole object is generated dynamically
+                # by summing non-None chunks initialized as source1
                 if header.lower() == "content-length":
                     return str(sum(len(c) for c in self.chunks
                                    if c is not None))
@@ -875,18 +904,30 @@ class TestFuncs(unittest.TestCase):
 
         node = {'ip': '1.2.3.4', 'port': 6200, 'device': 'sda'}
 
-        source1 = TestSource(['abcd', '1234', 'abc', None])
-        source2 = TestSource(['efgh5678'])
+        source1 = TestSource(['abcd', '1234', None,
+                              'efgh', '5678', 'lots', 'more', 'data'])
+        # incomplete reads of client_chunk_size will be re-fetched
+        source2 = TestSource(['efgh', '5678', 'lots', None])
+        source3 = TestSource(['lots', 'more', 'data'])
         req = Request.blank('/v1/a/c/o')
         handler = GetOrHeadHandler(
             self.app, req, 'Object', None, None, None, {},
             client_chunk_size=8)
 
+        range_headers = []
+        sources = [(source2, node), (source3, node)]
+
+        def mock_get_source_and_node():
+            range_headers.append(handler.backend_headers['Range'])
+            return sources.pop(0)
+
         app_iter = handler._make_app_iter(req, node, source1)
         with mock.patch.object(handler, '_get_source_and_node',
-                               lambda: (source2, node)):
+                               side_effect=mock_get_source_and_node):
             client_chunks = list(app_iter)
-        self.assertEqual(client_chunks, ['abcd1234', 'efgh5678'])
+        self.assertEqual(range_headers, ['bytes=8-27', 'bytes=16-27'])
+        self.assertEqual(client_chunks, [
+            'abcd1234', 'efgh5678', 'lotsmore', 'data'])
 
     def test_client_chunk_size_resuming_chunked(self):
 
@@ -994,3 +1035,146 @@ class TestFuncs(unittest.TestCase):
         # prime numbers
         self.assertEqual(bytes_to_skip(11, 7), 4)
         self.assertEqual(bytes_to_skip(97, 7873823), 55)
+
+    def test_get_shard_ranges_for_container_get(self):
+        ts_iter = make_timestamp_iter()
+        shard_ranges = [dict(ShardRange(
+            '.sharded_a/sr%d' % i, next(ts_iter), '%d_lower' % i,
+            '%d_upper' % i, object_count=i, bytes_used=1024 * i,
+            meta_timestamp=next(ts_iter)))
+            for i in range(3)]
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c', method='GET')
+        resp_headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(
+            200, 200, body_iter=iter(['', json.dumps(shard_ranges)]),
+            headers=resp_headers
+        ) as fake_conn:
+            actual = base._get_shard_ranges(req, 'a', 'c')
+
+        # account info
+        captured = fake_conn.requests
+        self.assertEqual('HEAD', captured[0]['method'])
+        self.assertEqual('a', captured[0]['path'][7:])
+        # container GET
+        self.assertEqual('GET', captured[1]['method'])
+        self.assertEqual('a/c', captured[1]['path'][7:])
+        self.assertEqual('format=json', captured[1]['qs'])
+        self.assertEqual(
+            'shard', captured[1]['headers'].get('X-Backend-Record-Type'))
+        self.assertEqual(shard_ranges, [dict(pr) for pr in actual])
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+
+    def test_get_shard_ranges_for_object_put(self):
+        ts_iter = make_timestamp_iter()
+        shard_ranges = [dict(ShardRange(
+            '.sharded_a/sr%d' % i, next(ts_iter), '%d_lower' % i,
+            '%d_upper' % i, object_count=i, bytes_used=1024 * i,
+            meta_timestamp=next(ts_iter)))
+            for i in range(3)]
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        resp_headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(
+            200, 200, body_iter=iter(['', json.dumps(shard_ranges[1:2])]),
+            headers=resp_headers
+        ) as fake_conn:
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+
+        # account info
+        captured = fake_conn.requests
+        self.assertEqual('HEAD', captured[0]['method'])
+        self.assertEqual('a', captured[0]['path'][7:])
+        # container GET
+        self.assertEqual('GET', captured[1]['method'])
+        self.assertEqual('a/c', captured[1]['path'][7:])
+        params = sorted(captured[1]['qs'].split('&'))
+        self.assertEqual(
+            ['format=json', 'includes=1_test'], params)
+        self.assertEqual(
+            'shard', captured[1]['headers'].get('X-Backend-Record-Type'))
+        self.assertEqual(shard_ranges[1:2], [dict(pr) for pr in actual])
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+
+    def _check_get_shard_ranges_bad_data(self, body):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        # empty response
+        headers = {'X-Backend-Record-Type': 'shard'}
+        with mocked_http_conn(200, 200, body_iter=iter(['', body]),
+                              headers=headers):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        lines = self.app.logger.get_lines_for_level('error')
+        return lines
+
+    def test_get_shard_ranges_empty_body(self):
+        error_lines = self._check_get_shard_ranges_bad_data('')
+        self.assertIn('Problem with listing response', error_lines[0])
+        self.assertIn('No JSON', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_not_a_list(self):
+        error_lines = self._check_get_shard_ranges_bad_data(json.dumps({}))
+        self.assertIn('Problem with listing response', error_lines[0])
+        self.assertIn('not a list', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_key_missing(self):
+        error_lines = self._check_get_shard_ranges_bad_data(json.dumps([{}]))
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('KeyError', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_invalid_shard_range(self):
+        sr = ShardRange('a/c', Timestamp.now())
+        bad_sr_data = dict(sr, name='bad_name')
+        error_lines = self._check_get_shard_ranges_bad_data(
+            json.dumps([bad_sr_data]))
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('ValueError', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_missing_record_type(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        sr = ShardRange('a/c', Timestamp.now())
+        body = json.dumps([dict(sr)])
+        with mocked_http_conn(
+                200, 200, body_iter=iter(['', body])):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        error_lines = self.app.logger.get_lines_for_level('error')
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('unexpected record type', error_lines[0])
+        self.assertIn('/a/c', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_wrong_record_type(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        sr = ShardRange('a/c', Timestamp.now())
+        body = json.dumps([dict(sr)])
+        headers = {'X-Backend-Record-Type': 'object'}
+        with mocked_http_conn(
+                200, 200, body_iter=iter(['', body]),
+                headers=headers):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        error_lines = self.app.logger.get_lines_for_level('error')
+        self.assertIn('Failed to get shard ranges', error_lines[0])
+        self.assertIn('unexpected record type', error_lines[0])
+        self.assertIn('/a/c', error_lines[0])
+        self.assertFalse(error_lines[1:])
+
+    def test_get_shard_ranges_request_failed(self):
+        base = Controller(self.app)
+        req = Request.blank('/v1/a/c/o', method='PUT')
+        with mocked_http_conn(200, 404, 404, 404):
+            actual = base._get_shard_ranges(req, 'a', 'c', '1_test')
+        self.assertIsNone(actual)
+        self.assertFalse(self.app.logger.get_lines_for_level('error'))
+        warning_lines = self.app.logger.get_lines_for_level('warning')
+        self.assertIn('Failed to get container listing', warning_lines[0])
+        self.assertIn('/a/c', warning_lines[0])
+        self.assertFalse(warning_lines[1:])
